//****************************************************
//*
//*  Dummy UserAnalysis Script to be used with gui.C 
//*  Functions in this scripts are called by GUIMainMacro.C  
//*  
//*  In this script, global functions, UserInitialize() nad UserAnalysis()
//*  Must be defined.  UserInitialize() is called at job initialization 
//*  to initialize histograms, etc.  It is also called when reset historgam
//*  menu is selected from JSF Control Panel.  UserAnalysis() is called 
//*  during the event loop after executing Process functions of JSF modules
//*  and display event daia but before display histogram.  See GetEvent()
//*  function in GUIMainMacro.C, where UserAnalysis() is called.
//*
//*  UserSetOptions() and UserTerminate() may be defined in this file.
//*  UserSetOptions() are called before declaration of JSF modules.
//*  It can be used to set parameters optional for user analsis.
//*  UserTerminate() is called during the JSF termination process.
//*
//*  When runmode=4, UserModuleDefine() must be defined in this file.
//*  It is used to define JSF modules specific to user analysis.  
//*  
//*$Id: UserAnalysis.C,v 1.5 2007/11/12 13:06:56 fujiik Exp $
//*
//****************************************************


//_________________________________________________________
void UserInitialize()
{
//----------------------------------
// set parameters
//----------------------------------
#if 0
   //   S4TPCPadHit::SetSpatialResolution("experimental"); // default is "flat".

   // Set IP constraint ....
   // S4TPCTrack::SetIPConstraint(kTRUE);

   // Artificial bunch tagging
   Double_t t0 = 0.;
   S4TPCTrack::SetT0pedestal(t0); // [ns]
#endif

   S4CALHitBuf::SetHitMakingScheme(1); // idealistic (smeared 1 hit/1 exact hit)

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  S4VTXTrackMaker *vtxtrackmaker = (S4VTXTrackMaker *)gJSF->FindModule("S4VTXTrackMaker");
  S4ITTrackMaker  *ittrackmaker  = (S4ITTrackMaker *)gJSF->FindModule("S4ITTrackMaker");
  S4TPCTrackMaker *tpctrackmaker = (S4TPCTrackMaker *)gJSF->FindModule("S4TPCTrackMaker");
  S4HYBTrackMaker *htrackmaker   = (S4HYBTrackMaker *)gJSF->FindModule("S4HYBTrackMaker");
  htrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  htrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);
}

//_________________________________________________________
void UserAnalysis()
{
   S4PFOMakerBuf *pfombp = (S4PFOMakerBuf *)
                              gJSF->FindModule("S4PFOMaker")->EventBuf();
//   pfombp->DebugPrint();  // Remove comment to print information of PFOs.

#if 1
   DrawPFOs(); // draw option ("p","t") = (w/ prehits, w/ tracks)
#else
   DrawPFOs("p"); // draw option ("p","t") = (w/ prehits, w/ tracks)
#endif
}

static TCanvas     *cvp    = 0;
static TTUBE       *imudp  = 0;
static TTUBE       *omudp  = 0;
static TTUBE       *itubep = 0;
static TTUBE       *otubep = 0;
static TRotMatrix  *orotp = 0;
static TNode       *mudtrapnode = 0;
static TNode       *omudnodep = 0;
static TNode       *imudnodep = 0;
static TNode       *onodep = 0;
static TNode       *inodep = 0;
static Int_t        color  = 0;

//_________________________________________________________
void DrawPFOs(const Char_t *opt = "")
{
   TDirectory *last = gDirectory;
//----------------------------------
// initialize event display
//----------------------------------
#if 1
   if (!cvp) {
     cvp = new TCanvas("OED", "Event Display", 400, 10, 610, 610);
   } else {
      cvp->cd();
      cvp->Clear();
   }
   if( gROOT->GetVersionInt() < 51504 ) {
     TView   *vwp = new TView(1);
   }
   else {
     TView   *vwp = TView::CreateView(1,0,0);
     vwp->SetSystem(1);
   }
   vwp->SetRange(-360.,-360.,-360.,+360.,+360.,+360.);
   Int_t ierr ;
   //vwp->SetView(10.,80.,80.,ierr);
   vwp->SetView(0,0,90,ierr);
   vwp->UnZoom();
#else
   cvp = (TCanvas *)gROOT->FindObject("EventDisplay");
   if (!cvp) return;
   else      cvp->cd();
#endif

   if (!omudp)  omudp  = new TTUBE("MUDO","MUDO","void",765.,765.,800.);
   if (!imudp)  imudp  = new TTUBE("MUDI","MUDI","void",440.,440.,535.);
   if (!otubep) otubep = new TTUBE("CALO","CALO","void",350.,350.,410.);
   if (!itubep) itubep = new TTUBE("CALI","CALI","void",210.,210.,270.);
   //if (!orotp)  orotp  = new TRotMatrix("orot","orot",10.,80.,10.,80.,10.,80.);
   if (!orotp)  orotp  = new TRotMatrix("orot","orot", 0, 0, 0, 0, 0, 0 );

   if (!omudnodep) {
     omudnodep = new TNode("OMUDNODE","OMUDNODE","MUDO", 0, 0, 0, "orot" );
     omudnodep->cd();
     imudnodep = new TNode("IMUDNODE","IMUDNODE","MUDI" );
     //   if (!onodep) {
     //onodep = new TNode("ONODE","ONODE","CALO",0.,0.,0.,"orot");
      onodep = new TNode("ONODE","ONODE","CALO");
      //onodep->cd();
      inodep = new TNode("INODE","INODE","CALI");
   }
   
   otubep->SetLineColor(8);
   itubep->SetLineColor(8);
   omudp->SetLineColor(9);
   imudp->SetLineColor(9);
   //onodep->Draw("pad same");
   omudnodep->Draw("pad same");

   TObjArray *pfosp = ((S4PFOMakerBuf *)gJSF->FindModule("S4PFOMaker")
                         ->EventBuf())->GetOutputBufPtr();

   if (pfosp && pfosp->GetEntries()) {
      TIter next(pfosp);
      S4PFObject *pfp = 0;
      while ((pfp = (S4PFObject *)next())) {
         S4CALCluster *clp  = pfp->GetClusterPtr();
         if (clp) {
#ifdef __USEPREHITID__
            S4CALPreHit  *php = ((S4CALExactHit *)
                                    ((S4CALHit *)clp->At(0))->GetExactHitPtr()
                                )->GetPreHitPtr();
#else
            S4CALPreHit  *prep = ((S4CALExactHit *)
                                     ((S4CALHit *)clp->At(0))->GetExactHitPtr()
                                 )->GetPreHitPtr();
            S4TPCPostHit *posp = prep->GetPostHitPtr();
            S4VExactHit  *php  = static_cast<S4VExactHit *>(posp)
                               ? static_cast<S4VExactHit *>(posp)
                               : static_cast<S4VExactHit *>(prep);
#endif
	    
            Double_t Epr = php->GetTotalEnergy();
            Double_t Ecl = clp->GetEnergy();
            Double_t sig = 0.4/TMath::Sqrt(Epr);

            if (TMath::Abs(Ecl - Epr) > 0*sig*Epr) {

	      Int_t pdgc = pfp->GetPDGCode();

	      switch (TMath::Abs(pdgc)) {
	      case 11:
		color=6;       // electron -> purple
		break;
	      case 13:
		color=9;       // muon     -> blue
		break;
	      case 22:
		color=5;       // photon   -> yellow
		break;
	      case 211:
		color=2;       // pi       -> red
		break;
	      case 130:
		color=28;      // K0L      -> brown
		break;
	      case 2212:
		color=8;       // proton   -> green
		break;
	      case 2112:
		color=7;       // neutron  -> cyan                
		break;
	      case 12:
		color=23;       // neutrino  -> grey
		break;
              case 14:
		color=24;       // neutrino  -> grey
		break;
              case 16:
		color=25;       // neutrino  -> grey
		break;		
	      default:
		color=20;      // other    -> grey  
		break;
	      }
	      pfp->Draw(color,opt);
            }
         } else {
            Int_t pdgc = pfp->GetPDGCode();

	    switch (TMath::Abs(pdgc)) {
	      case 11:
		color=6;       // electron -> purple
		break;
	      case 13:
		color=9;       // muon     -> blue
		break;
	      case 22:
		color=5;       // photon   -> yellow
		break;
	      case 211:
		color=2;       // pi       -> red
		break;
	      case 130:
		color=28;      // K0L      -> brown
		break;
	      case 2212:
		color=8;       // proton   -> green
		break;
	      case 2112:
		color=7;       // neutron  -> cyan                
		break;
	      case 12:
		color=40;       // neutrino  -> grey
		break;
              case 14:
		color=40;       // neutrino  -> grey
		break;
              case 16:
		color=40;       // neutrino  -> grey
		break;		
	      default:
		color=20;      // other    -> grey  
		break;
	    }
	    pfp->Draw(color,opt);
            //color++;
            //color %= 10;
          //  pfp->Draw(color,opt);
         }
      }
   }

   S4MUDMCTruth    *ehit    = (S4MUDMCTruth*)gJSF->FindModule("S4MUDMCTruth");
   S4MUDMCTruthBuf *mctbuf  = (S4MUDMCTruthBuf*)ehit->EventBuf();
   TObjArray       *mudhits = mctbuf->GetExactHitsPtr();
   
   if ( mudhits ) {
     TIter mudnext( mudhits );
     S4MUDExactHit *mudhit = 0;
     while( (mudhit = (S4MUDExactHit*)mudnext()) ) {

       TPolyMarker3D *pm3dpp = new TPolyMarker3D(1);
       
       pm3dpp->SetBit(kCanDelete);
       pm3dpp->SetMarkerStyle(20);
       //       pm3dpp->SetMarkerColor(4);
       pm3dpp->SetMarkerSize(0.5);

       Int_t pdgc = TMath::Abs(mudhit->GetPDGCode());

       //if ( pdgc != 12 && pdgc !=14 && pdgc != 16 )
       //  std::cout << "MUD ExactHit PDG code = " << pdgc << std::endl;

       switch (pdgc) {
       case 11:
	 pm3dpp->SetMarkerColor(6);       // electron -> purple                  
	 break;
       case 13:
	 pm3dpp->SetMarkerColor(9);       // muon     -> blue 
	 break;
       case 22:
	 pm3dpp->SetMarkerColor(5);       // photon   -> yellow
	 break;
       case 211:
	 pm3dpp->SetMarkerColor(2);       // pi       -> red
	 break;
       case 130:
	 pm3dpp->SetMarkerColor(28);      // K0L      -> brown
	 break;
       case 2212:
	 pm3dpp->SetMarkerColor(8);      // proton   -> green
	 break;
       case 2112:
	 pm3dpp->SetMarkerColor(7);       // neutron  -> cyan
	 break;
       case 12:
	 pm3dpp->SetMarkerSize(0.2);
	 pm3dpp->SetMarkerColor(40);       // neutrino  -> gray
	 break;
       case 14:
	 pm3dpp->SetMarkerSize(0.2);       
	 pm3dpp->SetMarkerColor(40);       // neutrino  -> gray
	 break;
       case 16:
	 pm3dpp->SetMarkerSize(0.2);       
	 pm3dpp->SetMarkerColor(40);       // neutrino  -> gray
	 break;
       default:
	 pm3dpp->SetMarkerColor(20);       // other    -> grey  
	 break;
       }
       
       pm3dpp->SetPoint(0, mudhit->GetPosition().X(),
                           mudhit->GetPosition().Y(),
                           mudhit->GetPosition().Z());

       pm3dpp->Draw("same");       
     }
   }
   cvp->Update();
   
   if ( gROOT->IsBatch() ) { 
     Char_t fname[256];
     sprintf(fname,"evdisp_%d.eps",gJSF->GetEventNumber());
     cvp->Print(fname);
   }
   last->cd();
}

//_________________________________________________________
void DrawHist()
{
}
