void PlotDouble(const Char_t *fname = "results/jsf.pfoanl.uds91.v0.std.10k.root")
{
  // ------------------------------------------------------------
  //  Open shared libraries
  // ------------------------------------------------------------
  gSystem->Load("libJSF.so");
  gSystem->Load("libS4Utils.so");
  gSystem->Load("libAnlib.so");

  // ------------------------------------------------------------
  //  Open event file and apply cuts
  // ------------------------------------------------------------
  TFile *filep = TFile::Open(fname);
  TNtupleD *tupep = (TNtupleD *)gROOT->FindObject("hevt");

  //tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8&&mass>70&&mass<110","goff");
  tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8","goff");
  TEventList *elistp = (TEventList *)gROOT->FindObject("elist");

  const Double_t kNsig = 2.;	// fit range for gaussian fit in units of sigma

  // ------------------------------------------------------------
  //  Define histograms
  // ------------------------------------------------------------
#if 1
  Double_t dEmin = -15.;
  Double_t dEmax = +15.;
  Double_t dEvmn = -20.;
  Double_t dEvmx = +20.;
  Double_t Emin  =   0.;
  Double_t Emax  = 100.;
#else
  Double_t dEmin = -30.;
  Double_t dEmax = +30.;
  Double_t dEvmn = -40.;
  Double_t dEvmx = +40.;
  Double_t Emin  =   0.;
  Double_t Emax  = 500.;
#endif
  TH1D *hDetkpbp  = new TH1D("hDetkpbp","",120,dEmin,dEmax);
  TH1D *hDeempbp  = new TH1D("hDeempbp","",120,dEmin,dEmax);
  TH1D *hDehdpbp  = new TH1D("hDehdpbp","",120,dEmin,dEmax);
  TH1D *hDoubl    = new TH1D("hDoubl"  ,"",100,dEvmn,dEvmx);
  TH1D *hEah      = new TH1D("hEah"    ,"",100,dEvmn,dEvmx);
  TH1D *hDE       = new TH1D("hDE"     ,"",100,dEvmn,dEvmx);
  TH1D *hDecm     = new TH1D("hDecm"   ,"",100,dEvmn,dEvmx);
  TH1D *hEtkpbp   = new TH1D("hEtkpbp" ,"",100, Emin, Emax);
  TH1D *hEempbp   = new TH1D("hEempbp" ,"",100, Emin, Emax);
  TH1D *hEhdpbp   = new TH1D("hEhdpbp" ,"",100, Emin, Emax);
  TH1D *hEahpbp   = new TH1D("hEahpbp" ,"",100, Emin, Emax);

  // ------------------------------------------------------------
  //  Loop over selected events and fill up histograms
  // ------------------------------------------------------------
  Int_t nevt = elistp->GetN();
  cerr << "Nevt = " << nevt << endl;
#if 1
  Long64_t   *evlist = elistp->GetList();
  ofstream fevlist("fevlist");
  fevlist << nevt << endl;
  for (Int_t i=0; i<nevt; i++) {
     fevlist << evlist[i] << endl;
  }
#endif

  for (Int_t i=0; i<nevt; i++) {
    Int_t ievt = elistp->GetEntry(i);	// get event number

    Double_t npj1, npj2, cosj1, cosj2, mass;
    Double_t etk, eem, ehd, etkpbp, eempbp, ehdpbp, eah, evis;
    Double_t ehdc, edblhdc, edbltk, edblem, edblhd, edbltkab, etkmpc, edbltkmpc, etkc, edbltkc;

    tupep->SetBranchAddress("npj1",&npj1);
    tupep->SetBranchAddress("npj2",&npj2);
    tupep->SetBranchAddress("cosj1",&cosj1);
    tupep->SetBranchAddress("cosj2",&cosj2);
    tupep->SetBranchAddress("mass",&mass);
    tupep->SetBranchAddress("etk",&etk);
    tupep->SetBranchAddress("eem",&eem);
    tupep->SetBranchAddress("ehd",&ehd);
    tupep->SetBranchAddress("etkpbp",&etkpbp);
    tupep->SetBranchAddress("eempbp",&eempbp);
    tupep->SetBranchAddress("ehdpbp",&ehdpbp);
    tupep->SetBranchAddress("eah",&eah);
    tupep->SetBranchAddress("evis",&evis);
    tupep->SetBranchAddress("edbltk",&edbltk);
    tupep->SetBranchAddress("edblem",&edblem);
    tupep->SetBranchAddress("edblhd",&edblhd);
    tupep->SetBranchAddress("edbltkab",&edbltkab);
    tupep->SetBranchAddress("etkmpc",&etkmpc);
    tupep->SetBranchAddress("edbltkmpc",&edbltkmpc);

    tupep->GetEntry(ievt);	// get this event

    if (TString(fname).Contains("hdexact")) {
      const Double_t kFcorr = 1.0;
    } else {
      const Double_t kFcorr = 1.065;
      //const Double_t kFcorr = 1.0308;
    }
    ehdc    = kFcorr * ehd;
    edblhdc = kFcorr * edblhd;

#if 0
    etkc    = etk;		// without Mp correction
    edbltkc = edbltk;		//
#else
    etkc    = etkmpc;		// with Mp correction
    edbltkc = edbltkmpc;	//
#endif

    hDetkpbp->Fill(etkc-etkpbp,1.);
    hDeempbp->Fill(eem-eempbp,1.);
    hDehdpbp->Fill(ehdc-ehdpbp,1.);
#if 0
    hDoubl  ->Fill(edbltkc+edblem+edblhdc,1.);
#else
    hDoubl  ->Fill(edbltkc-edbltkab+edblem+edblhdc,1.);
#endif
    hEah    ->Fill(-eah-1.e-8,1.);
#if 0
    hDE     ->Fill(edbltkc+edblem+edblhdc-eah,1.);
#else
    hDE     ->Fill(edbltkc-edbltkab+edblem+edblhdc-eah,1.);
#endif
    hDecm   ->Fill(etkc+eem+ehdc-etkpbp-eempbp-ehdpbp-eah,1.);
    hEtkpbp ->Fill(etkpbp,1.);
    hEempbp ->Fill(eempbp,1.);
    hEhdpbp ->Fill(ehdpbp,1.);
    hEahpbp ->Fill(eah,1.);
  }
  // ------------------------------------------------------------
  //  Draw histograms
  // ------------------------------------------------------------
  gStyle->SetOptFit(1);
  gStyle->SetOptStat(111111);

  TCanvas *c1p = new TCanvas("c1","",20,20,1620,820);
  TCanvas &c1  = *c1p;
  c1.Divide(4,2);
  Int_t id = 0;

  TPad *cpadp;
  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDetkpbp->Draw();
  ANLGaussFitter gfit(hDetkpbp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDetkpbp->SetFillColor(42);
  hDetkpbp->SetFillStyle(1000);
  hDetkpbp->GetXaxis()->SetTitle("#Delta E_{tk} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDeempbp->Draw();
  ANLGaussFitter gfit(hDeempbp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDeempbp->SetFillColor(50);
  hDeempbp->SetFillStyle(3001);
  hDeempbp->GetXaxis()->SetTitle("#Delta E_{em} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDehdpbp->Draw();
  ANLGaussFitter gfit(hDehdpbp,kNsig);
  gfit.GetFunction()->Draw("same");
  hDehdpbp->SetFillColor(9);
  hDehdpbp->SetFillStyle(3002);
  hDehdpbp->GetXaxis()->SetTitle("#Delta E_{hd} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDecm->Draw();
  ANLGaussFitter gfit(hDecm,kNsig);
  gfit.GetFunction()->Draw("same");
  hDecm->SetFillColor(50);
  hDecm->SetFillStyle(1000);
  hDecm->GetXaxis()->SetTitle("E_{vis}-E_{cm} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hEtkpbp->GetXaxis()->SetTitle("E_{pbp} [GeV]");

  hEtkpbp->SetFillColor(42);
  hEtkpbp->SetFillStyle(3001);
  hEtkpbp->Draw();
  hEtkpbp->SetMaximum(1000);

  hEempbp->SetFillColor(50);
  hEempbp->SetFillStyle(3001);
  hEempbp->Draw("same");

  hEhdpbp->SetFillColor(9);
  hEhdpbp->SetFillStyle(3002);
  hEhdpbp->Draw("same");

  hEahpbp->SetFillColor(38);
  hEahpbp->SetFillStyle(3001);
  hEahpbp->Draw("same");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDoubl->Draw();
#if 0
  ANLGaussFitter gfit(hDoubl,kNsig);
  gfit.GetFunction()->Draw("same");
#endif
  hDoubl->SetFillColor(50);
  hDoubl->SetFillStyle(1000);
  hDoubl->GetXaxis()->SetTitle("E_{double} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hEah->Draw();
#if 0
  ANLGaussFitter gfit(hEah,kNsig);
  gfit.GetFunction()->Draw("same");
#endif
  hEah->SetFillColor(50);
  hEah->SetFillStyle(1000);
  hEah->GetXaxis()->SetTitle("E_{ah} [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDE->Draw();
  ANLGaussFitter gfit(hDE,kNsig);
  gfit.GetFunction()->Draw("same");
  hDE->SetFillColor(50);
  hDE->SetFillStyle(1000);
  hDE->GetXaxis()->SetTitle("#Delta E_{rem} [GeV]");

  c1.cd();
}
