
//
// calcanl.C
//
// 

#include <iomanip>

void calanl()
{
//----------------------------------
// Reset ROOT
//----------------------------------

  gROOT->Reset();

  gErrorAbortLevel = kWarning + 1; 
  gRandom->SetSeed();
       
//----------------------------------
// Create JSF runtime environment
//----------------------------------

  JSFSteer *jsfp = new JSFSteer();
    
//----------------------------------
// Load user defined classes
//----------------------------------

  gSystem->Load("libS4Kalman.so");  
  gSystem->Load("libS4Geom.so");  
  gSystem->Load("libS4KalTrack.so");  
  gSystem->Load("libS4GENMCTruth.so");
  gSystem->Load("libS4HISTMCTruth.so");
  gSystem->Load("libU4VHitMaker.so");
  gSystem->Load("libU4KalDetector.so");  
  gSystem->Load("libU4TPCKalDetector.so");  
  gSystem->Load("libU4TPCHitMaker.so");
  gSystem->Load("libU4ITKalDetector.so");  
  gSystem->Load("libU4ITHitMaker.so");
  gSystem->Load("libU4VTXKalDetector.so");  
  gSystem->Load("libU4VTXHitMaker.so");
  gSystem->Load("libS4TPCMCTruth.so");
  gSystem->Load("libS4ITMCTruth.so");
  gSystem->Load("libS4VTXMCTruth.so");
  gSystem->Load("libS4TPCHitMaker.so");
  gSystem->Load("libS4ITHitMaker.so");
  gSystem->Load("libS4VTXHitMaker.so");
  gSystem->Load("libU4VTrackMaker.so");  
  gSystem->Load("libS4Kern.so");
  gSystem->Load("libU4TPCTrackMaker.so");  
  gSystem->Load("libU4ITTrackMaker.so");  
  gSystem->Load("libU4VTXTrackMaker.so");  
  gSystem->Load("libS4TPCTrackMaker.so");  
  gSystem->Load("libS4ITTrackMaker.so");  
  gSystem->Load("libS4VTXTrackMaker.so");  
  gSystem->Load("libU4HYBTrackMaker.so");
  gSystem->Load("libS4HYBTrackMaker.so");
  gSystem->Load("libS4TPCAnalysis.so");
  gSystem->Load("libS4CALMCTruth.so");
  gSystem->Load("libS4CALHitMaker.so");
  gSystem->Load("libS4CALClusterMaker.so");
  gSystem->Load("libS4CALAnalysis.so");

//----------------------------------
// Set output file
//----------------------------------

  TFile *foutp = new TFile("jsf.calanl.root","RECREATE");
  TFile &fout  = *foutp;
  jsfp->SetOutput(fout);

//----------------------------------
// Set input file
//----------------------------------

//  TFile file("jsfj4.pythia.root");
  TFile file("jsfj4.cal.root");
  file.ls();
  jsfp->SetInput(file);

  Int_t nevent=jsfp->Env()->GetValue("JSFSteer.Nevent",999999);  
  Int_t minevt=1;
  Int_t maxevt=minevt+nevent;

//----------------------------------
// Insert modules
//----------------------------------

  JSFJ4BufManager   *bufman        = new JSFJ4BufManager();

  S4GENMCTruth      *mctruthgen    = new S4GENMCTruth();
  S4HISTMCTruth     *mctruthhist   = new S4HISTMCTruth();
  S4VTXMCTruth      *mctruthvtx    = new S4VTXMCTruth();
  S4ITMCTruth       *mctruthit     = new S4ITMCTruth();
  S4TPCMCTruth      *mctruth       = new S4TPCMCTruth();
  S4CALMCTruth      *mctruthcal    = new S4CALMCTruth();

  S4VTXHitMaker     *hitmakervtx   = new S4VTXHitMaker();
  S4ITHitMaker      *hitmakerit    = new S4ITHitMaker();
  S4TPCHitMaker     *hitmaker      = new S4TPCHitMaker();
  S4VTXTrackMaker   *vtxtrackmaker = new S4VTXTrackMaker();
  S4ITTrackMaker    *ittrackmaker  = new S4ITTrackMaker();
  S4TPCTrackMaker   *tpctrackmaker = new S4TPCTrackMaker();
  S4HYBTrackMaker   *hybtrackmaker = new S4HYBTrackMaker();

  S4CALHitMaker     *calhitmaker   = new S4CALHitMaker();
  S4CALClusterMaker *calclustmaker = new S4CALClusterMaker();

  S4TPCAnalysis     *tpcanalysis   = new S4TPCAnalysis();
  S4CALAnalysis     *calanalysis   = new S4CALAnalysis();

//----------------------------------
// set parameters
//----------------------------------
#if 0 
  //   S4TPCPadHit::SetSpatialResolution("experimental"); // default is "flat".
 
  // Set IP constraint ....
  // S4TPCTrack::SetIPConstraint(kTRUE);

  // Artificial bunch tagging
     Double_t t0 = 0.;
     S4TPCTrack::SetT0pedestal(t0); // [ns]
#endif          

     S4CALHitBuf::SetHitMakingScheme(1); // idealistic (smeared 1 hit/1 exact hit)

//----------------------------------
// Job initialization
//----------------------------------
 
  jsfp->Initialize();

//----------------------------------
// Define KalDetector
//----------------------------------

  TKalDetCradle    *cradle = new TKalDetCradle();
  U4KalDetector    *det    = new U4KalDetector();
  U4TPCKalDetector *tpcdet = new U4TPCKalDetector("FillLayer");
  U4ITKalDetector  *itdet  = new U4ITKalDetector("FillLayer");
  U4VTXKalDetector *vtxdet = new U4VTXKalDetector("FillLayer");
  tpctrackmaker->SetDetector(*tpcdet);
  ittrackmaker ->SetDetector(*itdet);
  vtxtrackmaker->SetDetector(*vtxdet);
  hybtrackmaker->SetDetector(*det);

  //itdet->FBOff();
  cradle->Install(*det);
  cradle->Install(*tpcdet);
  cradle->Install(*itdet);
  cradle->Install(*vtxdet);
  cradle->Close();
  //cradle->SwitchOffMS();
  //cradle->SwitchOffDEDX();

//----------------------------------
// set subtrackers to hybtrackmaker 
//----------------------------------

  hybtrackmaker->SetMainTrackMakerPtr(tpctrackmaker);
  hybtrackmaker->AddSubTrackMakerPtr(ittrackmaker);
  htrackmaker->AddSubTrackMakerPtr(vtxtrackmaker);

//----------------------------------
// Start event loop
//----------------------------------

  for(Int_t ev=minevt;ev<=maxevt;ev++){
    if (!jsfp->GetEvent(ev)) break; 
    cerr << " ---- Event " << jsfp->GetEventNumber() << " ----" << endl;
    if (!jsfp->Process(ev)) {
      jsfp->FillTree();
      jsfp->Clear();
      break;
    }
    jsfp->FillTree();
    jsfp->Clear();
  }
  jsfp->Terminate();
  fout.Write();
}
