//////////////////////////////////////////////////////////////////
//
//  CAL Hit class in ROOT
//
//$Id: S4CALExactHit.cxx,v 1.17 2007/03/20 09:36:10 fujiik Exp $
//
//////////////////////////////////////////////////////////////////

#include <iomanip>
#include <iostream>

#ifndef __HasJSFFromLCIO__
#include "J4CALHit.hh"
#include "J4CLXHit.hh"
#endif
#include "S4CALExactHit.h"
#include "S4CALAddress.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif
using namespace std;

ClassImp(S4CALExactHit)

// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit()
              : S4VExactHit(0,                 // hit ID
                            0,                 // track ID
                            0,                 // mother track ID
                            0,                 // PDG code
                            0,                 // charge
                            TVector3(0.,0.,0.),// position
                            0,                 // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            0,                 // energy deposit
                            0),                // address ptr 
                fPreHitID(0),
                fCellID(0),
                fIsBarrel(kTRUE),
                fIsEM(kTRUE),
                fCellPosition(0.0, 0.0, 0.0),
                fCellXcm(0.0, 0.0, 0.0),
                fPreHitPtr(0)
{
}

// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(Int_t     hitID,       // hit ID
                             Int_t     pcode,       // PDG code
                      const TVector3  &pos,         // position [cm]
                             Double_t  tof,         // TOF      [nsec]
                             Double_t  edep,        // energy deposit [GeV]
                             Int_t     preHitID,    // pre hit ID
                             Int_t     cellID,      // cell ID
                             Int_t     isBarrel,    // energy deposit [GeV]
                             Int_t     isEM,        // energy deposit [GeV]
                       const TVector3 &xcell,       // cell position [cm]
                       const TVector3 &xcm,         // E-weighted pos [cm*GeV]
			     Int_t     trackID,
			     Int_t     mtrackID)
              : S4VExactHit(hitID,             // hit ID
                            trackID,           // track ID
                            mtrackID,          // mother track ID
                            pcode,             // PDG code
                            0,                 // charge
                            pos,               // position
                            tof,               // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            edep,              // energy deposit
                            0),                // address ptr 
                fPreHitID(preHitID),
                fCellID(cellID),
                fIsBarrel(isBarrel),
                fIsEM(isEM),
                fCellPosition(xcell),
                fCellXcm(xcm),
                fPreHitPtr(0)
{
}

#ifndef __HasJSFFromLCIO__
// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(J4CALHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            0,                          // charge
                            TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().y()
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().z()
                                    /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(0.,0.,0.),         // momentum
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
                fIsBarrel(ahit->IsBarrel()),            // is Barrel?
                fIsEM(ahit->IsEM()),                    // is EM?
                fCellPosition(ahit->GetXcell().x()/cm,  // cell position
                              ahit->GetXcell().y()/cm,
			      ahit->GetXcell().z()/cm),
                fCellXcm(ahit->GetXcm().x()/(GeV*cm),   // E-weighted pos
		         ahit->GetXcm().y()/(GeV*cm),
		         ahit->GetXcm().z()/(GeV*cm)),
                fPreHitPtr(0)
{
}


// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(J4CLXHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            ahit->GetCharge(),          // charge
                            TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().y()
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().z()
                                    /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(ahit->GetMomentum().x()/GeV, // momentum
				     ahit->GetMomentum().y()/GeV,
				     ahit->GetMomentum().z()/GeV ), 
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
		fIsBarrel(ahit->IsBarrel()),            // is Barrel?
		fIsEM(ahit->IsEM()),                    // is EM?
		fCellPosition(ahit->GetXcell().x()/cm,  // cell position
		              ahit->GetXcell().y()/cm,
			      ahit->GetXcell().z()/cm),
                fCellXcm(ahit->GetXcm().x()/(GeV*cm),   // E-weighted pos
		         ahit->GetXcm().y()/(GeV*cm),
		         ahit->GetXcm().z()/(GeV*cm)),
                fPreHitPtr(0)
{
}
#endif


// ------------------------------------------------------------------
S4CALExactHit::~S4CALExactHit()
{
}

// ------------------------------------------------------------------
void S4CALExactHit::Print(const Option_t *opt) const
{
  std::cout << std::setiosflags(std::ios::fixed);
  std::cout << "S4CALExactHit ====================";
  std::cout << " CellID=" << fCellID;
  std::cout << " PDGID=" << GetPDGCode();
  std::cout << " IsBarrel=" << fIsBarrel;
  std::cout << " IsEM=" << fIsEM;
  std::cout << " TOF(nsec)=" << GetTOF();
  std::cout << " EnergyDeposit(GeV)=" << GetEnergyDeposit();
  std::cout << std::endl;
  std::cout << " CellPos(cm)=(" << fCellPosition.x();
  std::cout << "," << fCellPosition.y();
  std::cout << "," << fCellPosition.z();
  std::cout << ") ";
  std::cout << " CellXcm(GeV*cm)=(" << fCellXcm.x();
  std::cout << "," << fCellXcm.y();
  std::cout << "," << fCellXcm.z();
  std::cout << ") ";
  std::cout << std::endl;
}

//---------------------------------------------------------------
Int_t S4CALExactHit::Compare(const TObject *hit) const
{
  return 0;
}

//---------------------------------------------------------------
void S4CALExactHit::SetSortBy(Char_t *opt)
{
}

//---------------------------------------------------------------
Bool_t S4CALExactHit::IsSortable() { return kTRUE; }

//---------------------------------------------------------------
//* GetCellID includes cellID correct depends on Jupiter version
Int_t S4CALExactHit::GetCellID()
{
  //* Jupiter version smaller equal zero for old generated data
  const Char_t* j4version = gJSF->Env()->GetValue( "J4.Version", "0.0" );
  Double_t jupiterVersion = atof( j4version );

#ifdef __DEBUG__
  std::cerr << "J4.Version=" << jupiterVersion << std::endl; 
#endif
  
  //-------------------------------------------------
  //* No correction if Jupiter version greater than 0
  if ( jupiterVersion > 0 ) return fCellID;
  //--------------------------------------------------

  //-------------------------------------------------
  //* If (J4.Version == 0), cellID need to be corrected
  //  - nBlocks=2, so convert to 3
  //  - nEMCells changes to same as nHDCells
  Double_t emYmax   = S4CALAddress::GetEMYMax();
  Double_t hdYmax   = S4CALAddress::GetHDYMax();
  Int_t    nTraps   = gJSF->Env()->GetValue( "J4CLX.NTraps", 12 );
  Double_t cellSize = gJSF->Env()->GetValue( "J4CLX.Cell.Size", 1.0 );

  Int_t    nOldBlocks    = 2*nTraps; // old blocks is 2 (Same blockID for Barrel and Endcap+Z)
  Int_t    nBlocks       = 3*nTraps; // new blocks is 3
  
  Int_t    nEMCells = (Int_t)( 2*emYmax*TMath::Tan( TMath::Pi()/nTraps )/cellSize );
  Int_t    nHDCells = (Int_t)( 2*hdYmax*TMath::Tan( TMath::Pi()/nTraps )/cellSize );

  Double_t barrelHalfZ   = gJSF->Env()->GetValue( "J4CLX.Barrel.HalfZ",280.0 );
  Double_t gap           = gJSF->Env()->GetValue( "J4CLX.BarrelEndcapGap",10.0 );
  Double_t emHalfZ       = barrelHalfZ-gap;
  Double_t hdHalfZ       = barrelHalfZ-gap;
  Int_t    nEMLayers     = gJSF->Env()->GetValue( "J4CLX.EM.NLayers",33 );
  Int_t    nHDLayers     = gJSF->Env()->GetValue( "J4CLX.HD.NLayers",46 );
  Int_t    nEMTrapStrips = (Int_t)( 2*emHalfZ / cellSize );
  Int_t    nHDTrapStrips = (Int_t)( 2*hdHalfZ / cellSize );
  Int_t    nEMStrips     = gJSF->Env()->GetValue("J4CLX.EM.NStrips",3);
  Int_t    nHDStrips     = gJSF->Env()->GetValue("J4CLX.HD.NStrips",3);

  //* nEMCells and nHDCells are same in new geometry
  Int_t    nOldEMTotals  = nEMCells*nEMStrips*nEMTrapStrips*nEMLayers;  
  Int_t    nEMTotals     = nHDCells*nEMStrips*nEMTrapStrips*nEMLayers;   
  
  //* current IDs
  Int_t    blockID     = fCellID % nOldBlocks;
  Int_t    remainID    = fCellID / nOldBlocks;
  Int_t    layerID     = (fIsEM) ? remainID%nEMLayers
                                 : (remainID-nOldEMTotals)%nHDLayers;
  Int_t    trapStripID = (fIsEM) ? (remainID/nEMLayers)%nEMTrapStrips
                                 : ((remainID-nOldEMTotals)/nHDLayers)%nHDTrapStrips;
  Int_t    stripID     = (fIsEM) ? (remainID/nEMLayers/nEMTrapStrips)%nEMStrips
                                 : ((remainID-nOldEMTotals)/nHDLayers/nHDTrapStrips)%nHDStrips;
  Int_t    cellID      = (fIsEM) ? remainID/nEMLayers/nEMTrapStrips/nEMStrips
                                 : (remainID-nOldEMTotals)/nHDLayers/nHDTrapStrips/nHDStrips;

  //------------------------------------------------------------------
  //* Old blockID configuration is Barrel/Endcap+Z(0-11), Endcap-Z(12-23)
  //* Change to Barrel(0-11), Endcap+Z(12-23), Endcap-Z(24-35)
  if ( !fIsBarrel ) blockID += nTraps;
  //------------------------------------------------------------------

  Int_t    globalID    = (fIsEM) ? (((cellID*nEMStrips+stripID)*nEMTrapStrips+trapStripID)*nEMLayers+layerID)*nBlocks+blockID
                                 : (nEMTotals+((cellID*nHDStrips+stripID)*nHDTrapStrips+trapStripID)*nHDLayers+layerID)*nBlocks+blockID;

  if ( fIsEM && globalID/nBlocks > nEMTotals ) {
    std::cerr << "**** EM CellID over the nEMTotals! Can be overlap with HD cellID!" << " "
	      << "cellID="   << globalID << " "
	      << "nEMTotal=" << nEMTotals << " "
              << std::endl;
  }

  return globalID;
}
