//*************************************************************************
//* --------------------
//* S4TPCPostHit
//* --------------------
//* (Description)
//*	TPC Hit class.
//*	Mother class : S4VPostHit
//*     
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************

#include <string>
#include <fstream>

#ifndef __HasJSFFromLCIO__
#include "J4TPCPostHit.hh"
#endif

#include "S4TPCPostHit.h"


ClassImp(S4TPCPostHit)

Int_t S4TPCPostHit::fTPCPostHitSortBy = 0; 

//=====================================================================
//* constructor -------------------------------------------------------

#ifndef __HasJSFFromLCIO__
S4TPCPostHit::S4TPCPostHit(const J4TPCPostHit *hitp)
            : S4VExactHit(hitp->GetPostHitID(),           // hit ID
                          hitp->GetTrackID(),             // track ID
                          hitp->GetMotherTrackID(),       // mother track ID
                          hitp->GetPDGEncoding(),         // particle ID
                          (Int_t)hitp->GetCharge(),       // charge
                          (1. / cm) * (TVector3(hitp->GetPrePosition().x(),
                                                hitp->GetPrePosition().y(),
                                                hitp->GetPrePosition().z())),
                          hitp->GetTof(),                 // TOF
                          (1 / GeV) * TVector3(hitp->GetMomentum().x(),
                                               hitp->GetMomentum().y(),
                                               hitp->GetMomentum().z()),
                          hitp->GetTotalEnergy()   / GeV, // total energy
                          0.,                             // energy deposit
                          0,                              // address ptr
                          0)                              // event ID 
{
}
#endif

S4TPCPostHit::S4TPCPostHit(Int_t     evtID,
                           Int_t     trackID,
                           Int_t     mothertrackID,
                           Int_t     hitID, 
                           Int_t     pcode, 
                           Int_t     charge,
                           const TVector3 &pos, 
                           const TVector3 &momentum,
                           Double_t  totE, 
                           Double_t  tof)
            : S4VExactHit(hitID,  trackID, mothertrackID, pcode,
                          charge, pos,     tof,           momentum,
                          totE,   0.,      0,             evtID)
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4TPCPostHit::~S4TPCPostHit()
{
}

#if 0
//=====================================================================
//* SetSortBy --------------------------------------------------------

void S4TPCPostHit::SetSortBy(Char_t *opt )
{
   switch (opt[2]) {
      case 'a':   // sort by "TrackID"
         fTPCPostHitSortBy = 0;
         break;
      case 'm':    // sort by "Time"
      default:
         fTPCPostHitSortBy = 1;
         break;
   }      
}

//=====================================================================
//* IsSoatable --------------------------------------------------------

Bool_t S4TPCPostHit::IsSortable() const { return kTRUE; }
   
//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4TPCPostHit::Compare(const TObject *obj) const
{
   switch (fTPCPostHitSortBy) {
      case 0: {
         Int_t trackid  = GetTrackID();
         Int_t trackido = dynamic_cast<const S4TPCPostHit *> (obj)
                                                    ->GetTrackID();
         return (trackid < trackido ? -1 : (trackid > trackido ? 1 : 0));
      }  
      case 1: {
      default:
         Double_t tof  = GetTOF();
         Double_t tofo = dynamic_cast<const S4TPCPostHit *> (obj)->GetTOF();
         return (tof < tofo ? -1 : (tof > tofo ? 1 : 0));
      }
   }
}
#endif

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4TPCPostHit::DebugPrint(const Option_t *opt)
{
   using namespace std;

   cerr << "------<S4TPCPostHit::DebugPrint>---------" << endl
        << " post hit ID     = "  << GetHitID()         << endl
        << " track ID        = "  << GetTrackID()       << endl
        << " mother track ID = "  << GetMotherTrackID() << endl
        << " pdg code        = "  << GetPDGCode()       << endl
        << " charge          = "  << GetCharge()        << endl
        << " position        = (" << GetPosition().X()  << ", "
                                  << GetPosition().Y()  << ", "
                                  << GetPosition().Z()  << ")" 
                                                        << endl
        << " momentum        = (" << GetMomentum().X()  << ", " 
                                  << GetMomentum().Y()  << ", " 
                                  << GetMomentum().Z()  << ")" 
                                                        << endl
        << " energy          = "  << GetTotalEnergy()   << endl
        << " tof             = "  << GetTOF()           << endl; 
}
