//*************************************************************************
//* --------------------
//* S4BabyTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include "S4BabyTrackMakerBuf.h"
#include "S4BabyExactHit.h"
#include "S4BabyTrack.h"
#include "S4BabyVectorHit.h"

//#define __DEBUG__
#define __LEAVESINGLEHITTRACK__

ClassImp(S4BabyTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4BabyTrackMakerBuf::S4BabyTrackMakerBuf(const char* name,
				       const char *title, 
				       S4BabyTrackMaker *module,
	                               Int_t bufsize)
		 :U4VTrackMakerBuf(name, title, module, bufsize)
{
}	       
	       
S4BabyTrackMakerBuf::S4BabyTrackMakerBuf(S4BabyTrackMaker *module,
			           const char *name,
	                           const char *title, 
	                           Int_t bufsize)
		 :U4VTrackMakerBuf(module, name, title, bufsize)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------

S4BabyTrackMakerBuf::~S4BabyTrackMakerBuf()
{
}

//=====================================================================
//* MakeTracks ----------------------------------------------------------

void S4BabyTrackMakerBuf::MakeTracks()
{
	if (!IsInitialized()) {
		std::cerr << "ERROR  S4BabyTrackMakerBuf::MakeTracks" << std::endl
		     << "...... cluster data not yet initialized." << std::endl;
		return;
	} else if (fDone) {
		return;
	}
	
	fDone = kTRUE;
	
	TObjArray* inputbufp 
		= ((S4BabyMCTruthBuf*)fInputDataBufPtr)->GetExactHitsPtr();
#ifdef __DEBUG__
	std::cerr << " S4BabyTrackMakerBuf::MakeTracks: Nhits of inputbuf = " 
	     << inputbufp->GetEntries() << std::endl;
#endif	
	S4BabyExactHit::SetSortBy("TrackNumber");
	inputbufp->UnSort();
	inputbufp->Sort();
	
	TObjArray tracktmp(2000);	
	
	Int_t tracklast = -9999;
		
	TIter next(inputbufp);
	S4BabyExactHit *exacthitp;
	while ((exacthitp = (S4BabyExactHit *)next())) {
		Int_t trackcur = exacthitp->GetTrackID();
		if ( trackcur != tracklast ) {
			if ( tracktmp.GetEntries() > 0 ) {
				
#ifdef __LEAVESINGLEHITTRACK__
				if (tracktmp.GetEntries() > 0) {
#else
				if (tracktmp.GetEntries() > 2) {
#endif
					AppendTrack( &tracktmp, &fOutputBuf );
#ifdef __DEBUG__
					std::cerr << "S4BabyTrackMakerBuf::MakeTracks: trackNo = "
					     << tracklast << " NEntries = " 
					     << tracktmp.GetEntries() << std::endl;
#endif
				}
				tracktmp.Clear();
			}
			tracklast = trackcur;
		} 
		tracktmp.Add ( exacthitp );
	}
		
	if ( tracktmp.GetEntries() > 0 ) {
		AppendTrack( &tracktmp, &fOutputBuf);
		tracktmp.Clear();
	}
}

//=====================================================================
//* AppendTrack -------------------------------------------------------

void S4BabyTrackMakerBuf::AppendTrack(TObjArray* tracktmpp, TObjArray* outputbufp)
{
	
	S4BabyTrack  *trackp = new S4BabyTrack();
	TObjArray    celltmp(1000);
	
	S4BabyExactHit::SetSortBy("CellNumber");
	tracktmpp->UnSort();
	tracktmpp->Sort();
	
	Int_t celllast = -9999;
		
	TIter next(tracktmpp);
	S4BabyExactHit *exacthitp;
	S4BabyExactHit *lastexacthitp = 0;
	
	while ((exacthitp = (S4BabyExactHit *)next())) {
	
		Int_t cellcur = exacthitp->GetCellNumber();
		if ( cellcur != celllast ) {
			if ( celltmp.GetEntries() > 0 ) {
				
				AppendVectorHit( &celltmp, trackp );
				celltmp.Clear();
			}
			celllast = cellcur;
		} 
		celltmp.Add ( exacthitp );
		lastexacthitp = exacthitp;
	}
	
	
	if ( celltmp.GetEntries() > 0 ) {
		AppendVectorHit( &celltmp, trackp );
		celltmp.Clear();
	}
	
	if ( trackp->GetEntries() > 0 ) {
		Int_t mothertrackN = lastexacthitp->GetMotherTrackID();
		S4BabyTrack *mothertrackp = FindTrackByNo(mothertrackN);
		trackp->SetMotherTrackPtr(mothertrackp);
		outputbufp->Add(trackp);
	} else {
		delete trackp;
	}	
	
}

//=====================================================================
//* AppendVectorHit ---------------------------------------------------

void S4BabyTrackMakerBuf::AppendVectorHit(TObjArray* celltmpp, S4BabyTrack* trackcurp)
{

	Int_t 		trackno = -9999;
	Int_t 		cellno  = -9999;
	S4BabyVectorHit*	vectorhitp  = 0;	
	
	TIter next(celltmpp);
	S4BabyExactHit *exacthitp;
	while ((exacthitp = (S4BabyExactHit *)next())) {
	    if (!vectorhitp) {
		vectorhitp  = new S4BabyVectorHit();
	    }
	    S4BabyTrackHit* hitp = new S4BabyTrackHit(exacthitp);
	    
	    if (vectorhitp->GetEntries() == 0) {           // 1st hit
		trackno = hitp->GetTrackID();
		cellno  = hitp->GetCellNumber();			

		vectorhitp->Append(hitp);
		trackcurp->Append(vectorhitp);
	    } else {

		if (hitp->GetTrackID() == trackno) {
		    if (hitp->GetCellNumber() == cellno) {
			vectorhitp->Add(hitp);

		    } else {
		    	std::cerr << " S4BabyTrackMakerBuf::AppendVectorHit:LogicError!!"
			     << " new Cell not expected." << std::endl;
		    }	   
		} else { 
		   std::cerr << " S4BabyTrackMakerBuf::AppendVectorHit:LogicError!!"
			<< " new Track not expected." << std::endl;	   
		}
	    }
	}
	
	if (vectorhitp && !vectorhitp->IsKilled()) {
#ifdef __DEBUG__
	   vectorhitp->DebugPrint();
#endif
//	   vectorhitp->KillHits();
	   if (vectorhitp->TestBit(S4BabyVectorHit::kELooping)) {
	       trackcurp->Remove(vectorhitp);
	       delete vectorhitp;
	   }
	}
}

S4BabyTrack * S4BabyTrackMakerBuf::FindTrackByNo(Int_t trackno)
{
	TIter next(&fOutputBuf);
	S4BabyTrack *trackp = 0;
	while((trackp = (S4BabyTrack *)next())){
	    Int_t trackN = trackp->GetTrackID();
	    if(trackno == trackN) return trackp;
	}
//	std::cerr<< " Not Find TrackNumber = " << trackno << std::endl;
	return trackp;
}

















