//*************************************************************************
//* -----------
//* S4CALAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CAL data.
//* (Update Record)
//*	2004/10/24  S.Yamamoto	Original version.
//*	2004/12/31  S.Yamamoto	CALE added.
//*************************************************************************

#include "JSFSteer.h"
#include "JSFHelicalTrack.h" 
#include "S4CALAnalysis.h"
#include <iostream>
#include <sstream>

#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4CALHitMaker.h"
#include "S4CALCluster.h"
#include "S4CALClusterMakerBuf.h"
#include "TF1.h"
ClassImp(S4CALAnalysis)

//___________________________________________________________________________
S4CALAnalysis::S4CALAnalysis(const Char_t *name, const Char_t *title)
             : JSFModule(name, title)
{
   //std::cerr << " constructor for S4CALAnalysis is called..." << std::endl;
}

//___________________________________________________________________________
S4CALAnalysis::~S4CALAnalysis()
{
}

//___________________________________________________________________________
Bool_t S4CALAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   fHCALEne      = new TH1D("CALE", "CalE", 500, 0, 0.3);
   fHClusterEne  = new TH1D("CALClusterE", "ClusterE", 500, 0, 10);
   fHPreHitEne   = new TH1D("CALPreHitE", "PreHitE", 500, 0, 5.5);
   fHPostHitEne  = new TH1D("CALPostHitE", "PostHitE", 500, 0, 5.5);
   fHEneRatio0   = new TH1D("CALEneRatio0", "Ecl/Epre", 500, 0, 4);
   fHEneRatio1   = new TH1D("CALEneRatio1", "Ecl/Epost", 500, 0, 4);
   fHEneRatio2   = new TH1D("CALEneRatio2", "Epre/Epost", 500, 0, 2);
   fHTrackEne    = new TH1D("CALTrackEne", "TrackE", 500, 0, 10);
   fHEneRatiotk  = new TH1D("CALEneRatiotk", "Epre/Etk", 500, 0.9, 2.);
   fHEneRatiotk2 = new TH1D("CALEneRatiotk2", "Epost/Etk", 500, 0.9, 2.);
   fHDistDiff    = new TH1D("CALDistDiff", "Abs(Tk-Cl)", 500, -0.9, 10.);
   fHEneTof      = new TH2D("CALTofEne", "Tof:Ecl", 500, 0., 25.,500, 0., 10.);
   fHEhitTof     = new TH2D("CALTofHit", "Tof:Hit", 500, 0., 100.,500, 0., 1.);
   fHdTdP        = new TH2D("CALdTdP", "dTdP", 500, 0., 100.,500, 0., 500.);

   last->cd();

   return kTRUE;
}


//___________________________________________________________________________
Bool_t S4CALAnalysis::Process(Int_t ev)
{
   MonitorClusters(ev);
#if 1
   std::cerr << "Analysis::Process evt " << ev << " is finished" << std::endl;
#endif
   return kTRUE;   
}   

//___________________________________________________________________________
void S4CALAnalysis::MonitorClusters(Int_t ev)
{
   std::cerr << "ev = " << ev << std::endl;
   TDirectory *last = gDirectory;
   gFile->cd("/");

   TObjArray *outputbufhp = dynamic_cast<S4CALHitBuf *> 
                           (gJSF->FindModule("S4CALHitMaker")->EventBuf())
                                                         ->GetOutputBufPtr();
   TIter nexthit(outputbufhp);
   S4CALHit *calhitp;
   Double_t Ecal= 0.;
   while ((calhitp = dynamic_cast<S4CALHit *> (nexthit()))) {
      Double_t Eh  = calhitp->GetEnergyDeposit();
      Ecal += Eh;
   }
      fHCALEne->Fill(Ecal, 1.);  

   TObjArray *outputbufp = dynamic_cast<S4CALClusterMakerBuf *> 
                           (gJSF->FindModule("S4CALClusterMaker")->EventBuf())
                                                         ->GetOutputBufPtr();


   TIter nextcluster(outputbufp);
   S4CALCluster *clp;

   while ((clp = dynamic_cast<S4CALCluster *> (nextcluster()))) {
      Double_t Ecl = clp->GetEnergy();
      //     fHClusterEne->Fill(Ecl*1.73, 1.); // for electron 5GeV
      //     fHClusterEne->Fill(Ecl*1.39*0.96*0.88, 1.);  // for pion 5GeV
      fHClusterEne->Fill(Ecl, 1.);  // for pion 5GeV

      Double_t Tcl = clp->GetTime();
      TVector3 Pcl = clp->GetPosition(); 

      //*** dTOf::dDistance (Hit-Cluster) ***
      TIter nexthit(clp); // prepare hit iterator
      S4CALHit *hitp = 0;
      while ((hitp = static_cast<S4CALHit *>(nexthit()))) {  
         if (hitp->IsLocked()) continue;
         Double_t     Eh  = hitp->GetEnergyDeposit();
         Double_t     Th  = hitp->GetTOF();
         fHEhitTof->Fill(Th,Eh,1.);
	 Double_t dT = Th - Tcl;
	 TVector3 Ph = hitp->GetHitPosition(); 
	 Double_t dP =(Pcl - Ph).Mag();
         fHdTdP->Fill(dT,dP,Eh);
      }

      //*** PreHitEnergy, ClusterEnergy, TrackEnergy Relation ***
      hitp   = static_cast<S4CALHit *>(clp->At(0)); 
      S4CALExactHit *ehitp = static_cast<S4CALExactHit *>(hitp->GetExactHitPtr());
      S4CALPreHit *cphitp = static_cast<S4CALPreHit *>(ehitp->GetPreHitPtr());
      S4TPCPostHit *poshitp = cphitp->GetPostHitPtr();
      //     Double_t     Epre   = cphitp->GetEnergy();
      Double_t     Epre   = cphitp->GetMomentum().Mag();
      Double_t     Epost  = poshitp? poshitp->GetTotalEnergy() : 0.;
      std::cerr << "(Epre,cphitp) = ("
		<< Epre  << ", "
		<< cphitp << ")" << std::endl;
      fHPreHitEne->Fill(Epre, 1.);
      fHPostHitEne->Fill(Epost, 1.);
   
      //Double_t Eratio = Epre/Ecl;
      Double_t Eratio0 = Epre != 0. ?  Ecl/Epre : 0.; //! from 2004.11.23
      fHEneRatio0->Fill(Eratio0, 1.);
      Double_t Eratio1 = Epost != 0. ? Ecl/Epost :0.; //! from 2005.02.04
      fHEneRatio1->Fill(Eratio1, 1.);
      Double_t Eratio2 = Epost != 0. ? Epre/Epost : 0.; 
      fHEneRatio2->Fill(Eratio2, 1.);
   
      TObjArray  &tracks    = clp->GetTracks();
      S4HYBTrack *htkp      = static_cast<S4HYBTrack *>(tracks[0]);
      Double_t    Etk       = htkp ? htkp->GetEnergy() : 0.;   
      Double_t    Eratiotk  = Etk != 0. ? Epre/Etk : 0.;
      Double_t    Eratiotk2 = Etk != 0. ? Epost/Etk : 0.;
#if 1
     std::cerr << "(Epre,Etk,Ecl) = ("
	       << Epost << ", "
	       << Epre << ", "
	       << Etk  << ", "
	       << Ecl  << ")" << std::endl;
#endif
     fHTrackEne->Fill(Etk, 1.);   
     fHEneRatiotk->Fill(Eratiotk, 1.);   
     fHEneRatiotk2->Fill(Eratiotk2, 1.);   

     //*** TOF analysis ***
     Double_t Tofcl = clp->GetTime();
     fHEneTof->Fill(Tofcl,Ecl, 1.);

     //////  track-cluster matching  //////
     JSFRPhiZ xexp; //expected hit position  
     if (htkp) {
        TVector3 xvact = clp->GetPosition();
        htkp->GetHelicalTrackPtr("Kalman")->IntersectWithCylinder(JSFRPhiZ(xvact.Perp(), xvact.Phi(), xvact.Z()), xexp);
	TVector3 xvexp(xexp.XYZ().X(),xexp.XYZ().Y(),xexp.XYZ().Z()); 
	Double_t DistDiff = (clp->GetPosition()-xvexp).Mag();
	using namespace std;
	cerr << "cluster = ("
	     << xvact.Perp() << ","
	     << xvact.Phi() << ","
             << xvact.Z() << ")" << endl;
	cerr << "track = ("
	     <<xvexp.Perp() <<  " "
	     <<xvexp.Phi() <<  " "
	     <<xvexp.Z() <<  " )" << endl;
	cerr << "clp->GetPosition() = ("
	     << xvact.X() << ","
	     << xvact.Y() << ","
             << xvact.Z() << ")" << endl;
	cerr << "tracl = ("
	     <<xvexp.X() <<  " "
	     <<xvexp.Y() <<  " "
	     <<xvexp.Z() <<  " )" << endl;
	fHDistDiff->Fill(DistDiff, 1.);
     }
   }  
   last->cd();
}

//___________________________________________________________________________
#if 0
void S4CALAnalysis::Draw(Option_t/* *opt */)
{
   gFile->cd("/");
   gDirectory->cd();
}
#endif
