//*************************************************************************
//* ---------------------
//* S4CALCHDClusterMaker Class
//* ---------------------
//* (Description)
//*   make charge hadron clusters
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALSmallClusterUtils.h"
#include "S4CALClusterUtils.h"
#include "S4CALTrackMatcher.h"
#include "S4CALCHDClusterMaker.h"
#include "JSFHelicalTrack.h"
#include "S4HYBTrackMakerBuf.h"
#include "S4CALAddress.h"

#include "JSFSteer.h"
#include "TNtuple.h"
#include <iostream>

//#define __MIP__

ClassImp(S4CALCHDClusterMaker)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALCHDClusterMaker::S4CALCHDClusterMaker()
  : fCALResolution(40.), fMaxDeviation(2.),
    fEMTrackDisCutRmin(0.), fEMTrackDisCutRmax(0.), fHDTrackDisCutRmin(0.),
    fHDTrackDisCutRmax(0.), fEMTrackDisCutStep(1.)
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALCHDClusterMaker::~S4CALCHDClusterMaker()
{
}
#if 0
//* small cluster based 2005/11/01
//==========================================================================
//* MakeCHDClusters --------------------------------------------------------

std::map<Int_t,S4CALHit*> S4CALCHDClusterMaker::MakeCHDClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
								std::map<Int_t,S4CALSmallCluster*> &sclmap,
								std::map<Int_t,S4HYBTrack*> &trkmap)
{

  std::map<Int_t,S4CALHit*>    hitmap;

  Double_t emcutR = fEMTrackDisCutRmin;
  Double_t hdcutR = fHDTrackDisCutRmin;

  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  std::map<Int_t,S4HYBTrack*>::iterator titr;

  std::map<Int_t,S4CALSmallCluster*> ilmap;

  //----- make sorted map with distinguishing MIP scls ------
  std::multimap<Double_t,S4CALSmallCluster*> tmpscls;
  std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;

    if(itr->second->IsMIP()){
      ilmap[itr->second->GetILTrackPtr()->GetTrackID()] = itr->second;
    }else{
      tmpscls.insert(std::pair<Double_t,S4CALSmallCluster*>(itr->second->GetEnergyDeposit(),itr->second));
    }

  }
  //---------------------------------------------------------

  //------------- sort track map ----------------------------
  std::multimap<Double_t,S4HYBTrack*> trkmap2;
  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){
    if(titr->second->IsLocked()) continue;
    trkmap2.insert(std::pair<Double_t,S4HYBTrack*>((titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs(),titr->second));
  }
  //---------------------------------------------------------------


  //--- make new clusters using all remaining tracks --------
  std::vector<S4CALRealCluster*> vec;
  std::multimap<Double_t,S4HYBTrack*>::reverse_iterator tmpitr;

  for(tmpitr = trkmap2.rbegin(); tmpitr != trkmap2.rend(); tmpitr++){

    if (tmpitr->second->IsLocked()) continue;

    if((tmpitr->first) < 1.e-10){
      tmpitr->second->Lock();
      continue;
    }else{
      S4CALRealCluster *cls = new S4CALRealCluster(tmpitr->second);
      cls->SetParticleType(kgHdCharged_p);
      Int_t trkID = tmpitr->second->GetTrackID();
      if(ilmap.count(trkID) > 0){ // this track has MIP scl
	cls->AddMIPSmallCluster(ilmap.find(trkID)->second);
	ilmap.find(trkID)->second->Lock();
      }
      vec.push_back(cls);
    }
  }
  //----------------------------------------------------------

  //------------------- track matching -----------------------
  Int_t    nsteps = (Int_t) ((fEMTrackDisCutRmax - emcutR)/fEMTrackDisCutStep);
  Double_t hdstep = (fHDTrackDisCutRmax - hdcutR)/nsteps;   // cm

  while(emcutR < fEMTrackDisCutRmax){
    
    for(Int_t k = 0; k < (Int_t) vec.size(); k++){

      if(vec[k] -> IsLocked())  continue; // cluster is enough large

      Double_t trkmom = (vec[k]->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();
      Double_t pionene = TMath::Sqrt(trkmom*trkmom+0.14*0.14);
#ifdef __MIP__
      // take MIP part into account...
      if(vec[k]->GetMIPEnergy() > 1.e-6) trkmom -= vec[k]->GetMIPEnergy();
#endif

      // we assume that sigma/sqrt(E) = 40% (= fCALResolution)
      // threshold is set at +2*sigma point (= fMaxDeviation*fCALResolution)
      Double_t upper = fMaxDeviation*(fCALResolution/100.)*TMath::Sqrt(pionene) + pionene;

      for(ritr = tmpscls.rbegin(); ritr != tmpscls.rend(); ritr++){

	if (ritr->second->IsLocked()) continue;

	Double_t cutR = 0.;
	(ritr->second->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);

	TVector3 ilpos = vec[k]->GetILPosition();

	//------- consider MIP hits --------
	if(ilpos.Mag() > 1.e-6){ // this track has MIP scl
#if 1
	  //--- temporary estimation ----
	  if((ritr->second->GetPosition()).Mag() < ilpos.Mag()) continue;
	  //if((ritr->second->GetPosition()).Mag() < ilpos.Mag()-5.) continue;
	  //-----------------------------
#else
	  // use deflection angle ??
#endif
	}
	//----------------------------------

	//------------ check track distance --------------
	Double_t dist = S4CALSmallClusterUtils::CalcTrackDistance(vec[k]->GetTrackPtr(),ritr->second->GetPosition());

	if(dist > cutR) continue;

	//---- take care about "core" of the cluster ----
	// not implemented yet
	//-----------------------------------------------

	vec[k]->Add(ritr->second);
	vec[k]->Update();
	ritr->second->Lock();
	//------------------------------------------------

	// set upper limit
#ifdef __MIP__
	upper += vec[k]->GetMIPEnergy();
#endif
	if((vec[k]->GetEnergy()) > upper){
	  vec[k]->Lock();
	  break;  // go to next track
	}

      } // end of itr
    }

    emcutR += fEMTrackDisCutStep;
    hdcutR += hdstep;

  } // end of while(emcutR < ...) loop
    //---------------------------------

  //----------------------------------------------------------


  //----------- unlock and add real clusters -----------------
  for(Int_t k = 0; k < (Int_t) vec.size(); k++){
    vec[k] -> Unlock();
    vec[k] -> GetTrackPtr()->Lock();
    vec[k] -> Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(vec[k]->GetClusterID(), vec[k]));
  }
  //----------------------------------------------------------


  //---------- insert remaining hits into a hitmap -----------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked()) continue;
    std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();
    for(Int_t l = 0; l < (Int_t) hitv.size(); l++){
      hitv[l]->Unlock();     // necessary!
      hitmap[hitv[l]->GetCellID()] = hitv[l];
    }
  }
  //----------------------------------------------------------

  return hitmap;

}
#elif 0
//* perform LDC clustering first 2005/11/01
//==========================================================================
//* MakeCHDClusters --------------------------------------------------------

std::map<Int_t,S4CALHit*> S4CALCHDClusterMaker::MakeCHDClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
								std::map<Int_t,S4CALSmallCluster*> &sclmap,
								std::map<Int_t,S4HYBTrack*> &trkmap)
{

  std::map<Int_t,S4CALHit*>    hitmap;

  Double_t emcutR = fEMTrackDisCutRmin;
  Double_t hdcutR = fHDTrackDisCutRmin;

  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  std::map<Int_t,S4HYBTrack*>::iterator titr;

  std::map<Int_t,S4CALSmallCluster*> ilmap;

  //------- make hitmap with distinguishing MIP scls --------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    itr->second->Lock();       // Lock all

    if(itr->second->IsMIP()){
      ilmap[itr->second->GetILTrackPtr()->GetTrackID()] = itr->second;
    }else{
      std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();
      for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
	hitv[k]->Unlock();     // necessary!
	hitmap[hitv[k]->GetCellID()] = hitv[k];
      }
    }

  }
  //---------------------------------------------------------

  //------------- sort track map ----------------------------
  std::multimap<Double_t,S4HYBTrack*> trkmap2;
  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){
    if(titr->second->IsLocked()) continue;
    trkmap2.insert(std::pair<Double_t,S4HYBTrack*>((titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs(),titr->second));
  }
  //---------------------------------------------------------------


  //--- make new clusters using all remaining tracks --------
  std::vector<S4CALRealCluster*> vec;

  std::multimap<Double_t,S4HYBTrack*>::reverse_iterator tmpitr;
  for(tmpitr = trkmap2.rbegin(); tmpitr != trkmap2.rend(); tmpitr++){

    if ( tmpitr->second->IsLocked() ) continue;

    if( (tmpitr->first) < 1.e-10 ){
      tmpitr->second->Lock();
      continue;
    }else{
      S4CALRealCluster *cls = new S4CALRealCluster(tmpitr->second);
      cls->SetParticleType(kgHdCharged_p);
      Int_t trkID = tmpitr->second->GetTrackID();
      if(ilmap.count(trkID) > 0){ // this track has MIP scl
	cls->AddMIPSmallCluster(ilmap.find(trkID)->second);
      }
      vec.push_back(cls);
    }
  }
  //----------------------------------------------------------

  //------------------ perform clustering --------------------

#if 0
  // LDC's parameter
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(10.);
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(50.);
  S4CALSmallClusterUtils::SetEMDCut(2.);
  S4CALSmallClusterUtils::SetHDDCut(5.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);

  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(15);
  S4CALSmallClusterUtils::SetMinLayCut(-1);
#else
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(8.);
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(40.);
  S4CALSmallClusterUtils::SetEMDCut(2.);
  S4CALSmallClusterUtils::SetHDDCut(5.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);

  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(15);
  S4CALSmallClusterUtils::SetMinLayCut(-1);
#endif
  
  //--------- container ------------
  // small clusters vector
  std::vector<S4CALSmallCluster*> sclvec;
  // merged small clusters vector
  std::vector<S4CALSmallCluster*> msclvec;
  //--------------------------------
  
  //----------- sort hits & (small) clustering -----------
  sclvec = S4CALSmallClusterUtils::MakeClustersLDC(hitmap, kTRUE);
  //------------------------------------------------------

  //----------- merge low multiplicity clusters ----------
  msclvec = S4CALSmallClusterUtils::MergeSmallClustersLDC(sclvec, kTRUE);
  //------------------------------------------------------

  //----------------------------------------------------------

  //------------------- track matching -----------------------
  Int_t    nsteps = (Int_t) ((fEMTrackDisCutRmax - emcutR)/fEMTrackDisCutStep);
  Double_t hdstep = (fHDTrackDisCutRmax - hdcutR)/nsteps;   // cm

  while(emcutR < fEMTrackDisCutRmax){
    
    for(Int_t k = 0; k < (Int_t) vec.size(); k++){

      if(vec[k] -> IsLocked())  continue; // cluster is enough large

      Double_t trkmom = (vec[k]->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();
#ifdef __MIP__
      // take MIP part into account...
      if(vec[k]->GetMIPEnergy() > 1.e-6) trkmom -= vec[k]->GetMIPEnergy();
#endif

      // we assume that sigma/sqrt(E) = 40% (= fCALResolution)
      // threshold is set at +2*sigma point (= fMaxDeviation*fCALResolution)
      Double_t upper = fMaxDeviation*(fCALResolution/100.)*TMath::Sqrt(trkmom) + trkmom;

      for(Int_t l = 0; l < (Int_t) msclvec.size(); l++){

	if (msclvec[l]->IsLocked()) continue;

	Double_t cutR = 0.;
	(msclvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);

	TVector3 ilpos = vec[k]->GetILPosition();

	//------------ check track distance --------------
	Double_t dist = S4CALSmallClusterUtils::CalcTrackDistance(vec[k]->GetTrackPtr(),msclvec[l]->GetPosition());

	if(dist > cutR) continue;

	vec[k]->Add(msclvec[l]);
	vec[k]->Update();
	msclvec[l]->Lock();
	//------------------------------------------------

	// set upper limit
#ifdef __MIP__
	upper += vec[k]->GetMIPEnergy();
#endif
	if((vec[k]->GetEnergy()) > upper){
	  vec[k]->Lock();
	  break;  // go to next track
	}

      } // end of itr
    }

    emcutR += fEMTrackDisCutStep;
    hdcutR += hdstep;

  } // end of while(emcutR < ...) loop
    //---------------------------------

  //----------------------------------------------------------


  //----------- unlock and add real clusters -----------------
  for(Int_t k = 0; k < (Int_t) vec.size(); k++){
    vec[k] -> Unlock();
    vec[k] -> GetTrackPtr()->Lock();
    vec[k] -> Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(vec[k]->GetClusterID(), vec[k]));
  }
  //----------------------------------------------------------


  //------------ delete temporary small clusters -------------
  std::map<Int_t,S4CALHit*> hitmap2;

  for(Int_t l = 0; l < (Int_t) msclvec.size(); l++){
    if(msclvec[l]->IsLocked()) continue;
    std::vector<S4CALHit*> hvec = msclvec[l]->GetCALHitsVector();
    for(Int_t m = 0; m < (Int_t) hvec.size(); m++){
      hvec[m]->Unlock();
      hitmap2[(hvec[m]->GetCellID())] = hvec[m];
    }
    delete msclvec[l];
  }
  //----------------------------------------------------------

  return hitmap2;

}
#else
//* hit based
//==========================================================================
//* MakeCHDClusters --------------------------------------------------------

std::map<Int_t,S4CALHit*> S4CALCHDClusterMaker::MakeCHDClusters(std::map<Int_t,S4CALRealCluster*> &clmap,
								std::map<Int_t,S4CALSmallCluster*> &sclmap,
								std::map<Int_t,S4HYBTrack*> &trkmap)
{

  std::map<Int_t,S4CALHit*>    hitmap;

  Double_t emcutR = fEMTrackDisCutRmin;
  Double_t hdcutR = fHDTrackDisCutRmin;

  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  std::map<Int_t,S4HYBTrack*>::iterator titr;

  std::map<Int_t,S4CALSmallCluster*> ilmap;

  //------- make hitmap with distinguishing MIP scls --------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    itr->second->Lock();       // Lock all

    if(itr->second->IsMIP()){
      ilmap[itr->second->GetILTrackPtr()->GetTrackID()] = itr->second;
    }else{
      std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();
      for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
	hitv[k]->Unlock();     // necessary!
	hitmap[hitv[k]->GetCellID()] = hitv[k];
      }
    }

  }
  //---------------------------------------------------------

  //-------- sort hitmap ------------
  std::vector<S4CALHit*> hvec;
#if 0

  //----- sort by layer ID -----
  //  static const Int_t emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
  std::map<Int_t,S4CALHit*>::iterator hitr;
  std::multimap<Int_t,S4CALHit*> layermap;
  std::multimap<Int_t,S4CALHit*>::iterator litr;
  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    Int_t layerID = S4CALAddress::GetLayerID(hitr->second->GetCellID(),hitr->second->IsEM());
    if(!hitr->second->IsEM()) layerID += emnlayers;
    layermap.insert(std::pair<Int_t,S4CALHit*>(layerID,hitr->second));;
  }
  for(litr = layermap.begin(); litr != layermap.end(); litr++)
    hvec.push_back(litr->second);
  //----------------------------

#else
  std::map<Int_t,S4CALHit*>::iterator hitr;
  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++)
    hvec.push_back(hitr->second);
#endif
  //---------------------------------

  //------------- sort track map ----------------------------
  std::multimap<Double_t,S4HYBTrack*> trkmap2;
  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){
    if(titr->second->IsLocked()) continue;
    trkmap2.insert(std::pair<Double_t,S4HYBTrack*>((titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs(),titr->second));
  }
  //---------------------------------------------------------------


  //--- make new clusters using all remaining tracks --------
  std::vector<S4CALRealCluster*> vec;

  std::multimap<Double_t,S4HYBTrack*>::reverse_iterator tmpitr;
  for(tmpitr = trkmap2.rbegin(); tmpitr != trkmap2.rend(); tmpitr++){

    if ( tmpitr->second->IsLocked() ) continue;

    if( (tmpitr->first) < 1.e-10 ){
      tmpitr->second->Lock();
      continue;
    }else{
      S4CALRealCluster *cls = new S4CALRealCluster(tmpitr->second);
      cls->SetParticleType(kgHdCharged_p);
      Int_t trkID = tmpitr->second->GetTrackID();
      if(ilmap.count(trkID) > 0){ // this track has MIP scl
	cls->AddMIPSmallCluster(ilmap.find(trkID)->second);
      }
      vec.push_back(cls);
    }
  }
  //----------------------------------------------------------


  //* calculate distance when performing track matching

  //------------------- track matching -----------------------

  Int_t    nsteps = (Int_t) ((fEMTrackDisCutRmax - emcutR)/fEMTrackDisCutStep);
  Double_t hdstep = (fHDTrackDisCutRmax - hdcutR)/nsteps;   // cm

  while(emcutR < fEMTrackDisCutRmax){
    
    for(Int_t k = 0; k < (Int_t) vec.size(); k++){

      if(vec[k] -> IsLocked())  continue; // cluster is enough large

      Double_t trkmom = (vec[k]->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();
#ifdef __MIP__
      // take MIP part into account...
      if(vec[k]->GetMIPEnergy() > 1.e-6) trkmom -= vec[k]->GetMIPEnergy();
#endif

      // we assume that sigma/sqrt(E) = 40% (= fCALResolution)
      // threshold is set at +2*sigma point (= fMaxDeviation*fCALResolution)
      Double_t upper = fMaxDeviation*(fCALResolution/100.)*TMath::Sqrt(trkmom) + trkmom;

      for(Int_t l = 0; l < (Int_t) hvec.size(); l++){
      
	if (hvec[l]->IsLocked()) continue;

	Double_t cutR = 0.;

	/////////////////////////////////////////////////////////
	//* def.
#if 1
	//* default
	(hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
#elif 0
	//* cone-like
	if(trkmom > 2.){
	  static Double_t degree = 20.;
	  Double_t plus = ((S4CALSmallClusterUtils::GetInTrack(vec[k]->GetTrackPtr(),210.,270.))[0] - (hvec[l]->GetCellPosition())).Mag()*TMath::Tan(TMath::Pi()*degree/180.);
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR+plus);
	}else{
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
	}
#else
	//* cone-like 2
	if(trkmom > 2.){
	  static Double_t degree = 20.;
	  Double_t fromin = ((S4CALSmallClusterUtils::GetInTrack(vec[k]->GetTrackPtr(),210.,270.))[0] - (hvec[l]->GetCellPosition())).Mag();
	  Double_t plus = fromin*TMath::Pi()*degree/180.;
	  (hvec[l]->IsEM()) ? (cutR = emcutR+plus) : (cutR = hdcutR+plus);
	}else{
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
	}

#endif
	/////////////////////////////////////////////////////////////

	TVector3 ilpos = vec[k]->GetILPosition();

	//------------ check track distance --------------
	Double_t dist = S4CALSmallClusterUtils::CalcTrackDistance(vec[k]->GetTrackPtr(),hvec[l]->GetCellPosition());

	if(dist > cutR) continue;

	if(ilpos.Mag() > 1.e-6){ // this track has MIP scl
#if 0
	  if( (hvec[l]->GetCellPosition()).Mag() < ilpos.Mag() ) continue;
#else
	  // use deflection angle ??
#endif
	}

	vec[k]->Add(hvec[l]);
	vec[k]->Update();
	hvec[l]->Lock();
	//------------------------------------------------

	// set upper limit
#ifdef __MIP__
	upper += vec[k]->GetMIPEnergy();
#endif
	if((vec[k]->GetEnergy()) > upper){
	  std::cerr << "evergy exceed upper limit!" << std::endl;
	  vec[k]->Lock();
	  break;  // go to next track
	}

      } // end of itr
    }

    emcutR += fEMTrackDisCutStep;
    hdcutR += hdstep;

  } // end of while(emcutR < ...) loop
    //---------------------------------

  //////////////////////////////////////////////////////////////////////
  //* def.
#if 0
  //* try again if hits are not so matched
  while(emcutR < fEMTrackDisCutRmax + 5.){
    
    for(Int_t k = 0; k < (Int_t) vec.size(); k++){

      if(vec[k] -> IsLocked())  continue; // cluster is enough large

      Double_t trkmom = (vec[k]->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();

      if(trkmom < 2.)           continue;

      Double_t lower = -1.*fMaxDeviation*(fCALResolution/100.)*TMath::Sqrt(trkmom) + trkmom;
      if(vec[k]->GetEnergy() > lower) continue;

      // we assume that sigma/sqrt(E) = 40% (= fCALResolution)
      // threshold is set at +2*sigma point (= fMaxDeviation*fCALResolution)
      Double_t upper = fMaxDeviation*(fCALResolution/100.)*TMath::Sqrt(trkmom) + trkmom;

      for(Int_t l = 0; l < (Int_t) hvec.size(); l++){
      
	if (hvec[l]->IsLocked()) continue;

	Double_t cutR = 0.;

	//* def.
#if 1
	//* default
	(hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
#elif 0
	//* cone-like
	if(trkmom > 2.){
	  static Double_t degree = 20.;
	  Double_t plus = ((S4CALSmallClusterUtils::GetInTrack(vec[k]->GetTrackPtr(),210.,270.))[0] - (hvec[l]->GetCellPosition())).Mag()*TMath::Tan(TMath::Pi()*degree/180.);
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR+plus);
	}else{
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
	}
#else
	//* cone-like 2
	if(trkmom > 2.){
	  static Double_t degree = 20.;
	  Double_t fromin = ((S4CALSmallClusterUtils::GetInTrack(vec[k]->GetTrackPtr(),210.,270.))[0] - (hvec[l]->GetCellPosition())).Mag();
	  Double_t plus = fromin*TMath::Pi()*degree/180.;
	  (hvec[l]->IsEM()) ? (cutR = emcutR+plus) : (cutR = hdcutR+plus);
	}else{
	  (hvec[l]->IsEM()) ? (cutR = emcutR) : (cutR = hdcutR);
	}

#endif

	TVector3 ilpos = vec[k]->GetILPosition();

	//------------ check track distance --------------
	Double_t dist = S4CALSmallClusterUtils::CalcTrackDistance(vec[k]->GetTrackPtr(),hvec[l]->GetCellPosition());

	if(dist > cutR) continue;

	if(ilpos.Mag() > 1.e-6){ // this track has MIP scl
#if 0
	  if( (hvec[l]->GetCellPosition()).Mag() < ilpos.Mag() ) continue;
#else
	  // use deflection angle ??
#endif
	}

	vec[k]->Add(hvec[l]);
	vec[k]->Update();
	hvec[l]->Lock();
	//------------------------------------------------

	// set upper limit
	if((vec[k]->GetEnergy()) > upper){
	  std::cerr << "energy exceed upper limit!" << std::endl;
	  vec[k]->Lock();
	  break;  // go to next track
	}

      } // end of itr
    }

    emcutR += fEMTrackDisCutStep;
    hdcutR += hdstep;

  } // end of while(emcutR < ...) loop
    //---------------------------------
#endif
  //////////////////////////////////////////////////////////////////////////
  //----------------------------------------------------------


  //----------- unlock and add real clusters -----------------
  for(Int_t k = 0; k < (Int_t) vec.size(); k++){
    vec[k] -> Unlock();
    vec[k] -> GetTrackPtr()->Lock();
    vec[k] -> Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(vec[k]->GetClusterID(), vec[k]));
  }
  //----------------------------------------------------------

  return hitmap;

}
#endif
//==========================================================================
//* DefiniteCHDIs ----------------------------------------------------------

std::vector<Int_t> S4CALCHDClusterMaker::DefiniteCHDIs(std::map<Int_t,S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap)
{

  // Find and Lock definite charged hadron origined hits within EM region.
  // Rturn value is ID list of small clusters which locked here.

  std::vector<Int_t> idvec;
  std::vector<Int_t> failedid;
  std::vector<S4CALSmallCluster*> scls;

  std::map<Int_t,S4HYBTrack*>::iterator itr;

  std::multimap<Double_t,S4HYBTrack*> trkmap2;
  std::multimap<Double_t,S4HYBTrack*>::reverse_iterator titr;

  //------------------- sort trkmap ---------------------
  for(itr = trkmap.begin(); itr != trkmap.end(); itr++){

    if( itr->second->IsLocked() ) continue;

    Double_t mom = (itr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();

    //if(mom < 1.) continue;  // we don't use less than 1GeV/c tracks

    trkmap2.insert(std::pair<Double_t,S4HYBTrack*>(mom,itr->second));

  }
  //-----------------------------------------------------

  S4CALClusterUtils *util = new S4CALClusterUtils();

  std::map<Int_t,S4CALSmallCluster*>::iterator sitr;

  for(titr = trkmap2.rbegin(); titr != trkmap2.rend(); titr++){

    S4CALRealCluster* real = new S4CALRealCluster();
    for(sitr = sclmap.begin(); sitr != sclmap.end(); sitr++){
      if(sitr->second->IsLocked() || !(sitr->second->IsEM()) )  continue;
      //if(sitr->second->GetNHits() < 5) continue;
      Double_t dist = S4CALSmallClusterUtils::CalcTrackDistance(titr->second, sitr->second->GetPosition());
      if(dist > 7.) continue;
      real->Add(sitr->second);
      scls.push_back(sitr->second);
      sitr->second->Lock();
    }

    Double_t energy = 0.;

    for(Int_t k = 0; k < (Int_t) scls.size(); k++)
      energy += scls[k]->GetEnergy();

    if( (energy/(titr->first) < 0.9)
	&& (real->GetThrust() > 0.9)
	/*&& ((real->GetBroadening())/(real->GetTrhrust()) < 0.4)*/
	/*&& TMath::Abs(real->GetNHits()-38) < 4*/ ){
      for(Int_t i = 0; i < (Int_t) scls.size(); i++){
	idvec.push_back(scls[i]->GetClusterID());
      }
    }else{
      for(Int_t i = 0; i < (Int_t) scls.size(); i++){
	failedid.push_back(scls[i]->GetClusterID());
      }
      delete real;
    }

  } // end of loop for tracks

  delete util;

  //-------- unlock failed small clsuters -----------
  for(Int_t k = 0; k < (Int_t) failedid.size(); k++){
    sclmap.find(failedid[k])->second->Unlock();
  }
  //-------------------------------------------------

  return idvec;

}
