//*************************************************************************
//* --------------------
//* S4CALRealClusterMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes CALRealClusters.
//* (Requires)
//* (Provides)
//* (Update Record)
//*
//*************************************************************************

#include <iostream>
#include "JSFSteer.h"
#include "JSFHelicalTrack.h"
#include "S4CALRealClusterMakerBuf.h"
#include "S4CALSmallClusterMakerBuf.h"
#include "S4CALHitMaker.h"
#include "S4CALTrackMatcher.h"
#include "S4CALClusterUtils.h"
#include "S4TPCPostHit.h"
#include "S4CALPreHit.h"
#include "S4HYBTrackMakerBuf.h"
#include "S4CALAddress.h"
#include "S4CALExactHit.h"
#include "S4CALEMClusterMaker.h"
#include "S4CALMuonClusterMaker.h"
#include "S4CALCHDClusterMaker.h"
#include "S4CALNHDClusterMaker.h"
#include "JSFGenerator.h"
#include "JSFGeneratorParticle.h"

#include "TNtuple.h"
#include <cmath>


//---- method flags ----
#define __MUON__
//#define __DEFCHD__
//#define __DEFNHD__
#define __EM__
#define __CHD__
#define __NHD__

//#define __ECCHEAT__
//#define __USEHIT__
#ifdef __USEHIT__
//#define __GAMCHEAT__
#endif
//----------------------


#define __UNLOCK__
//#define __DEBUG__


ClassImp(S4CALRealClusterMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------

S4CALRealClusterMakerBuf::S4CALRealClusterMakerBuf(S4CALRealClusterMaker *module, 
						   const Char_t      *name, 
						   const Char_t      *title, 
						   Int_t bufsize)
  : JSFEventBuf(name, title, (JSFModule*)module),
    fDone(kFALSE),
    fInputDataBufPtr(0), fOutputBufPtr(0), fBufSize(bufsize)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALRealClusterMakerBuf::~S4CALRealClusterMakerBuf()
{
  if(fOutputBufPtr){
    fOutputBufPtr->Delete();
    delete fOutputBufPtr;
  }
}

//===================================================================== 
//* GetInputBufPtr ----------------------------------------------------

JSFEventBuf* S4CALRealClusterMakerBuf::GetInputDataBufPtr()
{
  return fInputDataBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ------------------------------------------------------

TObjArray* S4CALRealClusterMakerBuf::GetOutputBufPtr()
{
   return fOutputBufPtr;
}

//=====================================================================
//* GetEntries --------------------------------------------------------

Int_t S4CALRealClusterMakerBuf::GetEntries()
{
  if (!fDone) MakeRealClusters();
  return fOutputBufPtr->GetEntries();
}

//=====================================================================
//* SetInputDataBufPtr ------------------------------------------------

void S4CALRealClusterMakerBuf::SetInputDataBufPtr(JSFEventBuf *buf)
{
  fDone = kFALSE;
  fInputDataBufPtr = buf;
  if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
  if (fOutputBufPtr->GetEntries() > 0) fOutputBufPtr->Delete();
}

//=====================================================================
//* MakeRealClusters -------------------------------------------------

void S4CALRealClusterMakerBuf::MakeRealClusters()
{
  
  typedef std::map<Int_t, S4CALRealCluster*>  CLMAP;
  typedef std::map<Int_t, S4CALSmallCluster*> SCLMAP;
  typedef std::map<Int_t, S4HYBTrack*>        TRKMAP;
  typedef std::multimap<MatchedType_t, std::vector<Double_t> > MULTIMAP;
  SCLMAP::iterator itr;
  TRKMAP::iterator titr;
  CLMAP::iterator clitr;

  if (fDone) return; // check whether Real clusters are already made

  fDone = kTRUE;
  fOutputBufPtr->SetOwner();        // buffer owns Real clusters 


  //====================================================
  // make small cluster map & track map
  SCLMAP sclmap = MakeSmallClusterMap(0); // nhits > 0 small clusters are used 
  TRKMAP trkmap = MakeTrackMap();
  if(sclmap.size() == 0){
    std::cout << " There is no small cluster!" << std::endl;
    return;
  }
  //----------------------------------------------------
  //====================================================


  //====================================================
  // make clusters
  
  CLMAP clmap;  // temporary clusters list

  S4CALClusterUtils     *util     = new S4CALClusterUtils(); // clustering tool
  S4CALMuonClusterMaker *mumaker  = new S4CALMuonClusterMaker();
  S4CALEMClusterMaker   *emmaker  = new S4CALEMClusterMaker();
  S4CALCHDClusterMaker  *chdmaker = new S4CALCHDClusterMaker();
  S4CALNHDClusterMaker  *nhdmaker = new S4CALNHDClusterMaker();


  ///////////////////////////////////////////////////////////////////////////
  static TNtuple  *PidNt = new TNtuple("PidNt","particle ID","evt:pid:e:flag");
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    std::map<Int_t, Double_t>::iterator piditr;
    for(piditr = (itr->second->GetPIDMap()).begin(); piditr != (itr->second->GetPIDMap()).end(); piditr++){
      PidNt->Fill(gJSF->GetEventNumber(),piditr->first,piditr->second,-1);
    }
  }
  ///////////////////////////////////////////////////////////////////////////

#if 0
  //--- ECAL/HCAL matching (small cluster based) ---
  util -> EHMatcher(sclmap,
		    10.);   // cut value
  //------------------------------------------------
#endif

  //-- calculate disappeared energy (cheating!!) ---
  util -> CalcMissingEnergy();
  //------------------------------------------------

#ifdef __MUON__
  //--- muon clustering (cheated!!) ----------------
  mumaker -> MakeMuonClusters(clmap,sclmap,trkmap);
  //------------------------------------------------
#endif

#ifdef __DEFCHD__
  //------ lock definite charged hadron scls -------
  std::vector<Int_t> defid = chdmaker->DefiniteCHDIs(sclmap,trkmap);
  //------------------------------------------------
#endif

#ifdef __DEFNHD__
  //------ lock definite neutral hadron scls -------
  std::vector<Int_t> defnhdid = nhdmaker->DefiniteNHDIs(sclmap,trkmap);
  //------------------------------------------------
#endif

#ifdef __ECCHEAT__
#if 0
  //--- make clusters in endcap region (cheated) ---
  MakeECClusters(clmap,sclmap,trkmap);
  //------------------------------------------------
#else
  //--- k/pi clustering (cheated!!) ----------------
  //--- !!! VERY TMPORARY !!! ----------------------
  // take care charged particles (currently assuming 
  // only pi and K) in the EndCap region
  MakeKPIClusters(clmap,sclmap,trkmap);
  //------------------------------------------------
#endif
#endif

  ///////////////////////////////////////////////////////////////////////////
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    std::map<Int_t, Double_t>::iterator piditr;
    if ( !(itr->second->IsLocked()) ){
      for(piditr = itr->second->GetPIDMap().begin(); piditr != itr->second->GetPIDMap().end(); piditr++){	
	PidNt->Fill(gJSF->GetEventNumber(),piditr->first,piditr->second,0);  
      }
    }
  }
  ///////////////////////////////////////////////////////////////////////////

#ifdef __GAMCHEAT__
  //--- Add cheated gamma hits into clmap ----------
  MakeGammaCheatedClusters(clmap, sclmap);
  //------------------------------------------------
#else
#ifdef __EM__
  //--- Add EM-like small clusters into clmap ------
  emmaker -> SetTrackDistanceCut(7.);
  emmaker -> SetTubeRCut(7.);            // tube radius for clustering
  emmaker -> SetTubeHalfHCut(15.);       // tube half hight for clustering

  emmaker -> MakeEMClusters(clmap,sclmap,trkmap);
  //------------------------------------------------

#if 0
  //--- Associate EM-like clusters with track ------
  emmaker -> SetMinLayerCut(12.);   // cut value for track min layer mutching

  emmaker -> EMTrackMatch(clmap,sclmap,trkmap);
  //------------------------------------------------
#endif

#endif
#endif


  ///////////////////////////////////////////////////////////////////////////
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    std::map<Int_t, Double_t>::iterator piditr;
    if ( !(itr->second->IsLocked()) ){
      for(piditr = itr->second->GetPIDMap().begin(); piditr != itr->second->GetPIDMap().end(); piditr++){	
	PidNt->Fill(gJSF->GetEventNumber(),piditr->first,piditr->second,1);  
      }
    }
  }
  ///////////////////////////////////////////////////////////////////////////


#ifdef __DEFCHD__
  for(Int_t d = 0; d < (Int_t) defid.size(); d++){
    sclmap.find(defid[d])->second->Unlock();
  }
#endif

  std::map<Int_t,S4CALHit*> hitmap;

#ifdef __CHD__
  //--- Make charged hadron clusters using track ---
  chdmaker -> SetCALResolution(40.);    // percent/sqrt(E)
  chdmaker -> SetMaxDeviation(2.);      // allowed difference from track mom.
  chdmaker -> SetEMTrackDisCutRmin(0.);
  chdmaker -> SetEMTrackDisCutRmax(20.); // default
  //chdmaker -> SetEMTrackDisCutRmax(25.);
  chdmaker -> SetHDTrackDisCutRmin(0.);
  chdmaker -> SetHDTrackDisCutRmax(40.); // default
  //chdmaker -> SetHDTrackDisCutRmax(60.);
  chdmaker -> SetEMTrackDisCutStep(1.);

  //* return remaining hits
  hitmap = chdmaker -> MakeCHDClusters(clmap,sclmap,trkmap);
  //------------------------------------------------
#endif

#ifdef __DEFNHD__
  //---- release definite neutral hadron origined small clusters ----
  for(Int_t k = 0; k < (Int_t) defnhdid.size(); k++)
    sclmap.find(defnhdid[k])->second->Unlock();
  //-----------------------------------------------------------------
#endif

  //------ add all remaining hits into hitmap --------
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if(itr->second->IsLocked()) continue;
    std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();
    for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
      hitv[k]->Unlock();
      hitmap.insert(std::pair<Int_t,S4CALHit*>(hitv[k]->GetCellID(),hitv[k]));
    }
  }
  //--------------------------------------------------

  ///////////////////////////////////////////////////////////////////////////
  // temporary!
  std::map<Int_t,S4CALSmallCluster*> hsclmap;
  std::map<Int_t,S4CALHit*>::iterator hitr;
  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    S4CALSmallCluster* scl = new S4CALSmallCluster();
    scl->Add(hitr->second);
    scl->SetClusterID(hitr->second->GetCellID());
    scl->Update();
    hsclmap.insert(std::pair<Int_t,S4CALSmallCluster*>(scl->GetClusterID(),scl));
  }
  for(itr = hsclmap.begin(); itr != hsclmap.end(); itr++){
    std::map<Int_t, Double_t>::iterator piditr;
    if ( !(itr->second->IsLocked()) ){
      for(piditr = itr->second->GetPIDMap().begin(); piditr != itr->second->GetPIDMap().end(); piditr++){	
	PidNt->Fill(gJSF->GetEventNumber(),piditr->first,piditr->second,2);  
      }
    }
  }
#if 1
  for(itr = hsclmap.begin(); itr != hsclmap.end(); itr++){
    delete (itr->second);
  }
#endif
  ///////////////////////////////////////////////////////////////////////////


#ifdef __NHD__
  //------ Make neutral hadron clusters ------------
  nhdmaker -> MakeNHDClusters(clmap,hitmap);
  //------------------------------------------------
#endif


  ///////////////////////////////////////////////////////////////////////////
  ///////////////////////////// Fill Ntuple /////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////

  Double_t zmassec = 0.; 
  Double_t zmassba = 0.; 
  Double_t trkmom = 0.;
  Int_t nchd = 0;
  Int_t nem  = 0;
  Int_t nnhd = 0;
  Int_t nmu  = 0;
  Double_t energy = 0.;

  Int_t nele  = 0;
  Int_t ngam  = 0;
  Int_t ntrk  = 0;

  // for emtotal
  Double_t eetrack = 0.;
  Double_t eepi    = 0.;
  Double_t eegam   = 0.;
  Double_t eee     = 0.;
  Double_t eek     = 0.;
  Double_t eep     = 0.;
  Double_t een     = 0.;
  Double_t eemu    = 0.;
  Double_t eeck    = 0.;
  Double_t eeother = 0.;
  Double_t eemsum  = 0.;

  // for hdctotal
  Double_t hdcetrack = 0.;
  Double_t hdcepi    = 0.;
  Double_t hdcegam   = 0.;
  Double_t hdcee     = 0.;
  Double_t hdcek     = 0.;
  Double_t hdcep     = 0.;
  Double_t hdcen     = 0.;
  Double_t hdcemu    = 0.;
  Double_t hdceck    = 0.;
  Double_t hdceother = 0.;

  Double_t enhdsum   = 0.;

  Double_t emutrack  = 0.;

  static TNtuple* JetNt = new TNtuple("JetNt","Zqq-bar ntuple","nevt:zmassba:zmassec:echdtrack:eem:emutrack:enhd:neu:dis");
  static TNtuple *emtotal = new TNtuple("emtotal","em event","nevt:zmassba:zmassec:etrack:epi:egam:ee:ek:ep:en:emu:eck:eother:allee:allegam");
  static TNtuple *hdctotal = new TNtuple("hdctotal","chd event","zmassba:zmassec:etrack:epi:egam:ee:ek:ep:en:emu:eck:eother:allepi:allep:alleck");

  for ( clitr = clmap.begin(); clitr != clmap.end(); clitr++){

    EParticleType_t ptype = clitr->second->GetParticleType();

    if ( ptype == kgHdCharged_p ){
      ////////////////////////////////////////////////////
      std::cout << "hdc" << std::endl
		<< std::endl
		<< "!!!!!!! cls energy = "
		<< clitr->second->GetEnergy()
		<< std::endl
		<< "!!!!!!! cls track momentum = "
		<< (clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs()
		<< std::endl;
      /////////////////////////////////////////////////////
      nchd++;
      trkmom = (clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0)).Abs();

      energy = TMath::Sqrt(trkmom*trkmom + 0.14*0.14);

      JSF3DV tmpmom = clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.);
      Double_t tmppt = TMath::Abs(tmpmom.X()*tmpmom.X()+tmpmom.Y()*tmpmom.Y());
      (TMath::Abs(tmppt/tmpmom.Z()) < 210./270. || tmppt < 105.*9.e-3) ? zmassec += energy : zmassba += energy;
      //(TMath::Abs(tmppt/tmpmom.Z()) < 210./270. || tmppt < 2.*210.*9.e-3) ? zmassec += energy : zmassba += energy;

      //////////////// cheating ///////////////////////////////////
      Int_t pid = 0;
      Double_t etrack = energy;
      Double_t epi    = 0.;
      Double_t egam   = 0.;
      Double_t ee     = 0.;
      Double_t ek     = 0.;
      Double_t ep     = 0.;
      Double_t en     = 0.;
      Double_t emu    = 0.;
      Double_t eck    = 0.;
      Double_t eother = 0.;
      std::map<Int_t, Double_t>::iterator piditr;
      for(piditr = (clitr->second->GetPIDMap()).begin(); piditr != (clitr->second->GetPIDMap()).end(); piditr++){
	
	pid = piditr->first;
	if(TMath::Abs(pid) == 211)      epi += piditr->second;
	else if(TMath::Abs(pid) == 22) egam += piditr->second;
	else if(TMath::Abs(pid) == 11)   ee += piditr->second;
	else if(TMath::Abs(pid) == 130)  ek += piditr->second;
	else if(TMath::Abs(pid) == 2212) ep += piditr->second;
	else if(TMath::Abs(pid) == 2112) en += piditr->second;
	else if(TMath::Abs(pid) == 13)  emu += piditr->second;
	else if(TMath::Abs(pid) == 321) eck += piditr->second;
	else  eother += piditr->second;
      }
      //////////////////////////////////////////////////////////////

      hdcetrack += etrack;
      hdcepi    += epi;
      hdcegam   += egam;
      hdcee     += ee;
      hdcek     += ek;
      hdcep     += ep;
      hdcen     += en;
      hdcemu    += emu;
      hdceck    += eck;
      hdceother += eother;

    }else if(ptype == kgMuon_p){
      nmu++;
      trkmom = (clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0)).Abs();
      Double_t energy = TMath::Sqrt(trkmom*trkmom + 0.1*0.1);

      JSF3DV tmpmom = clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.);
      Double_t tmppt = TMath::Abs(tmpmom.X()*tmpmom.X()+tmpmom.Y()*tmpmom.Y());
      (TMath::Abs(tmppt/tmpmom.Z()) < 210./270. || tmppt < 105.*9.e-3) ? zmassec += energy : zmassba += energy;

      emutrack += TMath::Sqrt(trkmom*trkmom + 0.1*0.1);

    }else if( (ptype == kgElectron_p) || (ptype == kgPhoton_p) ||(ptype == kgElectronPlusPhoton_p)){

      //////////////////// cheating //////////////////////////////////////
      Int_t pid = 0;
      Double_t epi    = 0.;
      Double_t egam   = 0.;
      Double_t ee     = 0.;
      Double_t ek     = 0.;
      Double_t ep     = 0.;
      Double_t en     = 0.;
      Double_t emu    = 0.;
      Double_t eother = 0.;
      Double_t eck    = 0.;
      std::map<Int_t, Double_t>::iterator piditr;

      for(piditr = (clitr->second->GetPIDMap()).begin(); piditr != (clitr->second->GetPIDMap()).end(); piditr++){

	pid = piditr->first;
	if(TMath::Abs(pid) == 211)      epi += piditr->second;
	else if(TMath::Abs(pid) == 22) egam += piditr->second;
	else if(TMath::Abs(pid) == 11)   ee += piditr->second;
	else if(TMath::Abs(pid) == 130)  ek += piditr->second;
	else if(TMath::Abs(pid) == 2212) ep += piditr->second;
	else if(TMath::Abs(pid) == 2112) en += piditr->second;
	else if(TMath::Abs(pid) == 13)  emu += piditr->second;
	else if(TMath::Abs(pid) == 321) eck += piditr->second;
	else  eother += piditr->second;
      }

      eemsum += clitr->second->GetEnergy();
      nem++;
      (clitr->second->GetParticleType() == kgElectron_p) ? nele++ : ngam++;

      (TMath::Abs((clitr->second->GetPosition()).Z()) < 270.) ?
	zmassba += clitr->second->GetEnergy():
	zmassec += clitr->second->GetEnergy();

      if(clitr->second->GetTrackPtr())
	eetrack += (clitr->second->GetTrackPtr()->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();

      eepi    += epi;
      eegam   += egam;
      eee     += ee;
      eek     += ek;
      eep     += ep;
      een     += en;
      eemu    += emu;
      eeck    += eck;
      eeother += eother;
    }else{ // neutral hadron
      nnhd++;
      (TMath::Abs((clitr->second->GetPosition()).Z()) < 270.) ?
	zmassba += clitr->second->GetEnergy():
	zmassec += clitr->second->GetEnergy();
      enhdsum += clitr->second->GetEnergy();

    }
  }

  // for total sum
  Double_t alletrack = 0.;
  Double_t allepi    = 0.;
  Double_t allegam   = 0.;
  Double_t allee     = 0.;
  Double_t allek     = 0.;
  Double_t allep     = 0.;
  Double_t allen     = 0.;
  Double_t allemu    = 0.;
  Double_t alleck    = 0.;
  Double_t alleother = 0.;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    Int_t pid = 0;
    Double_t epi    = 0.;
    Double_t egam   = 0.;
    Double_t ee     = 0.;
    Double_t ek     = 0.;
    Double_t ep     = 0.;
    Double_t en     = 0.;
    Double_t emu    = 0.;
    Double_t eck    = 0.;
    Double_t eother = 0.;
    std::map<Int_t, Double_t>::iterator piditr;

    if(itr->second->IsMuon()){
      emu = itr->second->GetEnergyDeposit();
      continue;
    }

    for(piditr = (itr->second->GetPIDMap()).begin(); piditr != (itr->second->GetPIDMap()).end(); piditr++){

      pid = piditr->first;
      if(TMath::Abs(pid) == 211)      epi += piditr->second;
      else if(TMath::Abs(pid) == 22) egam += piditr->second;
      else if(TMath::Abs(pid) == 11)   ee += piditr->second;
      else if(TMath::Abs(pid) == 130)  ek += piditr->second;
      else if(TMath::Abs(pid) == 2212) ep += piditr->second;
      else if(TMath::Abs(pid) == 2112) en += piditr->second;
      else if(TMath::Abs(pid) == 13)  emu += piditr->second;
      else if(TMath::Abs(pid) == 321) eck += piditr->second;
      else  eother += piditr->second;
    }

#ifdef __ECCHEAT__
    if(TMath::Abs(itr->second->GetPosition().Z()) < 270.){
      allepi  += epi;
      allep   += ep;
      alleck  += eck;
    }
#else
    allepi += epi;
    allep  += ep;
    alleck += eck;
#endif

    allegam += egam;
    allee   += ee;
    allek   += ek;
    allen   += en;
    allemu  += emu;
    alleother += eother;

  }

  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){
    alletrack += (titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();
  }
  
  //-------------------------------------

  ntrk = (Int_t) trkmap.size();
  
  JetNt->Fill(gJSF->GetEventNumber(),zmassba,zmassec,hdcetrack,eemsum,emutrack,enhdsum,util -> GetNeutrinoEnergy(),util -> GetMissingEnergy());
  emtotal -> Fill(gJSF->GetEventNumber(),zmassba,zmassec,eetrack,eepi,eegam,eee,eek,eep,een,eemu,eeck,eeother,allee,allegam);
  hdctotal -> Fill(//gJSF->GetEventNumber(),
		   zmassba,zmassec,hdcetrack,hdcepi,hdcegam,hdcee,hdcek,hdcep,hdcen,hdcemu,hdceck,hdceother,allepi,allep,alleck);
  

  ////////////////////////// comment out /////////////////////////////

  std::cout << "################ Results of evt " << gJSF->GetEventNumber()
	    << " ################" << std::endl;

  std::cout << " total energy = "
	    << zmassec+zmassba+(util->GetMissingEnergy()) << std::endl
	    << " zmassec = " << zmassec << ", zmassba = " << zmassba
	    << ", dis = "
	    << util->GetMissingEnergy() << std::endl
	    << " zmassec/(zmassec+zmassba) = " << zmassec/(zmassec+zmassba)
	    << std::endl
	    << " allee  = " << allee << ", allegam = " << allegam
	    << std::endl
	    << " allepi = " << allepi << ",   allep = " << allep
	    << " alleck = " << alleck
	    << std::endl << std::endl;

  std::cout << " purity list:" << std::endl
	    << "  (gamma   :: "
	    << 100.*eegam/(eee+eegam+eepi+eep+eeck+een+eek+eemu+eeother)
	    << " )"
	    << std::endl
	    << "  (electron:: "
	    << 100.*eee/(eee+eegam+eepi+eep+eeck+een+eek+eemu+eeother)
	    << " )"
	    << std::endl
	    << "       egam:: "
	    << 100.*(eee+eegam)/(eee+eegam+eepi+eep+eeck+een+eek+eemu+eeother)
	    << std::endl
	    << "        chd:: "
	    <<  100.*(hdcepi+hdcep+hdceck)/(hdcepi+hdcep+hdceck+hdcen+hdcek+hdcee+hdcegam+hdcemu+hdceother)
	    << std::endl << std::endl;

  std::cout << " efficiency list:" << std::endl;
  std::cout << "  (gamma   :: "
	    << 100.*eegam/allegam << " )"
	    << std::endl
	    << "  (electron:: "
	    << 100.*eee/allee << " )"
	    << std::endl
	    << "       egam:: "
	    << 100.*(eee+eegam)/(allee+allegam)
	    << std::endl
	    << "       chd:: "
	    << 100.*(hdcepi+hdcep+hdceck)/(allepi+allep+alleck)
	    << std::endl;
  std::cout << " number of clusters:" << std::endl
	    << "  nmu = " << nmu << ", nem = " << nem << ", nchd = " << nchd
	    << ", nnhd = " << nnhd << std::endl;
  std::cout << "####################################################"
	    << std::endl;
  ///////////////////////////////////////////////////////////////////////

  ////////////////////////////////////////////////////////////////////////////
  
  static TNtuple *justsum = new TNtuple("justsum","","nevt:eedep:hedep:totale:dise");

  Double_t tsum = 0.;
  Double_t eemraw = 0.;
  Double_t ehdraw = 0.;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    tsum += itr->second->GetEnergy();
    (itr->second->IsEM()) ? eemraw += itr->second->GetEnergyDeposit() : ehdraw += itr->second->GetEnergyDeposit();
  }
  justsum -> Fill(gJSF->GetEventNumber(), eemraw, ehdraw, tsum, util -> GetMissingEnergy());


  ///////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////////////


  delete util;
  delete mumaker;
  delete emmaker;
  delete chdmaker;
  delete nhdmaker;

  fDone = kTRUE;


#ifdef __USEHIT__
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    delete (itr->second);
  }
#endif

  //---------- sort & add real clusters --------------
#if 1
  std::multimap<Double_t,S4CALRealCluster*> tmpcls;
  for(clitr = clmap.begin(); clitr != clmap.end(); clitr++){
    Double_t energy = clitr->second->GetEnergy();
    tmpcls.insert(std::pair<Double_t,S4CALRealCluster*>(energy,clitr->second));
  }

  // reset RealClusterID
  std::multimap<Double_t,S4CALRealCluster*>::reverse_iterator rtmpitr;
  Int_t rclID = 0;
  for(rtmpitr = tmpcls.rbegin(); rtmpitr != tmpcls.rend(); rtmpitr++){
    rclID++;
    rtmpitr->second->SetClusterID(rclID);
    rtmpitr->second->Update();
    GetOutputBufPtr()->Add(rtmpitr->second);
  }
#else
  Int_t rclID = 0;
  for(clitr = clmap.begin(); clitr != clmap.end(); clitr++){
    rclID++;
    clitr->second->SetClusterID(rclID);
    clitr->second->Update();
    GetOutputBufPtr()->Add(clitr->second);
  }
#endif
  //--------------------------------------------------

#ifdef __UNLOCK__
  UnlockAll();
#endif

}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4CALRealClusterMakerBuf::DebugPrint(const Char_t *)
{
}

//==========================================================================
//* MakeKPIClusters --------------------------------------------------------

void S4CALRealClusterMakerBuf::MakeKPIClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap)
{

  if(sclmap.size() == 0) return;

  std::map<Int_t, S4CALSmallCluster*>::iterator itr;
  typedef std::multimap<MatchedType_t, std::vector<Double_t> > MULTIMAP;

  ///// Cheating method!! ///////////////////////////////////////
  //                                                           //

  //--- Lock kaon and pion origined small clusters ------
  static Int_t kpinumber = 0;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    if ( TMath::Abs((itr->second->GetPosition()).Z()) > 270. ){
      if( itr->second->IsKPI(0.5) ){
	kpinumber++;
	//std::cout << "Am I kaon or pion ??:: " << kpinumber << std::endl;
	itr->second->Lock();
      }
    }
  }
  //--------------------------------------------
#if 0
  //----- choose k/pi tracks by cheating method ---------
  std::map<Int_t,S4HYBTrack*>::iterator kpititr;
  Int_t kpitrkID = 100000;

  for(kpititr = trkmap.begin(); kpititr != trkmap.end(); kpititr++){

    if( TMath::Abs(kpititr->second->GetPDGCode()) == 211 || TMath::Abs(kpititr->second->GetPDGCode()) == 321 ){

      S4CALRealCluster *kpicl = new S4CALRealCluster();
      kpicl->AddTrackPtr(kpititr->second);
      kpicl->SetParticleType(kgMuon_p);
      kpititr->second->Lock();
      kpicl->Update();
      clmap.insert(std::pair<Int_t,S4CALRealCluster*>(kpicl->GetClusterID(), kpicl));
      kpitrkID++;
    }
  }
#endif
}

//==========================================================================
//* MakeGammaCheatedClusters -----------------------------------------------
void S4CALRealClusterMakerBuf::MakeGammaCheatedClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap)
{

#ifdef __USEHIT__
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;
    if((itr->second->GetPIDMap()).count(22) == 0) continue;

    Double_t egam = ((itr->second->GetPIDMap()).find(22))->second;
    if(egam/(itr->second->GetEnergyDeposit()) < 0.9) continue;

    S4CALRealCluster *real = new S4CALRealCluster();
    real -> Add(itr->second);
    real -> SetClusterType(kgEmChargedPlusEmNeutral_cl);
    real -> SetParticleType(kgElectronPlusPhoton_p);
    real->Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(real->GetClusterID(),real));
    itr->second->Lock();

  }
#endif

}

//==========================================================================
//* MakeECClusters ---------------------------------------------------------

 void S4CALRealClusterMakerBuf::MakeECClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap)
{

  // Note that you shold switch on the flag "__USEHIT__" if you use this
  // function.

#ifdef __USEHIT__
  std::vector<std::pair<S4CALRealCluster*,std::vector<S4CALSmallCluster*> > > tmpvec;
  std::map<Int_t,S4CALRealCluster*>::iterator tmpitr;
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  S4HYBTrackMakerBuf *hybmbp = dynamic_cast<S4HYBTrackMakerBuf*>( gJSF->FindModule("S4HYBTrackMaker")->EventBuf() );

  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){

    if(itr->second->IsLocked()) continue;

    S4CALRealCluster* real = new S4CALRealCluster();
    std::vector<S4CALSmallCluster*> invec;
    //------------------- make one cluster ----------------------
    S4CALExactHit *ehit = dynamic_cast<S4CALExactHit*>((itr->second->GetCALHitsVector())[0]->GetExactHitPtr());
    if(!ehit) continue;
    S4CALPreHit *prehitp = ehit -> GetPreHitPtr();
    Int_t pid   = 0;
    Int_t trkid = 0;

    if(prehitp -> GetPostHitPtr()){
      pid   = prehitp->GetPostHitID();
      trkid = prehitp->GetPostHitPtr()->GetTrackID();
    }else{
      pid   = prehitp->GetPreHitID();
      trkid = prehitp->GetTrackID();
    }

    Int_t pdg = TMath::Abs((itr->second->GetPIDMap()).begin()->first);
    EParticleType_t ptype = kgNotDecided_p;

    //------------- Add track ----------------
    S4HYBTrack *trkp = dynamic_cast<S4HYBTrack*> (hybmbp->GetHYBTrackPtr(trkid));
    if(trkp && !(trkp->IsLocked())){
      trkp->Lock();
      real->AddTrackPtr(trkp);
      if(pdg == 2212 || pdg == 211 || pdg == 321) ptype = kgHdCharged_p;
      else if(pdg == 11)                          ptype = kgElectron_p;
    }else{
      if(pdg == 22)                               ptype = kgPhoton_p;
      else if(pdg == 2112 || pdg == 130)          ptype = kgHdNeutral_p;
    }
    //----------------------------------------

    real -> SetParticleType(ptype);
    real -> Add(itr->second);
    invec.push_back(itr->second);
    itr->second->Lock();

    std::map<Int_t,S4CALSmallCluster*>::iterator citr;

    //----- collect small clusters -----------
    for(citr = sclmap.begin(); citr != sclmap.end(); citr++){

      if(citr->second->IsLocked()) continue;

      S4CALExactHit *ehit2 = dynamic_cast<S4CALExactHit*>((citr->second->GetCALHitsVector())[0]->GetExactHitPtr());
      if(!ehit2) continue;
      S4CALPreHit *prehitp2 = ehit2 -> GetPreHitPtr();
      Int_t pid2   = 0;
      if(prehitp2 -> GetPostHitPtr()) pid2 = prehitp2->GetPostHitID();
      else                            pid2 = prehitp2->GetPreHitID();

      if(pid2 == pid){
	citr->second->Lock();
	real -> Add(citr->second);
      }
    }
    //----------------------------------------

    //----------------------------------------------------------------

    real->Update();
    //    static const Double_t eccalz   = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
    static const Double_t eccalz   = S4CALAddress::GetBarrelHalfZ();
    if(TMath::Abs(real->GetPosition().Z()) < eccalz){
      tmpvec.push_back(std::pair<S4CALRealCluster*, std::vector<S4CALSmallCluster*> >(real,invec));
    }else{
      clmap.insert(std::pair<Int_t,S4CALRealCluster*>(real->GetClusterID(),real));
    }
  }

  //----- release cheated clusters in the barrel region --------
  for(Int_t k = 0; k < (Int_t) tmpvec.size(); k++){
    if(tmpvec[k].first->GetTrackPtr()) tmpvec[k].first->GetTrackPtr()->Unlock();
    std::vector<S4CALSmallCluster*> vec = tmpvec[k].second;
    for(Int_t i = 0; i < (Int_t) vec.size(); i++) vec[i] -> Unlock();
    delete (tmpitr->second);
  }
  //------------------------------------------------------------

#endif

}

#ifdef __USEHIT__
//==========================================================================
//* MakeSmallClusterMap ----------------------------------------------------

std::map<Int_t, S4CALSmallCluster*> S4CALRealClusterMakerBuf::MakeSmallClusterMap(Int_t nhitscut)
{

  typedef std::pair<Int_t,S4CALSmallCluster*> SCLPAIR;
  std::map<Int_t, S4CALSmallCluster*> sclmap;

  Int_t sclID = 0;

  S4CALHitMaker *csclm=(S4CALHitMaker*)gJSF->FindModule("S4CALHitMaker");
  S4CALHitBuf   *csclmb=(S4CALHitBuf*)csclm->EventBuf();
  TObjArray  *cscl = csclmb->GetOutputBufPtr();

  TIter next(cscl);
  S4CALHit *calscl = 0;

  while( (calscl = (S4CALHit*)next()) ){

    //if(calscl->IsLocked()) continue;
    sclID = calscl -> GetCellID();
    S4CALSmallCluster* test = new S4CALSmallCluster();
    test->Add(calscl);
    test->SetClusterID(sclID);
    test->Update();
    sclmap.insert(SCLPAIR(sclID, test));

  }

  std::cout << "cscl entries = " << cscl->GetEntries()
	    << ", map size = " << sclmap.size() << std::endl;

  return sclmap;

}
#else
//==========================================================================
//* MakeSmallClusterMap ----------------------------------------------------

std::map<Int_t, S4CALSmallCluster*> S4CALRealClusterMakerBuf::MakeSmallClusterMap(Int_t nhitscut)
{

  typedef std::pair<Int_t,S4CALSmallCluster*> SCLPAIR;
  std::map<Int_t, S4CALSmallCluster*> sclmap;

  Int_t sclID = 0;

  S4CALSmallClusterMaker *csclm=(S4CALSmallClusterMaker*)gJSF->FindModule("S4CALSmallClusterMaker");
  S4CALSmallClusterMakerBuf *csclmb=(S4CALSmallClusterMakerBuf*)csclm->EventBuf();
  TObjArray  *cscl = csclmb->GetOutputBufPtr();

  TIter next(cscl);
  S4CALSmallCluster *calscl = 0;

  while( (calscl = (S4CALSmallCluster*)next()) ){

    if(calscl->IsLocked()) continue;
    if(calscl->GetNHits() > nhitscut){
      sclID = calscl -> GetClusterID();
      sclmap.insert(SCLPAIR(sclID, calscl));
    }
  }

  return sclmap;

}
#endif
//==========================================================================
//* MakeTrackMap  ----------------------------------------------------------

std::map<Int_t, S4HYBTrack*> S4CALRealClusterMakerBuf::MakeTrackMap()
{

  std::map<Int_t, S4HYBTrack*> trkmap;

  S4HYBTrackMakerBuf *hybmbp = dynamic_cast<S4HYBTrackMakerBuf*>( gJSF->FindModule("S4HYBTrackMaker")->EventBuf() );
  TObjArray *outputbufp = hybmbp -> GetOutputBufPtr();

  TIter nexttrack(outputbufp);
  S4HYBTrack *trackp = 0;

  //--- Lock mother tracks if any -------
  while ((trackp = dynamic_cast<S4HYBTrack *>(nexttrack()))) {
    S4HYBTrack *mtrack = dynamic_cast<S4HYBTrack*>(hybmbp->GetHYBTrackPtr(trackp->GetMotherTrackID()));
    if(mtrack) mtrack->Lock();
  }
  nexttrack.Reset();
  //-------------------------------------

  while ((trackp = dynamic_cast<S4HYBTrack *>(nexttrack()))) {

    if (trackp->IsLocked()) continue;

    trkmap.insert(std::pair<Int_t,S4HYBTrack*>(trackp->GetTrackID(), trackp));

  } // end of while

  return trkmap;

}

//==========================================================================
//* UnlockAll --------------------------------------------------------------

void S4CALRealClusterMakerBuf::UnlockAll()
{

  //================ Unlock S4HYBTrack objects ====================
  S4HYBTrackMakerBuf *hybmbp = dynamic_cast<S4HYBTrackMakerBuf*>( gJSF->FindModule("S4HYBTrackMaker")->EventBuf() );
  TObjArray *outputbufp = hybmbp -> GetOutputBufPtr();

  TIter nexttrack(outputbufp);
  S4HYBTrack *trackp = 0;

  while ((trackp = dynamic_cast<S4HYBTrack *>(nexttrack()))) {

    trackp->Unlock();
    if(!trackp->GetHelicalTrackPtr()) trackp->Lock();

  }
  //===============================================================

  //================= Unlock S4CALHit objects =====================
  S4CALHitMaker *chm=(S4CALHitMaker*)gJSF->FindModule("S4CALHitMaker"); 
  S4CALHitBuf   *chb=(S4CALHitBuf*)chm->EventBuf();                   
  TObjArray     *chts=chb->GetHits();

  TIter next(chts);
  S4CALHit *calhitp = 0;

  while( (calhitp = (S4CALHit*)next()) )    calhitp->Unlock();
  //===============================================================

}

//=====================================================================
//* MakeCheatedRealClusters -------------------------------------------

void S4CALRealClusterMakerBuf::MakeCheatedRealClusters()
{

  if (fDone) return; // check whether Real clusters are already made

  fDone = kTRUE;
  fOutputBufPtr->SetOwner();        // buffer owns Real clusters 

  //====================================================
  // make small cluster map & track map
  std::map<Int_t,S4CALSmallCluster*> sclmap = MakeSmallClusterMap(0);
  std::map<Int_t,S4CALSmallCluster*>::iterator itr;
  std::map<Int_t,S4HYBTrack*> trkmap = MakeTrackMap();
  if(sclmap.size() == 0){
    std::cout << " There is no small cluster!" << std::endl;
    return;
  }
  //----------------------------------------------------
  //====================================================


  //====================================================
  // make clusters
  
  std::map<Int_t,S4CALRealCluster*> clmap;  // temporary clusters list
  std::map<Int_t,S4CALRealCluster*>::iterator clitr;

  S4HYBTrackMakerBuf *hybmbp = dynamic_cast<S4HYBTrackMakerBuf*>( gJSF->FindModule("S4HYBTrackMaker")->EventBuf() );

  //-------- temporary small cluster map ---------
  Int_t l = 0;
  std::map<Int_t,S4CALSmallCluster*> tmpscls;
  for(itr = sclmap.begin(); itr != sclmap.end(); itr++){
    std::vector<S4CALHit*> hitv = itr->second->GetCALHitsVector();
    for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
      l++;
      S4CALSmallCluster* scl = new S4CALSmallCluster();
      scl->Add(hitv[k]);
      scl->Update();
      tmpscls.insert(std::pair<Int_t,S4CALSmallCluster*>(l,scl));
    }
  }
  //----------------------------------------------
  for(itr = tmpscls.begin(); itr != tmpscls.end(); itr++){

    if(itr->second->IsLocked()) continue;

    //------------------- make one cluster ----------------------
    S4CALRealCluster* real = new S4CALRealCluster();
    EParticleType_t ptype = kgNotDecided_p;

    std::map<Int_t,std::pair<Int_t,Double_t> > postmap = itr->second->GetPostHitIDMap();
    std::map<Int_t,std::pair<Int_t,Double_t> >::iterator pitr;
    Int_t maxpid   = 0;
    Int_t maxtrkid = 0;
    Double_t maxe  = 0.;

    for(pitr = postmap.begin(); pitr != postmap.end(); pitr++){
      if((pitr->second).second > maxe){
	maxe     = (pitr->second).second;
	maxpid   = (pitr->second).first;
	maxtrkid = pitr->first;
      }
    }
    Int_t pdg = TMath::Abs(maxpid);
    //------------- Add track ----------------
    S4HYBTrack *trkp = dynamic_cast<S4HYBTrack*> (hybmbp->GetHYBTrackPtr(maxtrkid));
    if(trkp && !(trkp->IsLocked()) && trkp->GetHelicalTrackPtr()){
      trkp->Lock();
      real->AddTrackPtr(trkp);
      if(pdg == 2212 || pdg == 211 || pdg == 321) ptype = kgHdCharged_p;
      else if(pdg == 11)                          ptype = kgElectron_p;
    }else{
      if(pdg == 22)                               ptype = kgPhoton_p;
      else if(pdg == 2112 || pdg == 130)          ptype = kgHdNeutral_p;
    }
    //----------------------------------------

    real->SetParticleType(ptype);
    real->Add(itr->second);
    itr->second->Lock();

    std::map<Int_t,S4CALSmallCluster*>::iterator citr;

    //----- collect small clusters -----------
    for(citr = tmpscls.begin(); citr != tmpscls.end(); citr++){

      if(citr->second->IsLocked()) continue;

      postmap = citr->second->GetPostHitIDMap();
      Int_t maxpid2   = 0;
      Int_t maxtrkid2 = 0;
      Double_t maxe2  = 0.;

      for(pitr = postmap.begin(); pitr != postmap.end(); pitr++){
	if((pitr->second).second > maxe2){
	  maxe2     = (pitr->second).second;
	  maxpid2   = (pitr->second).first;
	  maxtrkid2 = pitr->first;
	}
      }

      if(maxtrkid2 == maxtrkid){
	citr->second->Lock();
	real -> Add(citr->second);
      }
    }
    //----------------------------------------

    //----------------------------------------------------------------

    real->Update();
    clmap.insert(std::pair<Int_t,S4CALRealCluster*>(real->GetClusterID(),real));

  }
  //--------- delte temporary small clusters -----------
  for(itr = tmpscls.begin(); itr != tmpscls.end(); itr++)
    delete itr->second;
  //----------------------------------------------------


  //---------- sort & add real clusters --------------
  std::multimap<Double_t,S4CALRealCluster*> tmpcls;
  for(clitr = clmap.begin(); clitr != clmap.end(); clitr++){
    Double_t energy = clitr->second->GetEnergy();
    tmpcls.insert(std::pair<Double_t,S4CALRealCluster*>(energy,clitr->second));
  }
  std::multimap<Double_t,S4CALRealCluster*>::reverse_iterator rtmpitr;
  Int_t rclID = 0;
  for(rtmpitr = tmpcls.rbegin(); rtmpitr != tmpcls.rend(); rtmpitr++){
    rclID++;
    rtmpitr->second->SetClusterID(rclID);
    rtmpitr->second->Update();
    GetOutputBufPtr()->Add(rtmpitr->second);
  }
  //--------------------------------------------------

#ifdef __UNLOCK__
  UnlockAll();
#endif

}
