//*************************************************************************
//* ---------------------
//* S4CALTrackMatcher Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALSmallClusterUtils.h"
#include "S4CALTrackMatcher.h"
#include "JSFHelicalTrack.h"
#include "JSFSteer.h"
#include "TKalTrackSite.h"
#include "S4CALAddress.h"
#include "S4CALHit.h"
#include "S4CALClusterUtils.h"

#include "TNtuple.h"
#include <iostream>


ClassImp(S4CALTrackMatcher)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALTrackMatcher::S4CALTrackMatcher()
  :fMinNHits(4)
{
  //  static const Double_t calRin = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  //  static const Double_t emabsth = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  //  static const Double_t emactth = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  //  static const Int_t nemlayer = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);

  static const Double_t calRin  = S4CALAddress::GetBarrelInnerR();
  static const Double_t emabsth = S4CALAddress::GetEMAbsLayerThickness();
  static const Double_t emactth = S4CALAddress::GetEMActiveLayerThickness();
  static const Int_t nemlayer   = S4CALAddress::GetEMNLayers();

  fEMOuterR = calRin + nemlayer*(emabsth+emactth);

}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALTrackMatcher::~S4CALTrackMatcher()
{
}


//==========================================================================
//* MinLayerMatching -------------------------------------------------------

void S4CALTrackMatcher::TrackMinLayerMatching(S4CALSmallCluster *scls, S4HYBTrack *trk, Double_t distance)
{

  //  static const Double_t calinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  //  static const Double_t calinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);

  static const Double_t calinR = S4CALAddress::GetBarrelInnerR();
  static const Double_t calinZ = S4CALAddress::GetBarrelHalfZ();

  Double_t trkmatch = 1.e+10;

#if 1
  //--- check tracks if it is already matched -----
  if((scls->GetPFOMap()).count(kgTrackMinLayer) > 0){
    typedef std::multimap<MatchedType_t, std::vector<Double_t> >::iterator MATCHITR;
    std::pair<MATCHITR,MATCHITR> itr_range;
    itr_range = (scls->GetPFOMap()).equal_range(kgTrackMinLayer);
    MATCHITR mitr;
    for(mitr = itr_range.first; mitr != itr_range.second; mitr++){
      if(trk->GetTrackID() == (Int_t) (mitr->second)[0]){
	(scls->GetPFOMap()).erase(mitr);
	break;  // necessary!
      }
    }
  }
  //-----------------------------------------------
#else
  if(scls->GetPFOMap()->count(kgTrackMinLayer) > 0){
    (scls->GetPFOMap()).erase(kgTrackMinLayer);
  }
#endif

  S4CALClusterUtils util;
  std::vector<TVector3> posmom = S4CALSmallClusterUtils::GetInTrack(trk,calinR,calinZ);

  if(posmom.size() < 1) return;

  trkmatch = ( posmom[0] - (scls->GetMinLayerPos()) ).Mag();

  if ( trkmatch < distance ){
    std::vector<Double_t> tvector;
    tvector.push_back(trk->GetTrackID());
    tvector.push_back(trkmatch);
    tvector.push_back(distance);
    scls->AddPFOMap(kgTrackMinLayer, tvector);
  }

}


//==========================================================================
//* TrackTubeMatching ----------------------------------------------------

void S4CALTrackMatcher::TrackTubeMatching(S4CALSmallCluster *scls, S4HYBTrack *trk, Double_t radius, Double_t slopeangle)
{

  if(trk->IsLocked()) return;

  Double_t trkmatch = 0.;

  //--- check tracks if it is already matched -----
  if((scls->GetPFOMap()).count(kgTrackTube) > 0){
    typedef std::multimap<MatchedType_t, std::vector<Double_t> >::iterator MATCHITR;
    std::pair<MATCHITR,MATCHITR> itr_range;
    itr_range = (scls->GetPFOMap()).equal_range(kgTrackTube);
    MATCHITR mitr;
    for(mitr = itr_range.first; mitr != itr_range.second; mitr++){
      if(trk->GetTrackID() == (Int_t) (mitr->second)[0]){
	(scls->GetPFOMap()).erase(mitr);
	break;  // necessary!
      }
    }
  }
  //-----------------------------------------------

  Double_t cutR = radius;
#if 0
  if( !(scls->IsEM()) )
    cutR += (sclsposrho -  fEMOuterR)*TMath::Tan(slopeangle);
#endif

  S4CALClusterUtils util;
  trkmatch = S4CALSmallClusterUtils::CalcTrackDistance(trk, scls->GetPosition());

  if ( trkmatch < cutR ){
    Double_t trkmom = (trk->GetHelicalTrackPtr()->GetMomentum(0.)).Abs();
    std::vector<Double_t> tvector;
    tvector.push_back(trk->GetTrackID());
    tvector.push_back(trkmatch);   // temporary
    tvector.push_back(trkmom);
    tvector.push_back(radius);
    tvector.push_back(slopeangle);

    //---- Sort & Add --------------------------
#if 0

    if((scls->GetPFOMap()).count(kgTrackTube) > 0){ // if already exists

      typedef std::multimap<MatchedType_t,std::vector<Double_t> > TMP;
      TMP tmpmap;
      tmpmap = scls->GetPFOMap();
      (scls->GetPFOMap()).erase(kgTrackTube);

      TMP::iterator tmpitr;
      std::pair<TMP::iterator,TMP::iterator> tmppair = tmpmap.equal_range(kgTrackTube);
      Bool_t finished;
      finished = kFALSE;
      for(tmpitr = tmppair.first; tmpitr != tmppair.second; tmpitr++){
	if(finished || (tvector[1] > (tmpitr->second)[1]) ){
	  scls->AddPFOMap(kgTrackTube, tmpitr->second);
	}else{
	  scls->AddPFOMap(kgTrackTube, tvector);
	  scls->AddPFOMap(kgTrackTube, tmpitr->second);
	  finished = kTRUE;
	}
      }

    }else{
      scls->AddPFOMap(kgTrackTube, tvector);
    }

#else
    scls->AddPFOMap(kgTrackTube, tvector);
#endif

    //-------------------------------------------
  }

}


//==========================================================================
//* TrackThrustMatching ----------------------------------------------------

void S4CALTrackMatcher::TrackThrustMatching(S4CALSmallCluster *scls, S4HYBTrack *trk, Double_t cutperp)
{

  Double_t trkmatch = 0.;
  Double_t sclsenergy = 0.;
  TVector3 sclspos(0.,0.,0.);

  //--- check tracks if it is already matched -----
  if((scls->GetPFOMap()).count(kgTrackThrust) > 0){
    typedef std::multimap<MatchedType_t, std::vector<Double_t> >::iterator MATCHITR;
    std::pair<MATCHITR,MATCHITR> itr_range;
    itr_range = (scls->GetPFOMap()).equal_range(kgTrackThrust);
    MATCHITR mitr;
    for(mitr = itr_range.first; mitr != itr_range.second; mitr++){
      if(trk->GetTrackID() == (Int_t) (mitr->second)[0]){
	(scls->GetPFOMap()).erase(mitr);
	break;
      }
    }
  }
  //-----------------------------------------------

  //if(scls->GetNHits() < fMinNHits) return;
  if(scls->GetThrust() < 1.e-4) return;

  sclspos    = scls -> GetPosition();
  sclsenergy = scls -> GetEnergy();

  S4CALClusterUtils util;
  std::vector<TVector3> posmom = S4CALSmallClusterUtils::GetInTrack(trk);

  if(posmom.size() < 1) return;

  trkmatch = (posmom[0]-sclspos).Perp(scls->GetThrustVector()); // temporary
  //Double_t trkmatchsq = trkmatch*trkmatch*sclsenergy;
  //Double_t trkmatch2  = trkmatchsq*sclsenergy;

  if ( trkmatch < cutperp ){
    std::vector<Double_t> tvector;
    tvector.push_back(trk->GetTrackID());
    tvector.push_back(trkmatch);
    tvector.push_back(cutperp);
    scls->AddPFOMap(kgTrackThrust, tvector);
  }

}

//==========================================================================
//* TrackECMatching --------------------------------------------------------

void S4CALTrackMatcher::TrackECMatching(S4CALSmallCluster *scls, S4HYBTrack *trk, Double_t cut)
{

  // temporary!

  //  static const Double_t calinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  //  static const Double_t calinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);

  static const Double_t calinR = S4CALAddress::GetBarrelInnerR();
  static const Double_t calinZ = S4CALAddress::GetBarrelHalfZ();

  //---- check if small cluster is in (nearby) endcap region ------
  //if( (TMath::Abs((scls->GetPosition()).Z()) < calinZ) ) return;
  if( (TMath::Abs((scls->GetPosition()).Z()) < calinZ - 10.) ) return;
  //---------------------------------------------------------------

  Double_t phi  = 0.;
  Double_t phi1 = 0.;
  Double_t startx = (trk->GetHelicalTrackPtr()->GetCoordinate(0.)).X();
  Double_t starty = (trk->GetHelicalTrackPtr()->GetCoordinate(0.)).Y();

  //------ check track if it reach to endcap region ------------
  Int_t ir = trk->GetHelicalTrackPtr()->OriginToCylinder(calinR, calinZ,
							 phi, phi1, 1000,
							 startx, starty);
  if(ir != 1) return; // track intersect with barrel region
  //------------------------------------------------------------

  ir = trk->GetHelicalTrackPtr()->OriginToCylinder(calinR,
						   TMath::Abs((scls->GetPosition()).Z()),
						   phi, phi1, 1000,
						   startx, starty);

  JSF3DV tmppos = trk->GetHelicalTrackPtr()->GetCoordinate(phi1);
  JSF3DV tmpmom = trk->GetHelicalTrackPtr()->GetMomentum(phi1);

  TVector3 trkpos(tmppos.X(),tmppos.Y(),tmppos.Z());
  TVector3 trkmom(tmpmom.X(),tmpmom.Y(),tmpmom.Z());
  TVector3 sclspos = scls->GetPosition();

  //--------------- check the track direction ------------------
  if( (trkmom.Z())*(sclspos.Z()) < 0. )  return;
  //------------------------------------------------------------

  Double_t k = (sclspos - trkpos)*(trkmom*(1/trkmom.Mag()));
  Double_t trkmatch = (sclspos - (k*(trkmom*(1/trkmom.Mag())) + trkpos)).Mag();

  if ( trkmatch < cut ){
    std::vector<Double_t> tvector;
    tvector.push_back(trk->GetTrackID());
    tvector.push_back(trkmatch);   // temporary
    tvector.push_back(trkmom.Mag());
    tvector.push_back(cut);

    scls->AddPFOMap(kgTrackEC, tvector);
  }

}
