#ifndef __S4CDCHITWire__
#define __S4CDCHITWire__

//*************************************************************************
//* ---------------------
//* S4CDCHitWire Class
//* ---------------------
//* (Description)
//*	Class to store "S4CDCHit"s for a single wire.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCHitWire		: Array containing "S4CDCHit"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

class U4CDCHitCell;
class S4CDCHit;

#include "U4CDCHitWire.h"

#define __TMP__

class S4CDCHit;
class S4CDCHitWire : public U4CDCHitWire {

public:

  S4CDCHitWire(U4CDCTimeToXRelation *calib = 0);
  virtual ~S4CDCHitWire(); 
  
  virtual Int_t		Append(S4CDCHit *hit);
  virtual void 		DebugPrint(const Char_t *opt = "Brief");
  
  virtual void 		KillHits ();
  inline virtual void 	Revive() 	 { fKilled = kFALSE; }
  inline virtual Bool_t	IsKilled() const { return fKilled; }
  
  static void 		SetTwoHitSeparation(Double_t sep) 
  				{ fgTwoHitSeparation = sep; }  
  static void 		SetTwoHitSeparationZ(Double_t sepz) 
  				{ fgTwoHitSeparationZ = sepz; } 
  static void 		TPCMode(Bool_t mode = kTRUE) { fgTPCMode = mode; }
  				

public:
   enum {
      kEBelowThreshold = 0x00010000,    // Two small pulse height.
      kECannotSeparate = 0x00020000,    // Hit is killed becase of 2-hit dist.
      kEDeadChannel    = 0x00040000,    // Dead channel.
      kENoisyChannel   = 0x00080000,    // Noisy channel.
      kEModifiedByDeltaRay = 0x00100000     // Hit is modified because of DeltaRay.
   };
   
private:
  static void ChannelToWire(S4CDCHit *hit,
  			    Int_t &l, Int_t &c, Int_t &w, 
  			    Double_t &x, Double_t &y);
private:
  static Double_t 	fgTwoHitSeparation;	//
  static Double_t 	fgTwoHitSeparationZ;	//
  static Bool_t 	fgTPCMode;		//
  static Double_t       f2Tkpar[2];             //
  			    
  Bool_t fKilled;		// flag to tell hits are killed or not.

  ClassDef(S4CDCHitWire, 1) 	// CDC Hit Wire
};

#endif

