#ifndef __S4CDCTRACK__
#define __S4CDCTRACK__

//*************************************************************************
//* ---------------------
//* S4CDCTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4CDCHitCell"s for a single layer.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCTrack		: Array containing "S4CDCHitCell"s
//* (Update Record)
//*	1999/12/04  K.Fujii	 Original version.
//*     2003/0X/0X  Y.Nakashima  Modified for Kalman filter.
//*     2004/08/08  K.Fujii      Now inherits from U4CDCTrack.
//*************************************************************************

#include "S4CDCVectorHit.h"
#include "JSFHelicalTrack.h"
#include "S4VTrack.h"
#include "S4CDCTrackMaker.h"
#include "U4CDCTrack.h"

class S4CDCVectorHit;

class S4CDCTrack : public U4CDCTrack, public S4VTrack {
public:
  S4CDCTrack(S4CDCTrackMaker *trkm = 0);
  virtual ~S4CDCTrack();

  inline virtual S4CDCTrack *GetMotherTrackPtr() const { return fMotherTrackPtr; }
  inline virtual Int_t       GetPDGCode();
  inline virtual Int_t       GetTrackID();
  inline virtual Int_t       GetMotherTrackID();

  inline virtual void        SetMotherTrackPtr(S4CDCTrack *trackp) { fMotherTrackPtr = trackp; }

         virtual Int_t       Append(S4CDCVectorHit* vectorhit);

         virtual Bool_t      IsSortable() const;
         virtual Int_t       Compare(const TObject* obj) const;

         virtual void        DebugPrint(const Char_t* opt = "Brief");
  
  static         void        SetT0pedestal(Double_t t0 = 0)  // in [s]
                                  { S4CDCHit::SetT0pedestal(t0); }

private:
         S4CDCTrack          *fMotherTrackPtr;     // ptr to mother track
 
  ClassDef(S4CDCTrack, 1)                     // CDC Track
};

//-----------------------------------------------------------
//  inline function
//-----------------------------------------------------------

Int_t S4CDCTrack::GetPDGCode()
{
   TObjArray* vectorhit = (TObjArray*)((TObjArray *)this)->operator[](0);
   TObject*  trackhit  = vectorhit->operator[](0);
   Int_t pdgcode = ((S4CDCTrackHit *)trackhit)->GetPDGCode();
   return pdgcode;
}

Int_t S4CDCTrack::GetTrackID()
{
   TObjArray* vectorhit = (TObjArray*)((TObjArray *)this)->operator[](0);
   TObject*  trackhit  = vectorhit->operator[](0);
   Int_t trackid = ((S4CDCTrackHit *)trackhit)->GetTrackID();
   return trackid;
}

Int_t S4CDCTrack::GetMotherTrackID()
{
   TObjArray* vectorhit = (TObjArray*)((TObjArray *)this)->operator[](0);
   TObject*  trackhit  = vectorhit->operator[](0);
   Int_t mtrackid = ((S4CDCTrackHit *)trackhit)->GetExactHitPtr()->GetMotherTrackID();
   return mtrackid;
}

#endif
