//*************************************************************************
//* --------------------
//* S4CDCTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2001/08/22  K.Hoshina	Original version.
//*************************************************************************

#include <iostream>
#include "S4CDCTrackMakerBuf.h"
#include "S4CDCHitMakerBuf.h"
#include "S4CDCExactHit.h"
#include "S4CDCTrack.h"
#include "S4CDCVectorHit.h"

//#define __DEBUG__

ClassImp(S4CDCTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4CDCTrackMakerBuf::S4CDCTrackMakerBuf(const char* name,
				       const char *title, 
				       S4CDCTrackMaker *module,
	                               Int_t bufsize)
		 :U4CDCTrackMakerBuf(name, title, module, bufsize)
{
}	       
	       
S4CDCTrackMakerBuf::S4CDCTrackMakerBuf(S4CDCTrackMaker *module,
			           const char *name,
	                           const char *title, 
	                           Int_t bufsize)
		 :U4CDCTrackMakerBuf(module, name, title, bufsize)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------

S4CDCTrackMakerBuf::~S4CDCTrackMakerBuf()
{
}

//=====================================================================
//* MakeTracks ----------------------------------------------------------

void S4CDCTrackMakerBuf::MakeTracks()
{
	if (!IsInitialized()) {
		std::cerr << "ERROR  S4CDCTrackMakerBuf::MakeTracks" << std::endl
		     << "...... cluster data not yet initialized." << std::endl;
		return;
	} else if (fDone) {
		return;
	}
	
	fDone = kTRUE;
	
	TObjArray* inputbufp 
		= (TObjArray *)((S4CDCHitMakerBuf*)fInputDataBufPtr)->GetInputTObjArrayPtr();
#ifdef __DEBUG__
	std::cerr << " S4CDCTrackMakerBuf::MakeTracks: Nhits of inputbuf = " 
	     << inputbufp->GetEntries() << std::endl;
#endif	
	S4CDCExactHit::SetSortBy("TrackNumber");
	inputbufp->UnSort();
	inputbufp->Sort();
	
	TObjArray tracktmp(2000);	
	
	Int_t tracklast = -9999;
		
	TIter next(inputbufp);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
		Int_t trackcur = exacthitp->GetTrackID();
		if ( trackcur != tracklast ) {
			if ( tracktmp.GetEntries() > 0 ) {
				
				if (tracktmp.GetEntries() > 2) {
					AppendTrack( &tracktmp, &fOutputBuf);
				}
#ifdef __DEBUG__
				std::cerr << "S4CDCTrackMakerBuf::MakeTracks: trackNo = "
				     << tracklast << " NEntries = " 
				     << tracktmp.GetEntries() << std::endl;
#endif
				tracktmp.Clear();
			}
			tracklast = trackcur;
		} 
		tracktmp.Add ( exacthitp );
	}
		
	if ( tracktmp.GetEntries() > 0 ) {
		AppendTrack( &tracktmp, &fOutputBuf);
		tracktmp.Clear();
#ifdef __DEBUG__
        	std::cerr << " S4CDCTrackMakerBuf::MakeTrack " << std::endl;
#endif
	}
	
#ifdef __DEBUG__
//	std::cerr << " S4CDCTrackMakerBuf::MakeTracks: n = " << i << std::endl;
#endif
	
}

//=====================================================================
//* AppendTrack -------------------------------------------------------

void S4CDCTrackMakerBuf::AppendTrack(TObjArray* tracktmpp, TObjArray* outputbufp)
{
	
	S4CDCTrack  *trackp = new S4CDCTrack((S4CDCTrackMaker *)Module());
	TObjArray    celltmp(1000);
	
#if 0
	S4CDCExactHit::SetSortBy("CellID");
#else
	S4CDCExactHit::SetSortBy("Time"); // change by nakashi 031225
#endif
	tracktmpp->UnSort();
	tracktmpp->Sort();
	
	Int_t celllast = -9999;
	Int_t wirelast = -9999;
		
	TIter next(tracktmpp);
	S4CDCExactHit *exacthitp;
	S4CDCExactHit *lastexacthitp = 0;
	
	while ((exacthitp = (S4CDCExactHit *)next())) {
	
		Int_t cellcur = exacthitp->GetCellID();
		Int_t wirecur = exacthitp->GetWireNumber();
		if ( cellcur != celllast ) {
			if ( celltmp.GetEntries() > 0 ) {
				AppendVectorHit( &celltmp, trackp );
				celltmp.Clear();
			}
			celllast = cellcur;
		} 
#if 1 // change by nakashi
                if ( (wirecur == wirelast) && (wirecur == 0) ) {
			AppendVectorHit( &celltmp, trackp );
			celltmp.Clear();
                        break;
                } 
#endif
		celltmp.Add ( exacthitp );
		lastexacthitp = exacthitp;
		wirelast = wirecur;
	}
	
	if ( celltmp.GetEntries() > 0 ) {
		AppendVectorHit( &celltmp, trackp );
		celltmp.Clear();
	}
	
	if ( trackp->GetEntries() > 0 ) {
		Int_t mothertrackN = lastexacthitp->GetMotherTrackID();
	        S4CDCTrack *mothertrackp = FindTrackByNo(mothertrackN);
	        trackp->SetMotherTrackPtr(mothertrackp);
		outputbufp->Add(trackp);
	} else {
		delete trackp;
	}	
	
}

//=====================================================================
//* AppendVectorHit ---------------------------------------------------

void S4CDCTrackMakerBuf::AppendVectorHit(TObjArray* celltmpp, S4CDCTrack* trackcurp)
{

	Int_t 		trackno = -9999;
	Int_t 		cellid  = -9999;
	S4CDCVectorHit*	vectorhitp  = 0;	
	
	TIter next(celltmpp);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
	    if (!vectorhitp) {
		vectorhitp  = new S4CDCVectorHit();
	    }
	    S4CDCTrackHit* hitp = new S4CDCTrackHit(exacthitp);
	    
	    if (vectorhitp->GetEntries() == 0) {           // 1st hit
		trackno = hitp->GetTrackID();
		cellid  = hitp->GetCellID();			
		vectorhitp->Append(hitp);
		trackcurp->Append(vectorhitp);
	    } else {
		if (hitp->GetTrackID() == trackno) {
		    if (hitp->GetCellID() == cellid) {
			vectorhitp->Add(hitp);
		    } else {
		    	std::cerr << " S4CDCTrackMakerBuf::AppendVectorHit:LogicError!!"
			     << " new Cell not expected." << std::endl;
		    }	   
		} else { 
		   std::cerr << " S4CDCTrackMakerBuf::AppendVectorHit:LogicError!!"
			<< " new Track not expected." << std::endl;	   
		}
	    }
	}
	
	if (vectorhitp && !vectorhitp->IsKilled()) {
#ifdef __DEBUG__
	   vectorhitp->DebugPrint();
#endif
	   vectorhitp->KillHits();
	   if (vectorhitp->TestBit(S4CDCVectorHit::kELooping)) {
	       trackcurp->Remove(vectorhitp);
	       delete vectorhitp;
	   }
	}
}

S4CDCTrack * S4CDCTrackMakerBuf::FindTrackByNo(Int_t trackno)
{
        TIter next(&fOutputBuf);
        S4CDCTrack *trackp = 0;
        while((trackp = (S4CDCTrack *)next())){
            Int_t trackN = trackp->GetTrackNumber();
            if(trackno == trackN) return trackp;
        }
//      std::cerr<< " Not Find TrackNumber = " << trackno << std::endl;
        return trackp;
}
