#ifndef __S4JETMAKERBUF__
#define __S4JETMAKERBUF__

//*************************************************************************
//* ---------------------
//* S4JetMakerBuf Class
//* ---------------------
//* (Description)
//*	This set of classes makes Jets. 
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	S4JetMakerBuf		: buffering class
//* (Update Record)
//*	2004/10/05  S.Yamamoto	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "S4JetMaker.h"
#include "ANLJetFinder.h"

class S4JetSet;

class S4JetMakerBuf : public JSFEventBuf {
friend class S4JetMaker;

public:
   S4JetMakerBuf(S4JetMaker   *mod     = 0,
                 const Char_t *name    = "S4JetMakerBuf",
                 const Char_t *title   = "Jet Buffer", 
                       Int_t   bufsize = 50);
   virtual ~S4JetMakerBuf();
#if 0    
   virtual TObjArray *GetOutputBufPtr();      // returns pointer to jet buffer 
#endif
   virtual S4JetSet  *GetJetSetPtr(Int_t n = 0);  // returns pointer to n-th jetSet 
   virtual void       SetInputDataBufPtr(JSFEventBuf *buf); // set ptr to input buf
   virtual void       SetDefaultYcut(Double_t ycut) { fDefaultYcut = ycut; } // set ycut 
   virtual void       DebugPrint(const Char_t *opt = "");  
   
private:  
   virtual void	MakeJets(); 	               // Make Jets
   virtual void	StoreJetSet(ANLJetFinder *jfp);// Store JetSet(Set of Jets)
 
//--------------------------------------------------------------------------
private:
   Bool_t	fDone;	 	   // (1,0) = (JetMake done, not yet)
   JSFEventBuf *fInputDataBufPtr;  // Input data buffer 
   TObjArray   *fOutputBufPtr;     // Output buffer ptr
   Int_t        fBufSize;          // output buffer size
   Double_t     fDefaultYcut;      // Default Ycut 
   Int_t        fMaxNjets;         // Njets At Default Ycut 

   ClassDef(S4JetMakerBuf, 1) // Jet data buffer
};

#endif














