//*************************************************************************   
//* ---------------------                      
//* S4JetSet Class
//* ---------------------   
//* (Description) 
//*     represent Jet.
//* (Requires)
//*     S4PFObject
//* (Provides)  
//*     S4JetSet            : made object class
//* (Update Record)
//*     2004/10/16  S.Yamamoto  Original version.
//*************************************************************************
#include "S4Jet.h"
#include "S4JetSet.h"
#include <iostream>

ClassImp(S4JetSet)

//=====================================================================
//* constructor -------------------------------------------------------
S4JetSet::S4JetSet(ANLJetFinder *jfp, Bool_t ismine, Int_t njets)
         : TObjArray(njets),
           fJetFinderPtr(jfp),  
           fIsMine(ismine)
{
   if (jfp) {
     TIter nextjet(&(jfp->GetJets())); // prepare jet iterator
     ANLJet *jp = 0;
     while ((jp = static_cast<ANLJet *>(nextjet()))) {
        Add(new S4Jet(*jp));
     }
   }
}

//=====================================================================
//* destructor --------------------------------------------------------

S4JetSet::~S4JetSet()
{
   if (fIsMine) delete fJetFinderPtr;
}

//=====================================================================       
//* GetJetFinder ------------------------------------------------------

const ANLJetFinder &S4JetSet::GetJetFinder() const 
{
   return *fJetFinderPtr;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------  
                                          
void S4JetSet::DebugPrint(const Char_t *)
{ 
   using namespace std;

   Int_t njets = GetEntries();
   cerr << " -------------------------------------------- " << endl
        << "     S4JetSet::DebugPrint: Njets = " << njets   << endl
        << " -------------------------------------------- " << endl;
   if (!njets) return;

   TIter next(this);
   S4Jet *jp = 0;
   while ((jp = static_cast<S4Jet *>(next()))) {
      jp->DebugPrint();
   }
}
