#ifndef __S4VHIT__
#define __S4VHIT__
//*****************************************************************************
//* ==============
//* S4VHit
//* ==============
//*   Base class for METIS (smeared) hits.
//*
//* The following virtual functions must be implemented in a derived class:
//*   IsSortable()
//*     ---> Simply returns a Bool_t for a sortable hit.
//*   Compare(const TObject *obj)
//*     ---> Compare a hit object with this hit. Return the following values:
//*           -1, this < hit
//*            0, this = hit
//*           +1, this > hit
//*
//* (Update Record)
//*   2004.07.30  A.Sanchez  Primitive version.
//*   2004.10.19  S.Yamamoto added kPreHitID to ESortOption
//*   2004.11.28  S.Yamamoto added kPostHitID to ESortOption
//*****************************************************************************

#include "TAttLockable.h"
#include "TObjArray.h"
#include "TVector3.h"
#include "S4VExactHit.h"
#include "TVAddress.h"

#if 0
class S4VHit : public TObject, virtual public TAttLockable {
#else
class S4VHit : public TObject, public TAttLockable {
#endif

public:
  enum ESortOption { kEnergy=0, kTof, kTrackID, kAddress, kPosition, kCALType, kClusterType, kPreHitID, kPostHitID };

  S4VHit();
  S4VHit(Double_t          edep, 
         Double_t          tof, 
         const TVector3 &  pos, 
         TVAddress      *  addr, 
         S4VExactHit    *  hit = 0);
  virtual ~S4VHit();

  virtual void    DebugPrint(const Char_t *opt = "") {}

  // Sorting functions
  virtual Bool_t  IsSortable()                  const = 0;
  virtual Int_t   Compare  (const TObject *obj) const = 0;

  static void  SetSortBy(Int_t opt = kEnergy) { fSortBy = (ESortOption)opt; }
  static Int_t GetSortBy()                    { return (Int_t)fSortBy;      }

  // Data Getters
  inline virtual TObjArray*   GetExactHitsPtr()           const { return fExactHits;                        }
  inline virtual S4VExactHit* GetExactHitPtr(Int_t i = 0) const { return (S4VExactHit*)(fExactHits->At(i)); }
  inline virtual TVector3     GetHitPosition()            const { return fHitPosition;                      }
  inline virtual Double_t     GetEnergyDeposit()          const { return fEnergyDeposit;                    }
  inline virtual Double_t     GetTOF()                    const { return fTOF;                              }
  inline virtual TVAddress*   GetAddress()                const { return fAddress;                          }

  // Data Setters
  inline virtual void AddExactHitPtr(S4VExactHit* ahit) { fExactHits->Add(ahit); }
  inline virtual void SetHitPosition(TVector3 v)        { fHitPosition = v;      }
  inline virtual void AddEnergyDeposit(Double_t e)      { fEnergyDeposit += e;   }
  inline virtual void SetEnergyDeposit(Double_t e)      { fEnergyDeposit = e;    }
  inline virtual void SetTOF(Double_t tof)              { fTOF = tof;            }
  inline virtual void SetAddress(TVAddress* address)    { fAddress = address;    }

         virtual void AddExactHit(S4VExactHit *);

protected:
  TVector3     fHitPosition;       //
  Double_t     fEnergyDeposit;     //
  Double_t     fTOF;               //
  TVAddress   *fAddress;           //
  TObjArray   *fExactHits;         //

private:
  static ESortOption fSortBy;      //!

  ClassDef(S4VHit, 1) // a (smeared) hit base class
};

#endif
