//*************************************************************************
//* -----------
//* S4TPCAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze TPC data.
//* (Update Record)
//*	2004/09/01  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>

#include "JSFHelicalTrack.h"

#include "S4PrimaryVertex.h"
#include "S4GENMCTruthBuf.h"

#include "S4TPCHit.h"
#include "S4TPCHitMakerBuf.h"
#include "S4TPCAnalysis.h"
#include "S4TPCTrackHit.h"
#include "S4TPCTrack.h"
#include "S4TPCTrackMakerBuf.h"

ClassImp(S4TPCAnalysis)

//=====================================================================
//* constructor -------------------------------------------------------

S4TPCAnalysis::S4TPCAnalysis(const Char_t *name, const Char_t *title)
             : JSFModule(name, title)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4TPCAnalysis::~S4TPCAnalysis()
{
}

//=====================================================================
//* Initialize --------------------------------------------------------

Bool_t S4TPCAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   fTrackHistPtr = new TNtupleD("htrack", "", "cl:chi2:t0:pt:cos:ndf:cpa:q:x:y:n");
   fHitHistPtr   = new TNtupleD("hhit", "", "cx:cy:cz:edep:mom:path:engy:pid");
   //fMassHistPtr  = new TNtupleD("hmass", "", "mass");

   last->cd();
   return kTRUE;
}

//=====================================================================
//* Process -----------------------------------------------------------

Bool_t S4TPCAnalysis::Process(Int_t ev)
{
   MonitorTracks(ev);
   return kTRUE;   
}   

//=====================================================================
//* MonitorTracks -----------------------------------------------------

void S4TPCAnalysis::MonitorTracks(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   TObjArray *trackaryp = static_cast<S4TPCTrackMakerBuf *>
                         (gJSF->FindModule("S4TPCTrackMaker")->EventBuf())
                        ->GetOutputBufPtr();
   TIter nexttrack(trackaryp);
   TIter nextpv(static_cast<S4GENMCTruthBuf *> 
               (gJSF->FindModule("S4GENMCTruth")->EventBuf())
               ->GetInputBufPtr());

   S4TPCTrack *trackp = 0;
   while ((trackp = dynamic_cast<S4TPCTrack *>(nexttrack()))) {
      //trackp->SetIPConstraint();
      JSFHelicalTrack *heltrackp = trackp->GetHelicalTrackPtr();
      if (trackp->IsLocked()) {
#ifdef __DEBUG__
         std::cerr << "S4TPCTrack is Locked!" << std::endl;
#endif
         continue;
      }
      Int_t trackid = trackp->GetTrackID();
      S4PrimaryVertex   *pvp = 0;
      S4PrimaryParticle *ppp = 0;
      while (!ppp && (pvp = dynamic_cast<S4PrimaryVertex *>(nextpv()))) {
         TIter nextpp(pvp->GetParticles());
         while ((ppp = dynamic_cast<S4PrimaryParticle *>(nextpp()))) {
            if (trackid == ppp->GetTrackID()) {
               break;
            }
         }
      }
      nextpv.Reset();
#ifdef __DEBUG__
      if (!ppp) std::cerr << "PrimaryParticle is NULL! " << std::endl;
#endif
      S4TPCExactHit *mcp = !ppp ? dynamic_cast<S4TPCTrackHit *>
                                 (trackp->At(0))->GetHitPtr()->GetExactHitPtr()
                                : 0;
      Double_t cl   = heltrackp->GetCL(); 
      Double_t chi2 = trackp->GetChi2(); 
      Double_t t0   = trackp->GetT0(); 
      TVector3 p    = ppp ? ppp->GetMomentum() : mcp->GetMomentum();
      Double_t pt   = p.Pt();
      Double_t cos  = p.CosTheta();
      Int_t    ndf  = trackp->GetNDF(); 
      Double_t cpa  = heltrackp->GetHelixParameter().kappa;
      Double_t q    = ppp ? ppp->GetCharge() : mcp->GetCharge();
      Double_t x    = trackp->GetPosition().X();
      Double_t y    = trackp->GetPosition().Y();
      Int_t    n    = trackp->GetNhits();
      fTrackHistPtr->Fill(cl, chi2, t0, pt, cos, ndf, cpa, q, x, y, n);

      //S4TPCTrackHit::SetSortBy("Time");
      //trackp->UnSort();
      //trackp->Sort();
      TIter nexthit(trackp);
      S4TPCTrackHit *trackhitp = 0;
      while ((trackhitp = dynamic_cast<S4TPCTrackHit *>(nexthit()))) {
         S4TPCHit      *hitp  = trackhitp->GetHitPtr();
         S4TPCExactHit *ehitp = hitp->GetExactHitPtr();
         Double_t cx   = ehitp->GetPosition().X(); 
         Double_t cy   = ehitp->GetPosition().Y(); 
         Double_t cz   = ehitp->GetPosition().Z();
         Double_t engy = ehitp->GetTotalEnergy();
         Double_t pid  = ehitp->GetPDGCode();
         Double_t edep = ehitp->GetEnergyDeposit();
         Double_t mom  = ehitp->GetMomentum().Mag();
         Double_t path =(ehitp->GetPostPosition()
                       - ehitp->GetPrePosition()).Mag();
         fHitHistPtr->Fill(cx, cy, cz, edep, mom, path, engy, pid);
      }
   }
#if 0
   Int_t size = trackaryp->GetEntries();
   for (Int_t i = 0; i < size - 1; i++) {
      S4TPCTrack *ttp1 = static_cast<S4TPCTrack *>(trackaryp->At(i));
      if (ttp1->IsLocked()) continue;
      for (Int_t j = i + 1; j < size; j++) {
         S4TPCTrack *ttp2 = static_cast<S4TPCTrack *>(trackaryp->At(j));
         if (ttp2->IsLocked()) continue;
         if (ttp1->GetCharge() * ttp2->GetCharge() < 0) {
            ANL4DVector fv   = Get4DVector(ttp1) + Get4DVector(ttp2);
            Double_t    mass = fv.GetMass();
            fMassHistPtr->Fill(mass, 0);
         }
      }
   }
#endif   
   last->cd();
}

//=====================================================================
//* Terminate ---------------------------------------------------------

Bool_t S4TPCAnalysis::Terminate()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   last->cd();
   return kTRUE;
}

//=====================================================================
//* Draw --------------------------------------------------------------

void S4TPCAnalysis::Draw(Option_t * /*opt*/)
{
}

//=====================================================================
//* Get4DVector -------------------------------------------------------
#if 0
ANL4DVector S4TPCAnalysis::Get4DVector(S4TPCTrack *ttp)
{
   Int_t    pdg = ttp->GetPDGCode();
   Double_t mass;
   if      (pdg ==   11) mass = 0.511e-3;
   else if (pdg ==   13) mass = 0.105658;
   else if (pdg ==  211) mass = 0.13957;
   else if (pdg == 2212) mass = 0.93827;
   else                  mass = 0.; 
   return ANL4DVector(TMath::Sqrt(mass * mass + ttp->GetMomentum().Mag2()),
                      ttp->GetMomentum().X(),
                      ttp->GetMomentum().Y(),
                      ttp->GetMomentum().Z());
}
#endif
