//*************************************************************************
//* --------------------
//* S4TPCHitMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad hits.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/08/22  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4TPCHitMakerBuf.h"
#include "S4TPCExactHit.h"
#include "S4TPCHit.h"
#include "TRandom.h"

//#define __DEBUG__

using namespace std;

ClassImp(S4TPCHitMakerBuf)

Double_t S4TPCHitMakerBuf::fgMergeRangeRPhi = 1.;   // [cm]
Double_t S4TPCHitMakerBuf::fgMergeRangeZ    = 5.;   // [cm]
Bool_t   S4TPCHitMakerBuf::fgIsMergeHit     = kFALSE;

//=====================================================================
//* constructor -------------------------------------------------------

S4TPCHitMakerBuf::S4TPCHitMakerBuf(const Char_t  *name,
                                   const Char_t  *title,
                                   S4TPCHitMaker *module,
                                   Int_t          bufsize)
                : U4TPCHitMakerBuf(name, title, module, bufsize),
                  fInputBufPtr(0)
{
#ifdef __DEBUG__
   cerr << " constructor for S4TPCHitMakerBuf is called..." << endl;
#endif
}          
          
S4TPCHitMakerBuf::S4TPCHitMakerBuf(S4TPCHitMaker *module,
                                   const Char_t  *name,
                                   const Char_t  *title, 
                                   Int_t          bufsize)
                : U4TPCHitMakerBuf(module, name, title, bufsize),
                  fInputBufPtr(0)
{
#ifdef __DEBUG__
   cerr << " constructor for S4TPCHitMakerBuf is called..." << endl;
#endif
}          

//=====================================================================
//* destructor --------------------------------------------------------

S4TPCHitMakerBuf::~S4TPCHitMakerBuf()
{
}


//=====================================================================
//* MakeHits ----------------------------------------------------------

void S4TPCHitMakerBuf::MakeHits()
{
#ifdef __DEBUG__
   cerr << " S4TPCHitMakerBuf::MakeHits() is called..." << endl;
#endif

   if (!fInputBufPtr) {
      cerr << "ERROR  S4TPCHitMakerBuf::MakeHits"        << endl
           << "...... cluster data not yet initialized." << endl;
      return;
   }
   else if (IsDone()) return;

   SetDone(kTRUE);

   TObjArray &outbuf = *GetOutputBufPtr();
   S4TPCExactHit *ehitp = 0;
   if (!fInputBufPtr->GetEntries()) return;
   TIter nextin(fInputBufPtr);
   while ((ehitp = dynamic_cast<S4TPCExactHit *>(nextin()))) {
      S4TPCHit *hitp = new S4TPCHit(ehitp);
      outbuf.Add(hitp);
#ifdef __DEBUG__
      hitp->DebugPrint();
#endif
   }

   // ---------------------------------------------------------------
   // Two hit separation 
   // ---------------------------------------------------------------
   if (fgIsMergeHit) { 
      S4TPCHit::SetSortBy("LayerID");
      outbuf.UnSort();
      outbuf.Sort();

      S4TPCHit *hitp = 0;
      TObjArray buf;
      TIter nextout(&outbuf);
      Int_t lastr = dynamic_cast<S4TPCHit *>(outbuf.At(0))->GetPadRowNumber();
      while ((hitp = dynamic_cast<S4TPCHit *>(nextout()))) {
         Int_t curr = hitp->GetPadRowNumber();
         if (curr == lastr) buf.Add(hitp);
         else if (buf.GetEntries() > 0) {
            if (buf.GetEntries() > 1) MergeHit(buf);
            buf.Clear();
            buf.Add(hitp);
         }
         lastr = curr;
      }
      if (buf.GetEntries() > 1) MergeHit(buf);
   }
}

//=====================================================================
//* MergeHit ----------------------------------------------------------

void S4TPCHitMakerBuf::MergeHit(TObjArray &buf)
{
   S4TPCHit::SetSortBy("Phi");
   buf.UnSort();
   buf.Sort();

   S4TPCHit *curp  = 0;
   S4TPCHit *lastp = 0;
   Double_t  lastphi = -9999.;
   Double_t  lastz   = -9999.;
   Double_t  radius  = static_cast<S4TPCHit *>(buf.At(0))->GetPadRowRadius();
   TIter     next(&buf);
   while ((curp = static_cast<S4TPCHit *>(next()))) {
      Double_t curphi = curp->GetPhi();
      Double_t curz   = curp->GetZdrift() * curp->GetSide();
      if (TMath::Abs(curphi - lastphi) * radius < fgMergeRangeRPhi
       && TMath::Abs(curz - lastz) < fgMergeRangeZ) {
         curp->Lock();
         lastp->Lock();
#ifdef __DEBUG__
         cerr << "S4TPCHit is Merged!" << endl;
#endif
      }
      lastp   = curp;
      lastphi = curphi;
      lastz   = curz;
   }
}

