#include "UserAnalysis.h"
#include <sstream>

ClassImp(UserAnalysis)

const Int_t  kNmods  = 10;
const Int_t  kFullDriftTime = 4900;

//=================================================================
//----------------
//  constructor 

UserAnalysis::UserAnalysis(BabyDisplay *display = 0,
		           		   U4CDCFADCCalibration *calib = 0) :
		      VUserAnalysis(display,calib),
		      fLabel(0.007,0.96,0.20,0.99,"Counter Info","br")
{
    BabyDisplay::SetNpages(2);
}
	
//=================================================================
//----------------
//  destructor 
	
UserAnalysis::~UserAnalysis() 
{
   for (Int_t i = 0; i < kNmods; i++ ) {
   	if (hFADC  [i]) delete hFADC[i];
   }
   if (hFADC)     delete [] hFADC; 
}


//=================================================================
//-------------------------------
void UserAnalysis::InitRawFADC()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  
  hFADC  = new TH1F*[kNmods];
 
  for (Int_t i = 0; i < kNmods; i++) {
     sprintf(st,"hFADC%2.2d",i);
     hFADC[i] = new TH1F(st,"",3300,0.,6600.);
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawRawFADC()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");

  U4BabyTrackMaker    *tm = (U4BabyTrackMaker*)gJSF->FindModule("U4BabyTrackMaker");
  U4BabyTrackMakerBuf *tb = (U4BabyTrackMakerBuf *)tm->EventBuf();
  TObjArray           *ta = tb->GetTracks();

  U4BabyTrack *tp;
  TIter nexttrack(ta);
    
  if (!(GetDisplay()->IsInLoop())) {

    TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
    da->cd();
    
    Int_t page = GetDisplay()->GetPage();
    switch (page) {
      case 0: {
        U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
        U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
        U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
        U4CDCHitMaker     *hm = (U4CDCHitMaker*)gJSF->FindModule("U4CDCHitMaker");
        U4CDCHitMakerBuf  *hb = (U4CDCHitMakerBuf *)hm->EventBuf();
        TObjArray         *la = hb->GetOutputBufPtr();

        Int_t              ncdc = evt->GetNADCData();
        TClonesArray   *adcdata = evt->GetADCData();

        for (Int_t i = 0; i < kNmods; i++) {
          hFADC[i]->Reset();
        }

        Int_t npres = __NPRES__ ;

        for (Int_t i = 0; i < ncdc; i++) {
          U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
          Int_t a;
          Int_t id;
          Int_t cr = 0;                    // We have only 1 crate now.
          Int_t st = adcdatum->GetStationNo();
          Int_t ch = adcdatum->GetChannelNo();
          
          if (st > __CHAMBERSTLAST__) break; 
          br->EChannelToDChannel(cr,st,ch,id,a);
          if (a > 999999) break;

          Double_t ped  = GetCalib()->GetPedestal(st,ch);
          Double_t gain = GetCalib()->GetGain(st,ch);      
          Double_t tped = GetCalib()->GetTimePedestal(st,ch) + npres;
          Double_t tskl = GetCalib()->GetTimeScale(st,ch);
          Double_t adc  = (ped - adcdatum->GetADC())*gain;
          Double_t time = (adcdatum->GetTime() - tped)*tskl;
          if (a >= __NWIRESPERCELL__) time = kFullDriftTime - time;
          if ( id == 0 && a >= 0 && a < kNmods ) {
            hFADC[a]->Fill(time,adc);
          }
        }

    	da->Divide(1,kNmods);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hFADC[i]->GetXaxis()->SetTickLength(0.1);
      	  hFADC[i]->GetYaxis()->SetTickLength(0.005);
      	  hFADC[i]->GetXaxis()->SetNdivisions(30);
      	  hFADC[i]->GetYaxis()->SetNdivisions(4); 
      	  hFADC[i]->GetXaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->GetYaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->Draw();
    	}
        da->Update();

#if 1	// write markers on the histgram

    	U4CDCHitLayer *layerp;
    	TIter nextLayer(la);
    	while ((layerp = (U4CDCHitLayer *)nextLayer())) {
      	  TIter nextCell(layerp);
      	  U4CDCHitCell *cellp;
      	  while ((cellp = (U4CDCHitCell *)nextCell())) {
	    TIter nextWire(cellp);
	    U4CDCHitWire *wirep;
	    while ((wirep = (U4CDCHitWire *)nextWire())) {
	      Int_t layerNo = wirep->GetLayerNumber();
	      Int_t cellNo  = wirep->GetCellNumber();
	      Int_t wireNo  = wirep->GetWireNumber();
	      Int_t a       = __NWIRESPERCELL__ *layerNo + wireNo; 
	      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	      if (a < 0 || a > 9) continue;        // works only for baby
	      Char_t str[256];
	      sprintf(str,"hHit%2.2d",a);
	      DeleteGraph(str);
	      TGraph *gr = new TGraph();
	      gr->SetName(str);
	      da->cd(a+1);
	      U4CDCHit *hitp;
	      TIter nextHit(wirep);
	      Int_t nhits = 0;
	      while ((hitp = (U4CDCHit *)nextHit())) {
	        Double_t time = hitp->GetClusterPtr()->GetStartTime();
	        if (a >= 5) time = kFullDriftTime - time;
	        gr->SetPoint(nhits++,time,0.);
	        gr->SetFillColor(19);
	        gr->SetMarkerColor(1);
	        gr->SetMarkerStyle(23);
	        gr->SetMarkerSize(1.5);
	        gr->Draw("P");
	      }
	    }
          }
        }

        nexttrack.Reset();
        Int_t ntrks = 0;
        Int_t ntrkhits = 0;
        while ((tp = (U4BabyTrack *)nexttrack())) {
	  tp->DebugPrint();
          ntrks++;
          U4CDCTrackHit *trkhitp;
          U4CDCHit *hitp;
          TIter nexttrkhit(tp);
          while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
	    hitp = trkhitp->GetHitPtr();
	    hitp->DebugPrint();
	    Int_t layerNo = hitp->GetLayerNumber();
	    Int_t cellNo  = hitp->GetCellNumber();
	    Int_t wireNo  = hitp->GetWireNumber();
	    Int_t a       = 5*layerNo + wireNo;  // works only for baby
	    if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	    if (a < 0 || a > 9) continue;        // works only for baby
	    Double_t time = hitp->GetClusterPtr()->GetStartTime();
	    if (a >= 5) time = kFullDriftTime - time;
	    da->cd(a+1);
	    Char_t str[256];
	    sprintf(str,"hTrackHit%2.2d%3.3d",a,ntrkhits++);
	    DeleteGraph(str);
	    TGraph *gr = new TGraph();
	    gr->SetName(str);
	    gr->SetPoint(0,time,0.);
	    gr->SetFillColor(19);
	    gr->SetMarkerColor((1+ntrks)%50);
	    gr->SetMarkerStyle(8);
	    gr->SetMarkerSize(0.8);
	    gr->Draw("P");
          }
        }
    	break;
#endif
      }
    }
    da->cd();
    da->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
 
  hHODOX    = new TH1F("hHODOX","X v.s. HodoscopeHit", 5,0.,40.);
  hHODOY    = new TH1F("hHODOY","Y v.s. HodoscopeHit", 3,0.,15.);
  hHODOXY    = new TH2F("hHODOXY","Scatter plot of HodoscopeHit",
                                        5,0.,40., 3,0.,15.);
  			
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t ic1st = GetDisplay()->SkipChamberData();
  for (Int_t i = ic1st; i < ncdc; i++) {
     U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == __CHAMBERID__) continue;
     if (id != 1) continue;

     UInt_t hodobits  = adcdatum->GetADC();
cerr << " ch id a bits = " << ch << " " << id << " " << a <<
     " " << (void *)hodobits << endl;

     Int_t NHodoX[__HODOX__];
     Int_t NHodoY[__HODOY__];
     for (Int_t i=0; i<__HODOX__; i++) NHodoX[i] = 0;
     for (Int_t i=0; i<__HODOY__; i++) NHodoY[i] = 0;
     
     Int_t nhods = 0;
     for (Int_t i=0; i<__HODOX__; i++) {
         if ((hodobits >> -i) & 1) {
             nhods++;
             hHODOX->Fill(i,1.);
             NHodoX[i]++;
         }
     }

     for (Int_t i=0; i<__HODOY__; i++) {
         if ((hodobits >> -i - __HODOX__) & 1) {
             nhods++;
             hHODOY->Fill(i,1.);
             NHodoY[i]++;
         }
     }

     for (Int_t i=0; i<__HODOX__; i++) {
         if (!NHodoX[i]) continue;
         for (Int_t j=0; j<__HODOY__; j++) {
             if (!NHodoY[j]) continue;
             hHODOXY->Fill(i,j,1.);
         }
     }
     
  }

  if (!(GetDisplay()->IsInLoop())) {  
     TCanvas *daparent = (TCanvas *)gROOT->FindObject("DisplayArea"); 
     daparent->cd();
     
     Int_t run = gJSF->GetRunNumber();
     Char_t runno[256];
     sprintf(runno,"Run%4.4d", run);
     
     fLabel.SetLabel(runno);
     fLabel.SetFillColor(18);
     fLabel.SetTextSize(0.99);
     fLabel.Draw();
     
     TPad *da = new TPad("da", "Counter Info",0,0,1,0.95);
     da->Draw();
      
     da->cd();
     Int_t page = GetDisplay()->GetPage();
     switch (page) {
        case 1: {
           da->Divide(2,2);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHODOXY->Draw();
           da->cd(2);
           hHODOX->Draw();
           da->cd(3);
           hHODOY->Draw();
           break;
        }
        default:
           break;
     }
     daparent->cd(); daparent->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------------
void UserAnalysis::UpdateCounterInfo()
{
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
  
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t tc1adc = 0;
  Int_t tc2adc = 0;
  Int_t hod1sumadc = 0;
  Int_t csiadc = 0;
  
  for (Int_t i = 0; i < ncdc; i++) {
    U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st < __CHAMBERSTLAST__) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();
    br->EChannelToDChannel(cr,st,ch,id,a);
    Int_t adc  = (Int_t)adcdatum->GetADC();
    switch (a) {
    case 31: {
      tc1adc = (Int_t)adc;
      //      cout << "ADC of TC1 = " << tc1adc << endl;
      break;
    }
    case 33: {
      tc2adc = (Int_t)adc;
      //      cout << "ADC of TC2 = " << tc2adc << endl;
      break;
    }
    case 34: {
      hod1sumadc = (Int_t)adc;
      //      cout << "ADC of HOD1 Sum = " << hod1sumadc << endl;
      break;
    }
    case 35: {
      csiadc = (Int_t)adc;
      //      cout << "ADC of CsI = " << csiadc << endl;
      break;
    }
    default:
      break;
    }
  }

  Char_t cntinfo[256];
  if (gJSF->GetRunNumber() < 858) {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
            tc1adc, tc2adc, csiadc);
  } else {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  HOD1Sum = %4.4d,  TC2 = %4.4d, CsI = %4.4d",
            tc1adc, hod1sumadc, tc2adc, csiadc);
  }

  TGString *str = new TGString(cntinfo);
  GetDisplay()->UpdateCounterInfo(str);
}

//______________________________________________________________________________
// ----------
//  Steering
// ----------
//

void UserAnalysis::Initialize()
{
  // cerr << "User's Initialize() is called" << endl;
  InitRawFADC();
  InitCounters();
}

void UserAnalysis::DrawHist()
{
  Int_t page = GetDisplay()->GetPage();
 switch (page) {
     case 5:
     case 4:
     case 3:
     case 2:
     case 1:
     	DrawCounters();
        break;
     case 0:
  	DrawRawFADC();
  	break;
     default:
        break;
  }
}


