#include "UserAnalysis.h"
#include <strstream>

ClassImp(UserAnalysis)

const Int_t  kNmods  = 10;
const Int_t  kFullDriftTime = 6492;

//=================================================================
//----------------
//  constructor 

UserAnalysis::UserAnalysis(BabyDisplay *display = 0,
		           		   U4CDCFADCCalibration *calib = 0) :
		      VUserAnalysis(display,calib),fSpillThreshold(5000),
		      fLabel(0.007,0.96,0.20,0.99,"Counter Info","br")
{
  for (Int_t i=0; i<__HODOX__; i++) fHodoX[i] = 0;
  for (Int_t i=0; i<__HODOY__; i++) fHodoY[i] = 0;
}
	
//=================================================================
//----------------
//  destructor 
	
UserAnalysis::~UserAnalysis() 
{
   for (Int_t i = 0; i < kNmods; i++ ) {
   	if (hFADC  [i]) delete hFADC[i];
   	if (hTPK   [i]) delete hTPK[i];
   	if (hTQ    [i]) delete hTQ[i];
   	if (hTOFTQ [i]) delete hTOFTQ[i];
	if (hHHvsDL[i]) delete hHHvsDL[i];
	if (hIvsDL [i]) delete hIvsDL[i];
	if (hEVvsXSbs [i]) delete hEVvsXSbs[i];
	if (hEVvsXSds [i]) delete hEVvsXSds[i];
   }
   if (hFADC)     delete [] hFADC; 
   if (hTPK )     delete [] hTPK;
   if (hTQ  )     delete [] hTQ;
   if (hTOFTQ )   delete [] hTOFTQ;
   if (hHHvsDL)   delete [] hHHvsDL;
   if (hIvsDL )   delete [] hIvsDL;
   if (hEVvsXSbs) delete [] hEVvsXSbs;
   if (hEVvsXSds) delete [] hEVvsXSds;
}


//=================================================================
//-------------------------------
void UserAnalysis::InitRawFADC()
{
	
	cerr << " ************************************ " << endl;
	cerr << " ***       !!! ATTENTION !!!      *** " << endl;
	cerr << " *** This version of UserAnalysis *** " << endl;
	cerr << " ***     requires FormatID = 2.   *** " << endl;
	cerr << " ************************************ " << endl;
		
	
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");
  Char_t st[256];
  
  hFADC  = new TH1F*[kNmods];
  hTPK   = new TH2F*[kNmods];
  hTQ    = new TH2F*[kNmods];
  hTOFTQ = new TH2F*[kNmods];
 
  for (Int_t i = 0; i < kNmods; i++) {
     sprintf(st,"hFADC%2.2d",i);
     if (i < 8) hFADC[i] = new TH1F(st,"",2200,0.,6600.);
     else	hFADC[i] = new TH1F(st,"",3300,0.,6600.);

     sprintf(st,"hTPK%2.2d",i);
     hTPK[i] = new TH2F(st,"",200,0.,6600.,255,0.,255.);
     sprintf(st,"hTQ%2.2d",i);
     hTQ[i] = new TH2F(st,"",200,0.,6600.,200,0.,10000.);
     sprintf(st,"hTOFTQ%2.2d",i);
     hTOFTQ[i] = new TH2F(st,"",200,0.,6600.,200,0.,10000.);

  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawRawFADC()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");

  U4BabyTrackMaker  *tm = (U4BabyTrackMaker*)gJSF->FindModule("U4BabyTrackMaker");
  U4BabyTrackMakerBuf    *tb = (U4BabyTrackMakerBuf *)tm->EventBuf();
  TObjArray      *ta = tb->GetTracks();
  TObjArray      *tj = tb->GetJunkVHit();

  U4BabyTrack *tp;
  TIter nexttrack(ta);
  
  while ((tp = (U4BabyTrack *)nexttrack())) {
    U4CDCVectorHit *vhtp;
    TIter nextvhtp(tp);
    while ((vhtp = (U4CDCVectorHit *)nextvhtp())) {
      U4CDCTrackHit *trkhitp;
      U4CDCHit *hitp;
      TIter nexttrkhit(vhtp);
      while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
        hitp = trkhitp->GetHitPtr();
        std::cerr << " hitp = " << (void *)hitp << std::endl;
        Int_t layerNo = hitp->GetLayerNumber();
        Int_t cellNo  = hitp->GetCellNumber();
        Int_t wireNo  = hitp->GetWireNumber();
        Int_t a       = 5*layerNo + wireNo;  // works only for baby
        if (cellNo == 1) a = 10 + wireNo;    // works only for baby
        if (a < 0 || a > 9) continue;        // works only for baby
        Double_t time = hitp->GetClusterPtr()->GetStartTime();
        if (a >= 5) time = kFullDriftTime - time;
        Double_t peak = hitp->GetClusterPtr()->GetPeak();
        Double_t sum  = hitp->GetClusterPtr()->GetSum();
        cerr << "a, Peak, Sum : " << a << " " << peak << " " << sum << endl;
        hTPK[a]->Fill(time,peak);
        hTQ[a]->Fill(time,sum);
      }
    }
  }
 
  if (!(GetDisplay()->IsInLoop())) {

    TCanvas *da = (TCanvas *)gROOT->FindObject("DisplayArea");  
    da->cd();
    
    Int_t page = GetDisplay()->GetPage();
    switch (page) {
      case 0: {

        U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
        U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
        U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
        U4CDCHitMaker    *hm = (U4CDCHitMaker*)gJSF->FindModule("U4CDCHitMaker");
        U4CDCHitMakerBuf      *hb = (U4CDCHitMakerBuf *)hm->EventBuf();
        TObjArray      *la = hb->GetOutputBufPtr();

        Int_t              ncdc = evt->GetNADCData();
        TClonesArray   *adcdata = evt->GetADCData();

        for (Int_t i = 0; i < kNmods; i++) {
          hFADC[i]->Reset();
        }

        Int_t npres = __NPRES__ ;

        for (Int_t i = 0; i < ncdc; i++) {
          U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
          Int_t a;
          Int_t id;
          Int_t cr = 0;                    // We have only 1 crate now.
          Int_t st = adcdatum->GetStationNo();
          Int_t ch = adcdatum->GetChannelNo();
          
          if (st > __CHAMBERSTLAST__) break; 
          
          br->EChannelToDChannel(cr,st,ch,id,a);
          Double_t ped  = GetCalib()->GetPedestal(st,ch);
          Double_t gain = GetCalib()->GetGain(st,ch);      
          Double_t tped = GetCalib()->GetTimePedestal(st,ch) + npres;
          Double_t tskl = GetCalib()->GetTimeScale(st,ch);
          Double_t adc  = (ped - adcdatum->GetADC())*gain;
          Double_t time = (adcdatum->GetTime() - tped)*tskl;
          if (a >= __NWIRESPERCELL__) time = kFullDriftTime - time;
          if ( id == 0 && a >= 0 && a < kNmods ) {
            hFADC[a]->Fill(time,adc);
          }
        }

    	da->Divide(1,kNmods);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hFADC[i]->GetXaxis()->SetTickLength(0.1);
      	  hFADC[i]->GetYaxis()->SetTickLength(0.005);
      	  hFADC[i]->GetXaxis()->SetNdivisions(30);
      	  hFADC[i]->GetYaxis()->SetNdivisions(4); 
      	  hFADC[i]->GetXaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->GetYaxis()->SetLabelSize(0.20); 
      	  hFADC[i]->Draw();
    	}
        da->Update();

#if 1	// write markers on the histgram
#if 1
    	U4CDCHitLayer *layerp;
    	TIter nextLayer(la);
    	while ((layerp = (U4CDCHitLayer *)nextLayer())) {
      	  TIter nextCell(layerp);
      	  U4CDCHitCell *cellp;
      	  while ((cellp = (U4CDCHitCell *)nextCell())) {
	    TIter nextWire(cellp);
	    U4CDCHitWire *wirep;
	    while ((wirep = (U4CDCHitWire *)nextWire())) {
	      Int_t layerNo = wirep->GetLayerNumber();
	      Int_t cellNo  = wirep->GetCellNumber();
	      Int_t wireNo  = wirep->GetWireNumber();
	      Int_t a       = __NWIRESPERCELL__ *layerNo + wireNo; 
	      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	      if (a < 0 || a > 9) continue;        // works only for baby
	      Char_t str[256];
	      sprintf(str,"hHit%2.2d",a);
	      DeleteGraph(str);
	      TGraph *gr = new TGraph();
	      gr->SetName(str);
	      da->cd(a+1);
	      U4CDCHit *hitp;
	      TIter nextHit(wirep);
	      Int_t nhits = 0;
	      while ((hitp = (U4CDCHit *)nextHit())) {
	        Double_t time = hitp->GetClusterPtr()->GetStartTime();
	        if (a >= 5) time = kFullDriftTime - time;
	        gr->SetPoint(nhits++,time,0.);
	        gr->SetFillColor(19);
	        gr->SetMarkerColor(1);
	        gr->SetMarkerStyle(23);
	        gr->SetMarkerSize(1.5);
	        gr->Draw("P");
	      }
	    }
          }
        }

        nexttrack.Reset();
        Int_t ntrks = 0;
        Int_t ntrkhits = 0;
        while ((tp = (U4BabyTrack *)nexttrack())) {
	  tp->DebugPrint();
          ntrks++;
	  U4CDCVectorHit *vhtp;
	  TIter nextvhtp(tp);
	  while ((vhtp = (U4CDCVectorHit *)nextvhtp())) {
            U4CDCTrackHit *trkhitp;
            U4CDCHit *hitp;
            TIter nexttrkhit(vhtp);
            while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
	      hitp = trkhitp->GetHitPtr();
	      hitp->DebugPrint();
	      Int_t layerNo = hitp->GetLayerNumber();
	      Int_t cellNo  = hitp->GetCellNumber();
	      Int_t wireNo  = hitp->GetWireNumber();
	      Int_t a       = 5*layerNo + wireNo;  // works only for baby
	      if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	      if (a < 0 || a > 9) continue;        // works only for baby
	      Double_t time = hitp->GetClusterPtr()->GetStartTime();
	      if (a >= 5) time = kFullDriftTime - time;
	      da->cd(a+1);
	      Char_t str[256];
	      sprintf(str,"hTrackHit%2.2d%3.3d",a,ntrkhits++);
	      DeleteGraph(str);
	      TGraph *gr = new TGraph();
	      gr->SetName(str);
	      gr->SetPoint(0,time,0.);
	      gr->SetFillColor(19);
	      gr->SetMarkerColor((1+ntrks)%50);
	      gr->SetMarkerStyle(8);
	      gr->SetMarkerSize(0.8);
	      gr->Draw("P");
	    }
          }
        }
#endif
#if 1
        Int_t nvhits = 0;
        ntrkhits = 0;
	U4CDCVectorHit *vhtp;
	TIter nextvhtp(tj);
	while ((vhtp = (U4CDCVectorHit *)nextvhtp())) {
	  std::cerr << " VHit = " << (void *)vhtp << std::endl;
	  nvhits++;
          U4CDCTrackHit *trkhitp;
          U4CDCHit *hitp;
          TIter nexttrkhit(vhtp);
          while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
	    hitp = trkhitp->GetHitPtr();
	    std::cerr << " trkhitp = " << (void *)trkhitp
		      << " hitp = " << trkhitp->GetHitPtr() << std::endl;
	    hitp->DebugPrint();
	    Int_t layerNo = hitp->GetLayerNumber();
	    Int_t cellNo  = hitp->GetCellNumber();
	    Int_t wireNo  = hitp->GetWireNumber();
	    Int_t a       = 5*layerNo + wireNo;  // works only for baby
	    if (cellNo == 1) a = 10 + wireNo;    // works only for baby
	    if (a < 0 || a > 9) continue;        // works only for baby
	    Double_t time = hitp->GetClusterPtr()->GetStartTime();
	    if (a >= 5) time = kFullDriftTime - time;
	    da->cd(a+1);
	    Char_t str[256];
	    sprintf(str,"hVectorHit%2.2d%3.3d",a,ntrkhits++);
	    DeleteGraph(str);
	    TGraph *gr = new TGraph();
	    gr->SetName(str);
	    gr->SetPoint(0,time,0.);
	    gr->SetFillColor(19);
	    gr->SetMarkerColor((1+nvhits)%50);
	    gr->SetMarkerStyle(4);
	    gr->SetMarkerSize(0.8);
	    gr->Draw("P");
	  }
        }
#endif
    	break;
      }

      case 1: {
    	da->Divide(kNmods/2,4);
    	for (Int_t i = 0; i < kNmods; i++) {
      	  da->cd(i+1);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hTPK[i]->GetXaxis()->SetTickLength(0.04);
      	  hTPK[i]->GetYaxis()->SetTickLength(0.05);
      	  hTPK[i]->GetXaxis()->SetNdivisions(5);
      	  hTPK[i]->GetYaxis()->SetNdivisions(5); 
      	  hTPK[i]->GetXaxis()->SetLabelSize(0.10); 
      	  hTPK[i]->GetYaxis()->SetLabelSize(0.10); 
      	  hTPK[i]->Draw();
      	  da->cd(i+11);
      	  gStyle->SetOptStat(0);
      	  gPad->SetTopMargin(0.01);
      	  gPad->SetBottomMargin(0.2);
      	  hTQ[i]->GetXaxis()->SetTickLength(0.04);
      	  hTQ[i]->GetYaxis()->SetTickLength(0.05);
      	  hTQ[i]->GetXaxis()->SetNdivisions(5);
      	  hTQ[i]->GetYaxis()->SetNdivisions(5); 
      	  hTQ[i]->GetXaxis()->SetLabelSize(0.10); 
      	  hTQ[i]->GetYaxis()->SetLabelSize(0.10); 
      	  hTQ[i]->Draw();
    	}
        break;
#endif
      }
    }
    da->cd();
    da->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::InitCounters()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");

  hTOFADCTOF1 = new TH2F("hTOFADCTOF1","TOF v.s. ADC_TOF1",
  			128,0.,2048.,128,0.,2048.); 
  hTOFADCTOF2 = new TH2F("hTOFADCTOF2","TOF v.s. ADC_TOF2",
  			128,0.,2048.,128,0.,2048.); 
  hADCTOF12 = new TH2F("hADCTOF12","ADC_TOF1 v.s. ADC_TOF2",
  			128,0.,2048.,128,0.,2048.); 
  hTC1TC2   = new TH2F("hTC1TC2","TC1 v.s. TC2",
  			128,0.,2048.,128,0.,2048.); 
  hCH1CH2   = new TH2F("hCH1CH2","CH1 v.s. CH2",
  			128,0.,2048.,128,0.,2048.); 
  hCH1PBG   = new TH2F("hCH1PBG","CH1 v.s. PbG",
  			128,0.,2048.,128,0.,2048.);     			
  			
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawCounters()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Double_t tof     = 0.;
  Double_t adctof1 = 0.;
  Double_t adctof2 = 0.;
  Double_t adctc1  = 0.;
  Double_t adctc2  = 0.;
  Double_t adcch1  = 0.;
  Double_t adcch2  = 0.;
  Double_t adcpbg  = 0.;
  
  Int_t ic1st = GetDisplay()->SkipChamberData();
  for (Int_t i = ic1st; i < ncdc; i++) {
     U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = adcdatum->GetStationNo();
     Int_t ch = adcdatum->GetChannelNo();
     
     br->EChannelToDChannel(cr,st,ch,id,a);
     if (id == __CHAMBERID__) continue;
     
     Double_t adc  = adcdatum->GetADC();
                           
     if ( id == __TOFID__ ) {	// TOF
       if ( a == 1 ){
        tof = adcdatum->GetTime();
       }
     } else if ( id == __ADCID__ ) {	// ADC counters
        switch (a) {
       	   case 0: {
       	  	adctof1 = adc;
       	  	break;
       	   }
       	   case 1: {
       	  	adctof2 = adc;
       	  	break;
       	   }
       	   case 2: {
       	  	adctc1  = adc;
       	  	break;
       	   }
       	   case 3: {
       	  	adctc2  = adc;
       	  	break;
       	   }
           case 4: {
       	  	adcch1  = adc;
       	  	break;
       	   }
           case 5: {
       	  	adcch2  = adc;
       	  	break;
       	   }
           case 6: {
       	  	adcpbg  = adc;
       	  	break;
       	   }
      	   default:
       	   	break;
        }
     }
  }

  hTOFADCTOF1->Fill(tof,adctof1);
  hTOFADCTOF2->Fill(tof,adctof2);
  hADCTOF12->Fill(adctof1,adctof2);
  hTC1TC2->Fill(adctc1,adctc2);
  hCH1CH2->Fill(adcch1,adcch2);
  hCH1PBG->Fill(adcch1,adcpbg);
  

  if (!(GetDisplay()->IsInLoop())) {  
     TCanvas *daparent = (TCanvas *)gROOT->FindObject("DisplayArea"); 
     daparent->cd();
     
     Int_t run = gJSF->GetRunNumber();
     Char_t runno[256];
     sprintf(runno,"Run%4.4d", run);
     
     fLabel.SetLabel(runno);
     fLabel.SetFillColor(18);
     fLabel.SetTextSize(0.99);
     fLabel.Draw();
     
     TPad *da = new TPad("da", "Counter Info",0,0,1,0.95);
     da->Draw();
      
     da->cd();
     Int_t page = GetDisplay()->GetPage();
     switch (page) {
        case 2: {
           da->Divide(3,4);
           gStyle->SetOptStat(0);
           da->cd(1);
           hTOFADCTOF1->Draw();
           da->cd(2);
           DeleteHist("hTOFADCTOF1_px");
           hTOFADCTOF1->ProjectionX()->Draw();
           da->cd(3);
           DeleteHist("hTOFADCTOF1_py");
           hTOFADCTOF1->ProjectionY()->Draw();
           da->cd(4);
           hTOFADCTOF2->Draw();
           da->cd(5);
           DeleteHist("hTOFADCTOF2_py");
           hTOFADCTOF2->ProjectionY()->Draw();
           da->cd(6);
	   hADCTOF12->Draw();
           da->cd(7);
           hTC1TC2->Draw();
           da->cd(8);
           DeleteHist("hTC1TC2_px");
           hTC1TC2->ProjectionX()->Draw();
           da->cd(9);
           DeleteHist("hTC1TC2_py");
           hTC1TC2->ProjectionY()->Draw();
           da->cd(10);
           hCH1CH2->Draw();
           da->cd(11);
           hCH1PBG->Draw();
           da->cd(12);
           DeleteHist("hCH1PBG_py");
           hCH1PBG->ProjectionY()->Draw();
           break;
        }
        default:
           break;
     }
     daparent->cd(); daparent->Update();
  }
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::InitScaler()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");

  hHODOX    = new TH1F("hHODOX","X v.s. HodoscopeHit", 5,0.,40.); 
  hHODOY    = new TH1F("hHODOY","Y v.s. HodoscopeHit", 3,0.,15.); 
  hHODO2    = new TH2F("hHODO2","Scatter plot of HodoscopeHit",
  					5,0.,40., 3,0.,15.); 
  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::DrawScaler()
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  
  //Int_t nhits;
  //Bool_t duringspill = CheckScalerHits(nhits);
  
  for (Int_t i=0; i<__HODOX__; i++) {
     hHODOX->Fill(i*8.0,fHodoX[i]);
     for (Int_t j=0; j<__HODOY__; j++) {
     	Double_t value = fHodoX[i] * fHodoY[j];
     	hHODOY->Fill(j*5.0,fHodoY[j]);
     	hHODO2->Fill(i*8., j*5., value/1.e3);
     }
  }
      
  // Draw histgram on canvas ...

  if (!(GetDisplay()->IsInLoop())) {  
     TCanvas *daparent = (TCanvas *)gROOT->FindObject("DisplayArea"); 
     daparent->cd();
     
     Int_t run = gJSF->GetRunNumber();
     Char_t runno[256];
     sprintf(runno,"Run%4.4d", run);
     
     fLabel.SetLabel(runno);
     fLabel.SetFillColor(18);
     fLabel.SetTextSize(0.99);
     fLabel.Draw();
     
     TPad *da = new TPad("da", "Hodoscope Info",0,0,1,0.95);
     da->Draw();
      
     da->cd();
     Int_t page = GetDisplay()->GetPage();
     switch (page) {
        case 3: {
           da->Divide(2,2);
           gStyle->SetOptStat(0);
           da->cd(1);
           hHODOX->Draw();
           da->cd(2);
           hHODOY->Draw();
           da->cd(3);
           hHODO2->Draw();
           break;
        }
        default:
           break;
     }
     daparent->cd(); daparent->Update();
  }
  last->cd();
}

//=================================================================
//---------------------------------------------
Bool_t UserAnalysis::CheckScalerHits(Int_t &nhits)
{
  TDirectory *last = gDirectory;
  gFile->cd("/");
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t         ndata   = evt->GetNScalerData();
  TClonesArray *data	= evt->GetScalerData();
  
  nhits = 0;
  for (Int_t i=0; i<__HODOX__; i++) fHodoX[i] = 0;
  for (Int_t i=0; i<__HODOY__; i++) fHodoY[i] = 0;
    
  
  for (Int_t i = 0; i < ndata; i++) {
     U4CDCRawScalerDatum *datum = (U4CDCRawScalerDatum *)data->UncheckedAt(i);
     Int_t a;
     Int_t id;
     Int_t cr = 0;               // We have only 1 crate now.
     Int_t st = datum->GetStationNo();
     Int_t ch = datum->GetChannelNo();
     
     br->EChannelToDChannel(cr,st,ch,id,a);
     if (id != __SCLRGPIBID__) {
     	cerr << " DrawScalers::FormatID inconsistent!!! My ID = " << id << endl;
     	continue;
     }
     
     Int_t scaler  = datum->GetScaler();
                           
     switch (a) {
             case 0:
           	fHodoX[0] = scaler;
           	nhits += scaler;
           	break;
             case 1:
           	fHodoX[1] = scaler;
           	nhits += scaler;
           	break;
             case 2:
           	fHodoX[2] = scaler;
           	nhits += scaler;
           	break;
             case 3:
           	fHodoX[3] = scaler;
           	nhits += scaler;
           	break;
             case 4:
           	fHodoX[4] = scaler;
           	nhits += scaler;
           	break;
             case 5:
           	fHodoY[0] = scaler;
           	nhits += scaler;
           	break;
             case 6:
           	fHodoY[1] = scaler;
           	nhits += scaler;
           	break;
             case 7:
           	fHodoY[2] = scaler;
           	nhits += scaler;
           	break;
      	     default:
       	   	break;
     }
  }
  
  // Check this event is during spill or not ...
  
  Bool_t isinspill;
  
  if (nhits >= 5000) {
    isinspill = kTRUE;
  } else {
    isinspill = kFALSE;
  }
  
  last->cd();
  return isinspill;
}
//=================================================================
//-------------------------------
Double_t UserAnalysis::GetGPIB()
{

  TDirectory *last = gDirectory;
  gFile->cd("/");
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();

  TClonesArray      *data  = evt->GetGPIBData();  
  U4CDCRawGPIBDatum *datum = (U4CDCRawGPIBDatum *)data->UncheckedAt(0);
  Double_t gpibcurrent  = datum->GetGPIB();
  return gpibcurrent;

  last->cd();
}

//=================================================================
//-------------------------------
void UserAnalysis::InitDriftLengthVs()
{
  TDirectory *last = gDirectory;
  gJSF->GetOutput()->cd("/");

  hHHvsDL   = new TH2F*[kNmods];
  hIvsDL    = new TH2F*[kNmods];
  hEVvsXSbs = new TH2F*[2];
  hEVvsXSds = new TH2F*[2];
   
  for (Int_t i=0; i<kNmods; i++) {
    TString hhname("hHHvsDL"); 
    TString hhlabel("Hodoscope Hits vs DriftLength at wire"); 
    TString iname("hIvsDL"); 
    TString ilabel("Current vs DriftLength at wire"); 
    hhname  += i;
    hhlabel += i;
    iname  += i;
    ilabel += i;
    hHHvsDL[i] = new TH2F(hhname, hhlabel,200, 0.,100000., 1000, 0., 5.);
    hIvsDL [i] = new TH2F(iname, ilabel,400, 0.,400., 1000, 0., 5.);
  }
  
  for (Int_t i=0; i<2; i++) {
    TString bsname("hEVvsXSbs"); 
    TString bslabel("Event vs X at Sheelding Wire(between spills) Layer"); 
    TString dsname("hEVvsXSds"); 
    TString dslabel("Event vs X at Sheelding Wire(during spill) Layer"); 
    bsname  += i;
    bslabel += i;
    dsname  += i;
    dslabel += i;
    hEVvsXSbs[i] = new TH2F(bsname, bslabel,1000, 0.,1000., 2000, 1.5, 3.5);
    hEVvsXSds[i] = new TH2F(dsname, dslabel,1000, 0.,1000., 2000, 1.5, 3.5);
  }
  last->cd();
}

//=================================================================
//---------------------------------------------
void UserAnalysis::DrawDriftLengthVs()
{
  
  //cerr << "UserAnl::DrawDriftLengthVs is called " << endl;
  
  TDirectory *last = gDirectory;
  gFile->cd("/");
  
  U4BabyTrackMaker *tm = (U4BabyTrackMaker*)gJSF->FindModule("U4BabyTrackMaker");
  U4BabyTrackMakerBuf   *tb = (U4BabyTrackMakerBuf *)tm->EventBuf();
  TObjArray     *ta = tb->GetTracks();
  
  U4BabyTrack *tp;
  //U4BabyTrack *tplst = 0;
  TIter nexttrack(ta);
  TObjArray tracks[2];
  
  while ((tp = (U4BabyTrack *)nexttrack())) {
     U4CDCHit *htp = ((U4CDCTrackHit *)tp->At(0))->GetHitPtr();
     Int_t thislayer = htp->GetLayerNumber();
     //Int_t thiscell  = htp->GetCellNumber();
     //if (thiscell != 0) continue;		// skip unequipped cell
     tracks[thislayer].Add(tp);
  }
  
  Int_t    hodohits;
  Int_t    eventno = gJSF->GetEventNumber();
  Double_t current = GetGPIB() * 100000;
  Bool_t   duringspill = CheckScalerHits(hodohits);
  
  cerr << "evtno, current, hodohits, inspill = "
       << eventno << " " << current << " " << hodohits << " " << duringspill << endl;
    
  for (Int_t cellNo=0; cellNo<2; cellNo++) {
    TIter next(&tracks[cellNo]);
    U4BabyTrack *vectortrkp;			// vector track at cell 0
    Double_t driftLen;
    Double_t xatsheeld;
    Int_t wireNo;
    Int_t wireY ;
    Int_t histNo;   
    while ((vectortrkp = (U4BabyTrack *)next())) {
      U4CDCTrackHit *trkhitp;
      U4CDCHit *hitp;
      TIter nexttrkhit(vectortrkp);
      while ((trkhitp = (U4CDCTrackHit *)nexttrkhit())) {
        hitp = trkhitp->GetHitPtr();
        wireNo  = hitp->GetWireNumber();      	
        wireY   = cellNo* (__NWIRESPERCELL__ + 5) + wireNo;
        histNo  = cellNo* __NWIRESPERCELL__ + wireNo ;
        driftLen  = vectortrkp->Xat(wireY);
        hHHvsDL[histNo]->Fill((Double_t)hodohits, driftLen, 1.);
        hIvsDL [histNo]->Fill(current, driftLen, 1.);
      }
      
      xatsheeld = vectortrkp->Xat(__YATSHEELD__);
      if (duringspill) {
        hEVvsXSds[cellNo]->Fill((Double_t)eventno, xatsheeld, 1.);
        cerr << " X at sheelding wire(during spill) cell" << cellNo << " = "
             << xatsheeld << endl;      
      } else {
        hEVvsXSbs[cellNo]->Fill((Double_t)eventno, xatsheeld, 1.);
        cerr << " X at sheelding wire(between spills) cell" << cellNo << " = "
             << xatsheeld << endl;      
      }	
          
    }
  }


  // Draw histgram on canvas ...

  if (!(GetDisplay()->IsInLoop())) {  
     TCanvas *daparent = (TCanvas *)gROOT->FindObject("DisplayArea"); 
     daparent->cd();
     
     Int_t run = gJSF->GetRunNumber();
     Char_t runno[256];
     sprintf(runno,"Run%4.4d", run);
     
     fLabel.SetLabel(runno);
     fLabel.SetFillColor(18);
     fLabel.SetTextSize(0.99);
     fLabel.Draw();
     
     TPad *da = new TPad("da", "DriftLength and XDiff Info",0,0,1,0.95);
     da->Draw();
      
     da->cd();
     Int_t page = GetDisplay()->GetPage();
     switch (page) {
        case 4: {
           da->Divide(5,4);
           gStyle->SetOptStat(0);
           Int_t i;
           for (i=0; i<kNmods; i++) {
             da->cd(i+1);
             if(hHHvsDL[i]) hHHvsDL[i]->Draw();
           }
           for (Int_t i=0; i<kNmods; i++) {
             da->cd(i+kNmods+1);
             if(hIvsDL[i]) hIvsDL[i]->Draw();
           }
            break;
        }
        case 5: {
           da->Divide(2,2);
           gStyle->SetOptStat(0);
           for (Int_t i=0; i<2; i++) {
             da->cd(i+1);
             if(hEVvsXSbs[i]) hEVvsXSbs[i]->Draw();
           }
           for (Int_t i=0; i<2; i++) {
             da->cd(i+3);
             if(hEVvsXSds[i]) hEVvsXSds[i]->Draw();
           }
           break;
        }
        default:
           break;
     }
     daparent->cd(); daparent->Update();
  }
  last->cd();
} 

//=================================================================
//-------------------------------------
void UserAnalysis::UpdateCounterInfo()
{
  U4CDCRawData     *raw = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
  U4CDCRawDataBuf  *evt = (U4CDCRawDataBuf *)raw->EventBuf();
  U4CDCBeginRunBuf *br  = raw->GetBeginRunBuf();
  
  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t i;
  Int_t tof    = 0;
  Int_t adctc1 = 0;
  Int_t adctc2 = 0;
  Int_t adcch1 = 0;
  Int_t adcch2 = 0;
  Int_t adcpbg = 0;
  
  for (i = 0; i < ncdc; i++) {
    U4CDCRawADCDatum *adcdatum = (U4CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st < __CHAMBERSTLAST__) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();
    br->EChannelToDChannel(cr,st,ch,id,a);
    Int_t adc  = (Int_t)adcdatum->GetADC();
    if (id == __TOFID__) {
      if ( a == 1 ) {	
        tof = (Int_t)adcdatum->GetTime();
        }
    } else if (id == __ADCID__) {
      switch (a) {
        case 2: {
          adctc1 = adc;
          break;
        }
        case 3: {
          adctc2 = adc;
          break;
        }
        case 4: {
          adcch1 = adc;
          break;
        }
        case 5: {
          adcch2 = adc;
          break;
        }
        case 6: {
          adcpbg = adc;
          break;
        }
        default:
          break;
      }
    } 
  }
  Char_t cntinfo[256];
  sprintf(cntinfo,
          "TOF: %4.4d  ADC: TC1 = %4.4d, TC2 = %4.4d, CH1 = %4.4d, Pb = %4.4d",
	  tof, adctc1, adctc2, adcch1, adcpbg);

  TGString *str = new TGString(cntinfo);
  GetDisplay()->UpdateCounterInfo(str);
}

//______________________________________________________________________________
// ----------
//  Steering
// ----------
//

void UserAnalysis::Initialize()
{
  // cerr << "User's Initialize() is called" << endl;
  InitRawFADC();
  InitCounters();
  InitScaler();
  InitDriftLengthVs();
}

void UserAnalysis::DrawHist()
{
  Int_t page = GetDisplay()->GetPage();
 switch (page) {
     case 5:
     case 4:
        DrawDriftLengthVs();
        break;
     case 3:
     	DrawScaler();
        break;
     case 2:
     	DrawCounters();
        break;
     case 1:
     case 0:
  	DrawRawFADC();
  	break;
     default:
        break;
  }
}


