//*************************************************************************
//* --------------------
//* U4BabyTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2002/11/15  R.Kuboshima	   Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "U4BabyTrackMakerBuf.h"
#include "U4BabyTrack.h"
#include "U4CDCVectorHit.h"


ClassImp(U4BabyTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
U4BabyTrackMakerBuf::U4BabyTrackMakerBuf(const char *name, const char *title, 
				U4BabyTrackMaker *module, Int_t bufsize) 
                    :U4CDCTrackMakerBuf(name, title, module, bufsize)
{
}

U4BabyTrackMakerBuf::U4BabyTrackMakerBuf(U4BabyTrackMaker *module,
	                                 const char *name, 
			                 const char *title, Int_t bufsize)
                    :U4CDCTrackMakerBuf(module, name, title, bufsize)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4BabyTrackMakerBuf::~U4BabyTrackMakerBuf()
{  
}

//=====================================================================
//* SetHitData --------------------------------------------------------
#if 0
void U4BabyTrackMakerBuf::SetHitData(U4CDCHitMakerBuf *htbuf)
{
	fDone    = kFALSE;
	fHitData = htbuf;
	if (fTracks.GetEntries() > 0) fTracks.Delete();
}
#endif
//=====================================================================
//* IsInitialized -----------------------------------------------------

Bool_t U4BabyTrackMakerBuf::IsInitialized() const
{
   if (fInputDataBufPtr) return kTRUE;
   else return kFALSE;
}

//=====================================================================
//* GetNtracks --------------------------------------------------------

Int_t U4BabyTrackMakerBuf::GetNtracks()
{
   if (!fDone) MakeTracks();
   return fOutputBuf.GetEntries();
}

//=====================================================================
//* GetTracks ---------------------------------------------------------

TObjArray * U4BabyTrackMakerBuf::GetTracks()
{
   if (!fDone) MakeTracks();
   return &fOutputBuf;
}

//=====================================================================
//* GetJunkVHit ---------------------------------------------------------

TObjArray * U4BabyTrackMakerBuf::GetJunkVHit()
{
   if (!fDone) MakeTracks();
   return &fJunkVHit;
}

//=====================================================================
//* MakeTracks --------------------------------------------------------

void U4BabyTrackMakerBuf::MakeTracks()
{
   if (!IsInitialized()) {
      std::cerr << "ERROR  U4BabyTrackMakerBuf::MakeTracks"   << std::endl
                << "...... cluster data not yet initialized." << std::endl;
      return;
   } else if (fDone) {
      return;
   }
   if(fJunkVHit.GetEntries() > 0) fJunkVHit.Delete();

   fDone = kTRUE;

   TObjArray *inputbufp = GetHitMakerBufPtr()->GetOutputBufPtr();

   TObjArray truevhittmp(1000);    
        
   TIter nextlayer(inputbufp);
   U4CDCHitLayer *lyrp;

   while ((lyrp = (U4CDCHitLayer *)nextlayer())) {
      TIter nextcell(lyrp);
      U4CDCHitCell *celp;
      while ((celp = (U4CDCHitCell *)nextcell())) {
         MakeVectorHits(celp, &truevhittmp);
      }
   }
        
   if (truevhittmp.GetEntries() == 0) {
      std::cerr << "***U4BabyTrackMakerBuf::MakeTracks: no truevhit!***" 
                << std::endl;
      return;
   }

   TIter nextvhit(&truevhittmp);
   U4CDCVectorHit *vhitp;
#if 1
   std::cerr << "U4BabyTrackMakerBuf::MakeTracks: truevhittmp has "
             << truevhittmp.GetEntries() << " vhitp."
             << std::endl;

   while ((vhitp = (U4CDCVectorHit *)nextvhit())) {
      std::cerr << "   vhitp = " << vhitp 
                << ",  layerno = " << vhitp->GetLayerNumber()
                << ",  cellid = " << vhitp->GetCellID()
                << std::endl;
   }
   nextvhit.Reset();
#endif
   // --------------
   //  Fill b-array
   // --------------
   TObjArray barray(fNbins);
   Double_t db = (fBmax - fBmin) / fNbins;
#if 0 
   std::cerr << " fBmax = "  << fBmax
	     << " fBmin = "  << fBmin
	     << " fNbins = " << fNbins
	     << " db = "     << db << std::endl;
#endif
   while ((vhitp = (U4CDCVectorHit *)nextvhit())) {
      Double_t b = vhitp->GetIntercept();
      std::cerr << "b = " << b << std::endl; 
      if (b < -5.5 || b > 5.5) {
         std::cerr << "Kill vhitp = " << vhitp << std::endl; 
         delete vhitp;
	 continue;
      } else {     
         Int_t    i = (Int_t)((b - fBmin) / db);
         if (!barray[i]) barray[i] = new U4BabyTrack();
         std::cerr << "Add vhitp in barray[" << i << "]" << " = " << vhitp
                   << std::endl; 
         ((U4BabyTrack *)barray[i])->Add(vhitp);
      }
   }

   // --------------
   //  Merge b-array
   // --------------
   for (Int_t i = 1; i < fNbins; i++) {
      if (barray[i]   && ((U4BabyTrack *)barray[i])->GetEntries() && 
	  barray[i-1] && ((U4BabyTrack *)barray[i-1])->GetEntries()) {
         TIter next((U4BabyTrack *)barray[i-1]);
	 U4CDCVectorHit *vhitp;
         while ((vhitp = (U4CDCVectorHit *)next())) {
               std::cerr << "Marge vhitp in barray[" 
		         << i << "]" << " = " << vhitp
                         << std::endl; 
               ((U4BabyTrack *)barray[i])->Add(new U4CDCVectorHit(*vhitp));
         }
         delete barray[i-1];
	 barray[i-1] = 0;
      }
   }

   // --------------
   //  Find tracks
   // --------------
   Int_t trackno = 0;
   for (Int_t i = 0; i < fNbins; i++) {
      if (!barray[i]) continue;
      U4BabyTrack *tkp = (U4BabyTrack *)barray[i];
      tkp->SetOwner();
      Int_t nvhits = tkp->GetEntries();
      if (nvhits >= fNvhitCut) {
      //------------------------
      // Fit and collect vhitp.
      //------------------------
	 std::cerr << vhitp << " has more 2 vhits (^^)" << std::endl;
         U4BabyTrack *bst = 0;
         for (Int_t iv = 0; iv < nvhits-1; iv++) {
	    U4CDCVectorHit *vhitp1 = (U4CDCVectorHit *)tkp->At(iv); 
            Int_t layerno1 = vhitp1->GetLayerNumber();
	    std::cerr << vhitp1 << "'s layerno = " << layerno1 << std::endl;
	    Double_t lstchi2 = 1000000000.;
	    for (Int_t jv = iv+1; jv < nvhits; jv++) {
	       std::cerr << "jv = " << jv << std::endl;
	       U4CDCVectorHit *vhitp2 = (U4CDCVectorHit *)tkp->At(jv); 
	       std::cerr << "vhitp2 = " << vhitp2 << std::endl;
               Int_t layerno2 = vhitp2->GetLayerNumber();
	       std::cerr << vhitp2 << "'s layerno = " << layerno2 << std::endl;
               if (layerno1 == layerno2) continue; 
               U4BabyTrack *tmptkp = new U4BabyTrack();
               tmptkp->Add(new U4CDCVectorHit(*vhitp1));
               tmptkp->Add(new U4CDCVectorHit(*vhitp2));
	       tmptkp->SetOwner();
               tmptkp->Update();
               Double_t chi2 = tmptkp->GetChi2();
	       std::cerr << "chi2 =  " << chi2 
	                 << ",  lstchi2 =  " << lstchi2 << std::endl;
               if (chi2 < lstchi2) {
                  bst = tmptkp;
	          vhitp1->SetTrackPtr(tmptkp); 
	          vhitp2->SetTrackPtr(tmptkp); 
                  lstchi2 = chi2;
               } else {
	          delete tmptkp;
	       } 
            }
	 }

	 std::cerr << "bst = " << bst << std::endl;
      //------------------------
	 if (bst) bst->SetTrackNumber(trackno++); 
         fOutputBuf.Add(bst);
	 delete barray[i];
      } else {
	 std::cerr << "fJunkVhit = " << barray[i] << std::endl;
         std::cerr << "Add barray[" << i << "] in fJunkVhit!!!" 
                   << std::endl; 
	 TIter nextjunktrack((U4BabyTrack *)barray[i]);
	 U4CDCVectorHit *vhtPtr = 0;
	 while ((vhtPtr = (U4CDCVectorHit *)nextjunktrack())){
           fJunkVHit.Add(vhtPtr);
	 }
      }
   }
   std::cerr << "****End U4BabyTrackMakerBuf::MakeTracks ****" 
             << std::endl; 
}

//=====================================================================
//* MakeVectorHits  ---------------------------------------------------

void U4BabyTrackMakerBuf::MakeVectorHits(U4CDCHitCell *celp, TObjArray *truevhittmpp)
{
   ////// Very Temporary Treatment //////
   Double_t cellwidth  = 5.0;
   Double_t cellmargin = 0.1;
   ////// Very Temporary Treatment //////  

   Int_t nwires = celp->GetEntries();
   if (nwires < 4)    return;
        
   celp->UnSort();
   celp->Sort();          //sort wires

   for (Int_t lrsel = 0; lrsel < 2 ; lrsel++) { //center closs(1) or not(0)
      Int_t pivotcomb;
      for (pivotcomb = 0; pivotcomb < (nwires < 5 ? 1 : 3); pivotcomb++) {
         Int_t iw1st;
         Int_t iwlst;
         switch (pivotcomb) {
            case 1:
               iw1st = 0;
               iwlst = nwires - 2;
               break;
            case 2:
               iw1st = 1;
               iwlst = nwires - 1;
               break;
            case 0:
            default:
               iw1st = 0;
               iwlst = nwires - 1;
               break;
         }   

         U4CDCHitWire *wire1st = (U4CDCHitWire *)celp->At(iw1st);
         U4CDCHitWire *wirelst = (U4CDCHitWire *)celp->At(iwlst);
   
         TIter nextin1st(wire1st);
         TIter nextinlst(wirelst);

         U4CDCHit *ht1st;
         U4CDCHit *htmdl;
         U4CDCHit *htlst;

	 nextin1st.Reset();
         while ((ht1st = (U4CDCHit *)nextin1st())) {  //hits in 1st wire
            if (ht1st->IsLocked()) continue;           //skip if locked
            Double_t x1  = ht1st->Xdrift();
            if (x1 < -cellmargin || x1 > cellwidth+cellmargin) continue;
            Double_t dx1 = ht1st->DXdrift();
            if (dx1 > xDXpivot) continue;              //skip if bad quality
            nextinlst.Reset();
            while ((htlst = (U4CDCHit *)nextinlst())) { //hits in lst wire
               if (htlst->IsLocked()) continue;          //skip if locked 
               Double_t x2  = htlst->Xdrift();
               if (x2 < -cellmargin || x2 > cellwidth+cellmargin) continue;
               Double_t dx2 = htlst->DXdrift();
               if (dx2 > xDXpivot) continue;             //skip if bad quality

               U4CDCVectorHit *trialvhitp = new U4CDCVectorHit(); //make trial track
               trialvhitp->Add(new U4CDCTrackHit(ht1st,-1)); //add 1st hit
               trialvhitp->Add(new U4CDCTrackHit(htlst,2*lrsel-1)); //add lst hit
               if (lrsel == 1) trialvhitp->Update();
               Int_t i;
               for (i = 0; i < nwires; i++) {    // loop over middle wires
                  if (i == iw1st || i == iwlst) continue;  //skip pivots
                  Double_t diffmn = 999999.;
                  Int_t lrexpect;
                  U4CDCHitWire *wiremdl = (U4CDCHitWire *)celp->At(i);
                  if (lrsel == 0) {
                     lrexpect = -1;
                  } else {
                     lrexpect = (trialvhitp->Xat(wiremdl->Y()) < 0 ? -1 : 1);
                  }
                  TIter nextinmdl(wiremdl);
                  U4CDCHit *bst = 0;
                  while ((htmdl = (U4CDCHit *)nextinmdl())) {//hits in mdl wire
                     if (htmdl->IsLocked()) continue;   //skip if locked
                     Double_t x3 = htmdl->Xdrift();
                     if (x3 < -cellmargin || x3 > cellwidth+cellmargin) continue;
                     Double_t dx3 = htmdl->DXdrift();
                     if (dx3 > xDXpivot) continue;
                     U4CDCTrackHit *hitp = new U4CDCTrackHit(htmdl,lrexpect);
                     hitp->Lock();           // exclude this hit from fit
                     trialvhitp->Add(hitp);  // add this temporarilly to track 
                     trialvhitp->Update();   // and update residual
                     Double_t diff   = hitp->GetResidual();
                     Double_t diffct = xNsigmas*(dx1+dx2+dx2);
                     trialvhitp->Remove(hitp);  // remove this hit from track
                     delete hitp;               // delete this temp. hit
                     if (TMath::Abs(diff) > diffct) continue;  //resid. cut
                     if (TMath::Abs(diff) < diffmn) {     //select the best
                        diffmn = TMath::Abs(diff);
                        bst    = htmdl;
                     }
                  }
                  if (bst) {                       // add the best to track
                     trialvhitp->Add(new U4CDCTrackHit(bst,lrexpect));
                  }
               }
               if (trialvhitp->GetEntries() < 4 
                   || trialvhitp->GetChi2() > xChi2
                   || TMath::Abs(trialvhitp->GetSlope()) > xSlope) {
                  delete trialvhitp;               // track rejected
               } else {
                  truevhittmpp->Add(trialvhitp);
		  std::cerr << "trialvhitp = " << trialvhitp << ",  layerno = "
			    << trialvhitp->GetLayerNumber() << std::endl;
                  truevhittmpp->Add(new U4CDCVectorHit(*trialvhitp, kTRUE));
                  TIter nextht(trialvhitp);
                  U4CDCTrackHit *ht;
                  Int_t ngoodhits = 0;
                  while ((ht = (U4CDCTrackHit *)nextht())) { //poison used hits
                     U4CDCTrackHit * cht = ht;
                     if (TMath::Power(ht->GetResidual()/xKCNsigmas,2) <
                         trialvhitp->SGX2at(((U4CDCHit *)cht
                                                     ->GetHitPtr())->Ywire())) {
                         ((U4CDCHit *)cht->GetHitPtr())->Lock();
                         ngoodhits++;
                     }
                  }
                  if (ngoodhits >= 4) goto NEXT;
               }
            }
            NEXT: continue;
         }
      }
   }
}

#if 1 
void U4BabyTrackMakerBuf::DebugPrint(const Char_t *opt)
{
	Ntracks += fOutputBuf.GetEntries();
        std::cerr << fOutputBuf.GetEntries()
                  << " tracks in this event:" << std::endl;
	std::cerr << " Total Tracks are " << Ntracks << std::endl;
        TIter next(&fOutputBuf);
        U4BabyTrack *tkp;
        Int_t n = 0;
        while ((tkp = (U4BabyTrack *)next())) {
                std::cerr << std::setw(3) << ++n << " ";
                tkp->DebugPrint(opt);
        }
}
#endif
