//***************************************************************************
//* ----------------------
//* Cluster Datum
//* ----------------------
//***************************************************************************

#include "U4CDCCluster.h"
#include "U4CDCClusterBuf.h"
#include "U4CDCClusterMaker.h"
#include "U4CDCFADCCalibration.h"
#include <fstream>

using namespace std;

Int_t U4CDCCluster::fIntegrateNslots = 1;    //No of slots to integrate
Int_t U4CDCCluster::fNpres  = 100;
Int_t U4CDCCluster::fNposts = 100;
Int_t U4CDCCluster::fMinNslots = 999999;
Int_t U4CDCCluster::fMaxNslots = 999999;
Int_t U4CDCCluster::fSortBy =  0; // (0,1) = (time,charge)
Double_t U4CDCCluster::fSoftThresh  = 16.;
Double_t U4CDCCluster::fSplitThresh = 16.;

//---------------------------------------------------------------------------

ClassImp(U4CDCCluster)

U4CDCCluster::U4CDCCluster(U4CDCFADCCalibration *calib, Bool_t varped) : TObjArray(1),
        fDone(kFALSE), fUseVarPed(varped),
        fStationNo(0), fChannelNo(0),
        fStartTimeSlot(0), fLastTimeSlot(0),
        fSum(0.), fPeak(0.), fPedestal(0.),
        fStartTime(0.), fWidth(0.),
        fNpeds((fNpres - 20 > 0 ? fNpres - 20 : 10)),
        fCalib(calib), fParent(0) {}

U4CDCCluster::U4CDCCluster(U4CDCCluster & cl) : TObjArray(cl),
        fDone(cl.fDone), fUseVarPed(cl.fUseVarPed),
        fStationNo(cl.fStationNo), fChannelNo(cl.fChannelNo),
        fStartTimeSlot(cl.fStartTimeSlot), fLastTimeSlot(cl.fLastTimeSlot),
        fSum(cl.fSum), fPeak(cl.fPeak), fPedestal(cl.fPedestal),
        fStartTime(cl.fStartTime), fWidth(cl.fWidth),
        fNpeds(cl.fNpeds),
        fCalib(cl.fCalib), fParent(0) {}

U4CDCCluster::~U4CDCCluster() {}

void U4CDCCluster::SetNpeds(Int_t n)
{
        if (fNpeds != n) {
                fNpeds = n;
                fDone  = kFALSE;
        }
        if (!fDone) Update();
}

Double_t U4CDCCluster::GetPedestal()
{
        if (!fUseVarPed) {
                return fCalib->GetPedestal(fStationNo,fChannelNo);
        } else {
        if (!fDone) Update();
                return fPedestal;
        }
}

void U4CDCCluster::UseVarPed()
{
        if (!fUseVarPed) {
                fUseVarPed = kTRUE;
                fDone = kFALSE;
        }
        if (!fDone) Update();
}

void U4CDCCluster::UseConstPed()
{
        if (fUseVarPed) {
                fUseVarPed = kFALSE;
                fDone = kFALSE;
        }
        if (!fDone) Update();
}

Bool_t U4CDCCluster::Append(U4CDCRawADCDatum *adc)
{
        if (GetEntries() == 0) {
                fStationNo     = adc->GetStationNo();
                fChannelNo     = adc->GetChannelNo();
                fStartTimeSlot = adc->GetTime();
                fLastTimeSlot  = fStartTimeSlot;
                Add(adc);
                return kTRUE;
        } else {
                if (adc->GetStationNo() == fStationNo &&
                    adc->GetChannelNo() == fChannelNo &&
                    adc->GetTime() == fLastTimeSlot+1) {
                        fLastTimeSlot++;
                        Add(adc);
                        return kTRUE;
                } else {
                        return kFALSE;
                }
        }

}

void U4CDCCluster::Integrate()
{
        if (fIntegrateNslots <=1) return;
        Int_t nslots = GetEntries();
        for (Int_t islot = 0; islot < nslots; islot++) {
          Double_t adc     = 0;
          Int_t    divisor = 0;
          U4CDCRawADCDatum *adcp;
          for (Int_t i = 0; i < fIntegrateNslots; i++) {
            if (islot+i < nslots) {
              adcp = (U4CDCRawADCDatum *)At(islot+i);
              adc += adcp->GetADC();
              divisor++;
            }
          }
          adc  =  adc/divisor;
          adcp = (U4CDCRawADCDatum *)At(islot);
          adcp->SetADC((Int_t)adc);
        }
}

void U4CDCCluster::SplitInto(TObjArray &list)
{
        if (fParent) return;

        U4CDCCluster *cur = new U4CDCCluster(fCalib,fUseVarPed);
        cur->fParent = this;
#if 0
        Int_t nslots = GetEntries();
        Int_t islot;
        for (islot = fNpres; islot < nslots - fNposts; islot++) {
                cur->Append((U4CDCRawADCDatum *)At(islot));
        }

        if (cur->GetEntries() > 0) list.Add(cur);
        else                       delete cur;
#else
        Double_t ped  = fCalib->GetPedestal(fStationNo,fChannelNo);
        if (fUseVarPed) ped = fPedestal;

        Double_t vly  = 0.;
        Double_t pek  = -99999.;

        Bool_t   notyet  = kTRUE;       // peak not yet found
        Int_t    i1st    = fNpres;
        Int_t    ilst    = fNpres;
        Int_t    ipek    = fNpres;
        Int_t    in1st   = fNpres;

        Int_t nslots = GetEntries();
        Int_t islot;
        for (islot = fNpres; islot < nslots - fNposts; islot++) {
                U4CDCRawADCDatum *adclp = (U4CDCRawADCDatum *)At(islot-1);
                U4CDCRawADCDatum *adccp = (U4CDCRawADCDatum *)At(islot);
                U4CDCRawADCDatum *adcrp = (U4CDCRawADCDatum *)At(islot+1);

                Double_t adcl = ped - adclp->GetADC();
                Double_t adcc = ped - adccp->GetADC();
                Double_t adcr = ped - adcrp->GetADC();

                if (notyet && adcc < pek - fSplitThresh) { // peak found
                        notyet  = kFALSE;
                        vly     = pek;
                }

                if (!notyet && adcc > vly + fSplitThresh   // valley deep enough
			&& (ilst-i1st) > fMinNslots) {     // peak wide enough
                        Int_t i;                           // save the previous
                        for (i = i1st; i <= ilst; i++) {   // peak
                                cur->Append((U4CDCRawADCDatum *)At(i));
                        }
                        if (cur->GetEntries() > 0) {
                                list.Add(cur);
                                cur = new U4CDCCluster(fCalib,fUseVarPed);
                                cur->fParent = this;
                        }
                        notyet  = kTRUE;
                        i1st    = TMath::Max(in1st,ilst + 1);
                        if (ipek < i1st) {
                                pek  = vly;
                                ipek = ilst;
                        }
                }

                if (adcc > pek && adcc >= adcl && adcc >= adcr) { // peak candidate
                        pek  = adcc;
                        ipek = islot;
                }

                if (adcc < vly && adcc <= adcl && adcc <= adcr) { // valley candidate
                        if (notyet) {
                                vly  = adcc;
                                i1st = islot;   // 1st slot candidate
                        } else {
                                if ((islot-i1st) > fMinNslots) {
                                        vly = adcc;
                                        ilst = islot;   // last slot chandidate
                                }
                        }
                }

                if (adcc < fSoftThresh && adcc <= adcl && adcc <= adcr) {
                        if (notyet)   i1st  = islot;
                        else          in1st = islot;
                }

                if (!notyet && islot == (nslots - fNposts -1)) ilst = islot;
        }
        if (ilst > i1st) {// save the left over
                Int_t i;
                for (i = i1st; i <= ilst; i++) {
                        cur->Append((U4CDCRawADCDatum *)At(i));
                }
                list.Add(cur);
        } else {
                if (cur) delete cur;
        }
#endif
}

void U4CDCCluster::Update()
{
        if (fDone) return;
        fDone = kTRUE;
        Double_t ped    = fCalib->GetPedestal(fStationNo,fChannelNo);
        Double_t gain   = fCalib->GetGain(fStationNo,fChannelNo);
        Double_t tped   = fCalib->GetTimePedestal(fStationNo,fChannelNo);
        Double_t tscale = fCalib->GetTimeScale(fStationNo,fChannelNo);
#if 0 
        std::cerr << " n,a = " << fStationNo << "," << fChannelNo << std::endl
		  << " ped = " << ped 
		  << " gain = " << gain
		  << " tped = " << tped
		  << " tscale = " << tscale << std::endl;
#endif

        if (!fParent) {
                TIter next(this);
                Int_t nslots = GetEntries();
                if (nslots <= fNpres+fNposts) {
                        fPedestal  = ped;
                        fPeak      = 0.;
                        fSum       = 0.;
                        fStartTime = (fStartTimeSlot - tped)*tscale;
                        fWidth     = (nslots - fNpres - fNposts)*tscale;
                        std::cerr << "???? U4CDCCluster::Update() ???? "
                             << "cluster too narrow: "
                             << "Nslots = " << nslots << std::endl;
                        DebugPrint("Detailed");
                        return;
                }

                fPedestal = 0;
                U4CDCRawADCDatum *adcp;
                Int_t islot;
                for (islot = 0; islot < fNpres; islot++) {
                        adcp = (U4CDCRawADCDatum *)next();
                        if (islot < fNpeds) fPedestal += adcp->GetADC();
                }
                fPedestal /= fNpeds;

                if (fUseVarPed) ped = fPedestal;

                fSum       = 0;
                fPeak      = -99999.;
                fStartTime = fStartTimeSlot;
                for (; islot < nslots - fNposts; islot++) {
                        adcp = (U4CDCRawADCDatum *)next();
                        Double_t adc = ped - adcp->GetADC();
                        fPeak  = TMath::Max(fPeak,adc);
                        if (fStartTime == fStartTimeSlot && adc > fSoftThresh) {                                fStartTime += islot - fNpres;
                        }
                        fSum  += adc;
                }
                fSum *= gain;

                fStartTime = (fStartTime - tped)*tscale;
                fWidth     = (nslots - fNpres - fNposts)*tscale;
        } else {
                if (!fParent->fDone) fParent->Update();
                fPedestal = fParent->fPedestal;
                if (fUseVarPed) ped = fPedestal;

                TIter next(this);
                Int_t nslots = TMath::Min(GetEntries(), fMaxNslots);

                fSum       = 0;
                fPeak      = -99999.;
                fStartTime = fStartTimeSlot;
                Int_t islot;
                U4CDCRawADCDatum *adcp;
                for (islot = 0; islot < nslots; islot++) {
                        adcp = (U4CDCRawADCDatum *)next();
                        Double_t adc = ped - adcp->GetADC();
                        fPeak  = TMath::Max(fPeak,adc);
                        if (fStartTime == fStartTimeSlot && adc > fSoftThresh) {                                fStartTime += islot - fNpres;
                        }
                        fSum  += adc;
                }
                fSum *= gain;

                fStartTime = (fStartTime - tped)*tscale;
                fWidth     = nslots*tscale;
        }
}

Bool_t U4CDCCluster::IsSortable() const { return kTRUE; }

Int_t U4CDCCluster::Compare(const TObject *obj) const
{
        if (!fDone) ((U4CDCCluster *)this)->Update();
        if (!((U4CDCCluster *)obj)->fDone) ((U4CDCCluster *)obj)->Update();
        switch (fSortBy) {
            case 1: // sort by charge:  bigger one first:-)
                if (fSum > ((U4CDCCluster *)obj)->fSum) return -1;
                else if (fSum < ((U4CDCCluster *)obj)->fSum) return 1;
                else return 0;
            case 2: // sort by pulse height:  bigger one first:-)
                if (fPeak > ((U4CDCCluster *)obj)->fPeak) return -1;
                else if (fPeak < ((U4CDCCluster *)obj)->fPeak) return 1;
                else return 0;
            default: // sort by time: faster one first:-)
                if (fStartTime < ((U4CDCCluster *)obj)->fStartTime) return -1;
                else if (fStartTime > ((U4CDCCluster *)obj)->fStartTime) return 1;                else return 0;
        }
}

void U4CDCCluster::SetSortBy(Char_t *opt)
{
        switch (opt[0]) {
                case 'C':       // sort by "Charge"
                        fSortBy = 1;
                        break;
                case 'P':       // sort by "PulseHeight"
                        fSortBy = 2;
                        break;
                default:        // sort by "Time"
                        fSortBy = 0;
        }
}

void U4CDCCluster::DebugPrint(const Char_t *opt)
{
        std::cerr.setf(ios::right);
        std::cerr.setf(ios::fixed);
        std::cerr    << " (n,a)=(" << setw(2) << GetStationNo() << ","
                             << setw(1) << GetChannelNo() << ")"
                << " parent = " << setw(5) << (void *)fParent
                << " t = " << setw(6) << setprecision(1) << GetStartTime()
                << " w = " << setw(6) << setprecision(1) << GetWidth()
                << " Sum = " << setw(7) << setprecision(1) << GetSum()
                << " Pek = " << setw(5) << setprecision(1) << GetPeak()
                << " Ped = " << setw(5) << setprecision(1) << GetPedestal()
                << std::endl;
        std::cerr.unsetf(ios::right);
        std::cerr.unsetf(ios::fixed);

        if (strstr(opt,"Detailed")) {
                TIter next(this);
                U4CDCRawADCDatum *adc;
                while ((adc = (U4CDCRawADCDatum *)next())) {
                        adc->DebugPrint();
                }
        }
}
