#ifndef __U4CDCTRACK__
#define __U4CDCTRACK__

//*************************************************************************
//* ---------------------
//* U4CDCTrack Class
//* ---------------------
//* (Description)
//*	Class to store "U4CDCHitCell"s for a single layer.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4CDCTrack		: Array containing "U4CDCHitCell"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima     Original version.
//*	2003/0X/XX  Y.Nakashima     Implemented Kalman filter.
//*	2004/08/08  K.Fujii         Removed baby chamber code.
//*	2004/09/05  K.Fujii         Now inherits from U4VTrack.
//*************************************************************************

#include "U4VTrack.h"
#include "U4CDCVectorHit.h"
#include "U4CDCTrackHit.h"
#include "U4CDCTrackMaker.h"

#include "JSFHelicalTrack.h"

#include "TKalTrack.h"
#include "TKalTrackSite.h"

class U4CDCVectorHit;

class U4CDCTrack : public U4VTrack {
public:
  // ----------------------------------------------------------------------
  // C-tors and D-tor 
  // ----------------
  U4CDCTrack(U4CDCTrackMaker *trkm = 0);
  U4CDCTrack(const U4CDCTrack &tk);
  virtual ~U4CDCTrack();

  // ----------------------------------------------------------------------
  // Getters and setters
  // -------------------
         virtual Int_t     GetNhits      ();

  // ----------------------------------------------------------------------
  // Service methods
  // ---------------
         virtual void           Add(TObject *obj);

         virtual void           DebugPrint(const Char_t *opt = "Brief");
         virtual void           MakeKalHits(TObjArray &kalhits);
         virtual TKalTrackSite *GetFirstSitePtr(Bool_t     dir,
                                                TObjArray *hitsp = 0);
  
private:
  virtual void    KalmanFilter(Bool_t dir = kIterBackward);
  
private:

  ClassDef(U4CDCTrack, 1)		   // Baby Track
};

//-----------------------------------------------------------
//  inline function if any
//-----------------------------------------------------------

#endif
