//*************************************************************************
//* ---------------------
//* U4IPKalHit Class
//* ---------------------
//* (Description)
//* (Requires)
//*     TObjArray
//* (Provides)
//* (Update Record)
//*     2004/08/26  A.Yamaguchi    Original Ver.
//*************************************************************************

#include <iostream>
#include <iomanip>

#include "U4IPKalHit.h"
#include "TRandom.h"

Double_t    U4IPKalHit::fgDX = 3.e-5;   // [cm]
Double_t    U4IPKalHit::fgDZ = 1.e-2;   // [cm]
Double_t    U4IPKalHit::fgBfield = 3.;  // [kG]

ClassImp(U4IPKalHit)

//=====================================================================
//* constructor -------------------------------------------------------

U4IPKalHit::U4IPKalHit(Int_t m)
          : TVTrackHit(m) 
{
}
      
U4IPKalHit::U4IPKalHit(const U4IPMeasLayer &ms,
                             Double_t      *x,
                             Double_t      *dx, 
                             Double_t       b,
                             Int_t          m)
          : TVTrackHit(ms, x, dx, b, m)
{
}

U4IPKalHit::U4IPKalHit(const U4IPKalHit &hit)
          : TVTrackHit(hit)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

U4IPKalHit::~U4IPKalHit()
{
}

//=====================================================================
//* Create ------------------------------------------------------------

U4IPKalHit *U4IPKalHit::Create()
{
   Double_t A       = 14.00674 * 0.7 + 15.9994 * 0.3;
   Double_t Z       = 7.3;
   Double_t density = 1.205e-3;
   Double_t radlen  = 3.42e4;
   TMaterial &air = *new TMaterial("IPAir", "", A, Z, density, radlen, 0.);
   TVector3 xc(0., 0., 0.);
   TVector3 n (0., 1., 0.);
   U4IPMeasLayer &ml = *new U4IPMeasLayer(air, air, xc, n);

   Double_t meas [2]; 
   Double_t dmeas[2]; 
   meas [0] = gRandom->Gaus(0., fgDX);
   meas [1] = gRandom->Gaus(0., fgDZ);
   dmeas[0] = fgDX;
   dmeas[1] = fgDZ;
   return new U4IPKalHit(ml, meas, dmeas, fgBfield);
}

//=====================================================================
//* XvToMv ------------------------------------------------------------

TKalMatrix U4IPKalHit::XvToMv(const TVector3 &xv, Double_t /*t0*/) const
{
   return dynamic_cast<const U4IPMeasLayer &>(GetMeasLayer()).XvToMv(xv);
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void U4IPKalHit::DebugPrint(Option_t *) const
{
   using namespace std;
   cerr << "------------------- Site Info -------------------------" << endl;

   for (Int_t i=0; i<GetDimension(); i++) {
      Double_t x  = (*this)(i,0);
      Double_t dx = (*this)(i,1);
      cerr << " x[" << i << "] = " << setw(8) << setprecision(5) << x
           << "    "
           << "dx[" << i << "] = " << setw(6) << setprecision(2) << dx
           << setprecision(7)
           << resetiosflags(ios::showpoint)
           << endl;
   }
   cerr << "-------------------------------------------------------" << endl;
}
