//*************************************************************************
//* =================
//* U4VTrackHit Class
//* =================
//* (Description)
//*    Class for a single track hit.
//* (Requires)
//*    TObjArray
//* (Provides)
//*    U4VTrackHit :
//* (Update Record)
//*     2004/09/05  K.Fujii    Original version.
//*************************************************************************

#include "TMath.h"

#include "U4VTrackHit.h"

ClassImp(U4VTrackHit)

Int_t U4VTrackHit::fSortBy = 0;

//=====================================================================
//* constructor -------------------------------------------------------

U4VTrackHit::U4VTrackHit(TObject *htp)
            : fResidual(0.), 
              fT0(0.),
              fHitPtr(htp)
{
#if 0
		std::cerr << " U4VTrackHit::constructor "
			  << " fHitPtr = " << (void *)fHitPtr
			  << " TrackHit = " << (void *)this << std::endl;
#endif
}
	
U4VTrackHit::U4VTrackHit(const U4VTrackHit &hit)
            : fResidual(hit.fResidual),
              fT0(hit.fT0),
              fHitPtr(hit.fHitPtr)
{
   if (hit.IsLocked()) Lock();
   else                Unlock();
}

//=====================================================================
//* destructor --------------------------------------------------------

U4VTrackHit::~U4VTrackHit()
{
}

//=====================================================================
//* SetSortBy --------------------------------------------------------
                                                                                
void U4VTrackHit::SetSortBy(Char_t *opt)
{
   switch (opt[1]) {
      case '0':       // sort by "T0"
         fSortBy = 1;
         break;
      case 'h':       // sort by "Chi2"
      default:        // sort by "Chi2"
         fSortBy = 0;
         break;
   }
}

void U4VTrackHit::SetSortBy(Int_t key)
{
   fSortBy = key;
}
                                                                                
//=====================================================================
//* IsSoatable --------------------------------------------------------
                                                                                
Bool_t U4VTrackHit::IsSortable() const
{
   return kTRUE;
}
                                                                                
//=====================================================================
//* Compare -----------------------------------------------------------

Int_t U4VTrackHit::Compare(const TObject *obj) const
{
   switch (fSortBy) {
      case 1: { // Sort by T0
         Double_t t0o = ((U4VTrackHit *)obj)->GetT0();
         return (fT0 < t0o ? -1 : (fT0 > t0o ? +1 : 0));
      }
      case 0: // Sort by Chi2
      default: {
         Double_t res  = TMath::Abs(fResidual);
         Double_t reso = TMath::Abs(((U4VTrackHit *)obj)->GetResidual());
         return (res < reso ? -1 : (res > reso ? +1 : 0));
      }
   }
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void U4VTrackHit::DebugPrint(const Char_t *)
{
}
