#ifndef __U4VTXHIT__
#define __U4VTXHIT__
//*************************************************************************
//* ---------------------
//* U4VTXHit Class
//* ---------------------
//* (Description)
//*	Class for a single VTX hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4VTXHit		: TObject containing single hit data
//* (Update Record)
//*	2004/12/15  T.Fujikawa	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "TAttLockable.h"

class U4VTXHit : public TObject, public TAttLockable{

public:
  U4VTXHit(Int_t    layerID  = -9999,
	   Double_t layerR   = -9999.,
	   Double_t phi      = -9999.,
	   Double_t z        = -9999.);

  virtual ~U4VTXHit();

  inline virtual Int_t     GetLayerID() const { return fLayerID; }
  inline virtual Double_t  GetLayerR() const { return fLayerR; }
  inline virtual Double_t  GetPhi         () const { return fPhi;          }

  inline virtual Bool_t    IsDone         () const { return fDone;         }


  inline virtual Double_t  GetRPhi() { if (!fDone) Update(); return fRPhi;}
  inline virtual Double_t  GetDRPhi(){ if (!fDone) Update(); return fDRPhi;}
  inline virtual Double_t  GetZ()  { if (!fDone) Update(); return fZ;  }
  inline virtual Double_t  GetDZ() { if (!fDone) Update(); return fDZ; }
  
  inline virtual void      SetRPhi   (Double_t d)  { fRPhi    = d; }
  inline virtual void      SetDRPhi  (Double_t d)  { fDRPhi   = d; }
  inline virtual void      SetZ (Double_t d)  { fZ  = d; }
  inline virtual void      SetDZ(Double_t d)  { fDZ = d; }

  inline virtual void      SetDone   (Bool_t   b)  { fDone    = b; }

  virtual Bool_t	IsSortable()                const;
  virtual Int_t		Compare(const TObject *obj) const;
  static void           SetSortBy(Char_t *opt);
  virtual void 		DebugPrint(const Char_t *opt = "Brief");

 protected:
  virtual void	Update();
 private:
  virtual void	CalculatePosition();

 private:
  Bool_t	fDone;          // (1,0) = (cluster done, not yet)
  Double_t	fRPhi;          // r phi position(cm)
  Double_t	fDRPhi;         // error on r phi position(cm)
  Double_t	fZ;             // z position (cm)
  Double_t	fDZ;            // error on z position (cm)

  Int_t         fLayerID;       // layer number
  Double_t      fLayerR;        // layer radius
  Double_t      fPhi;           // phi

  static Int_t fgVTXHitSortBy;

  ClassDef(U4VTXHit, 1)            // VTX Hit
};

#endif
